/// \file access.h
/// \author Mauro Brunato
///
/// \brief Objects for shared/exclusive access
///
/// $Id: access.h,v 1.2 2005/06/16 13:22:22 cvs Exp $

/*##############################################################################
   
   Developed by NetMob Group, University of Trento, Italy

   University of Trento
   DIT - Department of Informatics and Telecommunications 
   Computer Networks and Mobility Research Program
   http://netmob.unitn.it/ 
   
   This file is part of the WilmaGate system, created as part
   of the WILMA Project, sponsored by the Autonomous
   Province of Trento and participated by ITC-irst, the University
   of Trento and Alpikom Spa. More info at

     http://www.wilmaproject.org/		

   To download the project:
      - in tarball format
        http://netmob.unitn.it/wilmagate.html
      - connect to CVS repository
        :pserver:cvs@rtm.unitn.it:/home/cvsroot
   In both previous cases, the download is password protected. To obtain
   the password you must send a mail to 

      wilmagate@dit.unitn.it

################################################################################

   The WilmaGate Licence of Use


   Copyright (c) 2003-2005
                - Universita` di Trento,
                  Dipartimento di Informatica e Telecomunicazioni (DIT)
                  Computer Networks and Mobility Research Program


   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (hereinafter
   the "WilmaGate") directly from the copyright owner, to use the WilmaGate
   without restriction of purpose or time, including without limitation the
   rights to use, copy, modify, and merge the WilmaGate, subject to the
   following conditions:

   The above copyright notice and this permission notice shall be included
   in all copies or substantial portions of the WilmaGate. 

   The WilmaGate shall not be sub-licenced either commercially or free of
   charge, as is or in part or as a part of any other package. 

   The licensee agrees to properly credit WilmaGate and its copyright 
   owners in any technical, scientific or divulgative publication, 
   whatever publishing means are used, that is based entirely or in part 
   on WilmaGate use, as well as in any booklet, web-site or brochure that 
   advertise services based on WilmaGate use. 

   If the WilmaGate is modified, the copyright owner shall be notified.

   THE WilmaGate IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
   OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
   IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
   CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
   TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
   SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 

##############################################################################*/

#ifndef _H_ACCESS_
#define _H_ACCESS_

#include <pthread.h>

#include "exceptions.h"

// Commented in addresses.h
namespace Common {

	/////////////////////////////////////////////////////////////////////
	///
	/// \brief Exception thrown by methods in class Common::AccessRights
	///
	class AccessException:
		public WilmaGateException {
	public:
		///
		/// \brief Constructor.
		///
		/// \param code Code specifying the error type
		/// (usually the anomalous semaphore count).
		///
		inline AccessException (int code);
	};

	inline AccessException::AccessException (int code):
		WilmaGateException ("Unexpected access value", code)
	{
	}

	//////////////////////////////////////////////////////////////////////
	///
	/// \brief Synchronization class.
	///
	/// Classes that need to coordinate shared and exclusive access
	/// (for example, shared read and exclusive write) to some member
	/// must include one or more objects of this class and call the appropriate
	/// methods. Of course, the member must be private and only managed by
	/// proper class methods.
	///
	/// This class may throw Common::AccessException objects
	/// for debug if some synchronization error occurs.
	///
	/// \remark
	/// 	- Programmers must take obsessive care to correctly balance
	///	  calls to this class, otherwise the program might deadlock.
	///	- Probably this class can also be subclassed, but I did not
	///	  test it.
	///
	class AccessRights {
	private:
		///
		/// \brief Current access count.
		///
		/// Counts the ongoing accesses to the critical section.
		/// If an exclusive access is being performed, then its value is -1.
		///
		int access;
		///
		/// \brief POSIX mutual exclusion object.
		///
		pthread_mutex_t mutex;
		///
		/// \brief POSIX condition synchronization object.
		///
		pthread_cond_t cond;
	public:
		///
		/// \brief Constructor. Initializes the counter and the POSIX synchronization objects.
		///
		AccessRights (void);
		///
		/// \brief Destructor. Releases the POSIX synchronization objects.
		///
		~AccessRights (void);
		///
		/// \brief Begin a shared access section of the code.
		///
		void beginShared (void) throw (AccessException);
		///
		/// \brief End a shared access section of the code.
		///
		void endShared (void) throw (AccessException);
		///
		/// \brief Begin an exclusive access section of the code.
		///
		void beginExclusive (void) throw (AccessException);
		///
		/// \brief End an exclusive access section of the code.
		///
		void endExclusive (void) throw (AccessException);
	};

} // namespace Common


#endif
