/// \file addresses.h
/// \author Mauro Brunato
///
/// \brief Address structures for common address types.
///
/// $Id: addresses.h,v 1.3 2005/06/16 13:22:22 cvs Exp $
///
/// Defines addresses at layer 2 and 3 (Ethernet and IPv4).
///

/*##############################################################################
   
   Developed by NetMob Group, University of Trento, Italy

   University of Trento
   DIT - Department of Informatics and Telecommunications 
   Computer Networks and Mobility Research Program
   http://netmob.unitn.it/ 
   
   This file is part of the WilmaGate system, created as part
   of the WILMA Project, sponsored by the Autonomous
   Province of Trento and participated by ITC-irst, the University
   of Trento and Alpikom Spa. More info at

     http://www.wilmaproject.org/		

   To download the project:
      - in tarball format
        http://netmob.unitn.it/wilmagate.html
      - connect to CVS repository
        :pserver:cvs@rtm.unitn.it:/home/cvsroot
   In both previous cases, the download is password protected. To obtain
   the password you must send a mail to 

      wilmagate@dit.unitn.it

################################################################################

   The WilmaGate Licence of Use


   Copyright (c) 2003-2005
                - Universita` di Trento,
                  Dipartimento di Informatica e Telecomunicazioni (DIT)
                  Computer Networks and Mobility Research Program


   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (hereinafter
   the "WilmaGate") directly from the copyright owner, to use the WilmaGate
   without restriction of purpose or time, including without limitation the
   rights to use, copy, modify, and merge the WilmaGate, subject to the
   following conditions:

   The above copyright notice and this permission notice shall be included
   in all copies or substantial portions of the WilmaGate. 

   The WilmaGate shall not be sub-licenced either commercially or free of
   charge, as is or in part or as a part of any other package. 

   The licensee agrees to properly credit WilmaGate and its copyright 
   owners in any technical, scientific or divulgative publication, 
   whatever publishing means are used, that is based entirely or in part 
   on WilmaGate use, as well as in any booklet, web-site or brochure that 
   advertise services based on WilmaGate use. 

   If the WilmaGate is modified, the copyright owner shall be notified.

   THE WilmaGate IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
   OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
   IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
   CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
   TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
   SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 

##############################################################################*/

#ifndef _H_ADDRESSES_
#define _H_ADDRESSES_

using namespace std;

extern "C" {
#include <arpa/inet.h>
#include <inttypes.h>
}

#include <cstdlib>
#include <iostream>

///
/// \brief Entities shared between the Gateway and the Gatekeeper
///
/// Entities defined within this namespace may be used by both projects.
///
namespace Common {

	//////////////////////////////////////////////////////////////////////////////
	///
	/// \brief Wrapper class for the POSIX IPv4 address type.
	///
	/// The IPv4 address is already defined in arpa/inet.h, but we add some utility
	/// functions in order to mask endianness issues.
	///
	class IPv4Address {
	private:
		///
		/// \brief Different representations of an IP address.
		///
		union AddressRepresentations {
			in_addr a;	///< Address represented as a 32-bit big-endian word.
			uint8_t c[4];	///< Address represented as a 4-byte array, host part last.
		} addr;
	public:
		///
		/// \brief Empty constructor.
		///
		/// Set the address to 0.0.0.0.
		///
		inline IPv4Address (void);
		///
		/// \brief Copy constructor.
		///
		/// \param a The address to be copied.
		///
		inline IPv4Address (const IPv4Address &a);
		///
		/// \brief String constructor.
		///
		/// \param s The string representation of an address.
		///
		inline IPv4Address (const string &s);
		///
		/// \brief C string constructor.
		///
		/// \param s The C string representation of an address.
		///
		inline IPv4Address (const char *s);
		///
		/// \brief in_addr constructor.
		///
		/// \param a The in_addr representation of an address.
		///
		inline IPv4Address (in_addr a);
		///
		/// \brief Long unsigned int constructor.
		///
		/// \param a The long uint representation of an address.
		///
		inline IPv4Address (uint32_t a);
		///
		/// \brief Return the address as an unsigned int.
		///
		inline uint32_t asInteger (void) const;
		///
		/// \brief Return the address as an in_addr struct
		///
		inline in_addr asInaddr (void) const;
		///
		/// \brief Return the host part of the address.
		///
		inline int host (void) const;
		///
		/// \brief Return the address in host form (with correct endianness).
		///
		inline uint32_t toHost (void) const;
		///
		/// \brief Although not elegant, an IP address must be incremented
		/// from time to time...
		///
		inline IPv4Address operator++ (void);
		///
		/// \brief Increment the address.
		///
		/// \param n The integer to be added (right parameter).
		///
		inline IPv4Address operator+ (int n) const;
		///
		/// \brief Compare two IP addresses.
		///
		/// \param a The address to be compared to (right parameter).
		///
		inline bool operator== (IPv4Address a) const;
		///
		/// \brief Compare two IP addresses.
		///
		/// \param a The address to be compared to (right parameter).
		///
		inline bool operator!= (IPv4Address a) const;
		///
		/// \brief Assign an integer to an IP address.
		///
		/// \param ip The IP address in network (big-endian) format (right parameter).
		///
		inline IPv4Address operator= (uint32_t ip);
		///
		/// \brief Mask the address with the given subnet mask.
		///
		/// \param mask The subnet mask, given as an IPv4 address.
		///
		/// \return The masked address.
		///
		inline IPv4Address operator& (IPv4Address mask) const;
		///
		/// \brief Write address to a binary stream
		///
		/// \param s The output stream.
		///
		inline ostream &toBinaryStream (ostream &s) const;
		///
		/// \brief Read address from a binary stream
		///
		/// \param s The input stream.
		///
		inline istream &fromBinaryStream (istream &s);
		///
		/// \brief Output an IP address in friendly form.
		///
		/// \param o The output stream (left parameter).
		/// \param a The Ip address (right parameter).
		///
		friend inline ostream &operator<< (ostream &o, IPv4Address a);
		///
		/// \brief Input an IP address.
		///
		/// \param i The input stream (left parameter).
		/// \param a The Ip address (right parameter).
		///
		friend istream &operator>> (istream &i, IPv4Address &a);
	}
	#ifndef DOXYSCAN
		__attribute__((packed))
	#endif
		;

	inline IPv4Address::IPv4Address (void)
	{
		addr.a.s_addr = 0UL;
	}

	inline IPv4Address::IPv4Address (const IPv4Address &ip)
	{
		addr.a.s_addr = ip.addr.a.s_addr;
	}

	inline IPv4Address::IPv4Address (const string &s)
	{
		inet_aton (s.c_str(), &addr.a);
	}

	inline IPv4Address::IPv4Address (const char *s)
	{
		inet_aton (s, &addr.a);
	}

	inline IPv4Address::IPv4Address (in_addr a)
	{
		addr.a = a;
	}

	inline IPv4Address::IPv4Address (uint32_t a)
	{
		addr.a.s_addr = a;
	}

	inline uint32_t IPv4Address::asInteger (void) const
	{
		return addr.a.s_addr;
	}

	inline in_addr IPv4Address::asInaddr (void) const
	{
		return addr.a;
	}

	inline int IPv4Address::host (void) const
	{
		return (addr.c[3]);
	}

	inline uint32_t IPv4Address::toHost (void) const
	{
		return ntohl (addr.a.s_addr);
	}

	inline IPv4Address IPv4Address::operator++ (void)
	{
		addr.a.s_addr = htonl (ntohl (addr.a.s_addr) + 1);
		return *this;
	}

	inline IPv4Address IPv4Address::operator+ (int n) const
	{
		IPv4Address a;
		a = htonl (ntohl (addr.a.s_addr) + n);
		return a;
	}

	inline bool IPv4Address::operator== (IPv4Address a) const
	{
		return addr.a.s_addr == a.addr.a.s_addr;
	}

	inline bool IPv4Address::operator!= (IPv4Address a) const
	{
		return addr.a.s_addr != a.addr.a.s_addr;
	}

	inline IPv4Address IPv4Address::operator= (uint32_t ip)
	{
		addr.a.s_addr = ip;
		return *this;
	}

	inline IPv4Address IPv4Address::operator& (IPv4Address mask) const
	{
		IPv4Address ip;
		ip.addr.a.s_addr = addr.a.s_addr & mask.addr.a.s_addr;
		return ip;
	}

	inline ostream &IPv4Address::toBinaryStream (ostream &s) const
	{
		s.write (reinterpret_cast<const char *>(&addr), sizeof(addr));
		return s;
	}

	inline istream &IPv4Address::fromBinaryStream (istream &s)
	{
		s.read (reinterpret_cast<char *>(&addr), sizeof(addr));
		return s;
	}

	inline ostream &operator<< (ostream &o, IPv4Address a)
	{
		return o << inet_ntoa (a.addr.a);
	}

	//////////////////////////////////////////////////////////////////////////////
	///
	/// \brief Class for the Ethernet MAC address
	///
	/// Some utility functions are added to the basic structure (a 6-bytes string).
	///
	/// \remarks No constructor allowed, because some classes that define EthernetMACAddress
	/// objects appear in unions.
	///
	class EthernetMACAddress {
	public:
		///
		/// \brief Length of the string to represent a MAC address.
		///
		static const int STRINGLENGTH = 18;
	private:
		///
		/// \brief A MAC address is composed of 6 bytes.
		///
		uint8_t addr[6];
		///
		/// \brief Utility function to obtain a MAC address from a string.
		///
		/// \param s The string, containing 6 2-digit parts separated by colons or hyphens
		///
		void fromString (const char *s);		
	public:
		/* merged by Flavio -- moved this method from private to public */
		///
		/// \brief Utility function to obtain a string from a MAC address.
		///
		/// The string will be composed of 6 2-digit parts separated by colons or hyphens.
		///
		/// \param s A pre-allocated character array.
		///
		/// \pre The pre-allocated array must be able to contain at least STRINGLENGTH characters.
		/// \return The pointer to s.
		///
		char *toString (char *s) const;
		/* ----------------- end --------------------*/
		///
		/// \brief Void constructor.
		///
		inline EthernetMACAddress (void);
		///
		/// \brief Copy constructor.
		///
		/// \param e The address to be copied.
		///
		inline EthernetMACAddress (const EthernetMACAddress &e);
		///
		/// \brief String constructor.
		///
		/// \param s The string containing a user-friendly representation of the address
		/// (see EthernetMACAddress::fromString for details).
		///
		inline EthernetMACAddress (const char *s);
		///
		/// \brief Compare two MAC addresses.
		///
		/// \param a The address to be compared.
		///
		inline bool operator== (const EthernetMACAddress &a) const;
		///
		/// \brief Compare two MAC addresses
		///
		/// \param a The address to be compared.
		///
		inline bool operator!= (const EthernetMACAddress &a) const;
		///
		/// \brief Set MAC address from a byte array
		///
		/// \param a The array containing the MAC address
		///
		inline EthernetMACAddress &operator= (const uint8_t *a);
		///
		/// \brief Return one byte of the address.
		///
		/// \param i Zero-based index of the byte.
		///
		inline uint8_t &operator[] (int i);
		///
		/// \brief Write address to a binary stream
		///
		/// \param s The output stream.
		///
		inline ostream &toBinaryStream (ostream &s) const;
		///
		/// \brief Read address from a binary stream
		///
		/// \param s The input stream.
		///
		inline istream &fromBinaryStream (istream &s);
		///
		/// \brief Output the address in friendly form.
		///
		/// \param o The output stream.
		/// \param a The MAC address.
		///
		/// \return The output stream.
		///
		friend inline ostream &operator<< (ostream &o, const EthernetMACAddress &a);
		///
		/// \brief Input a MAC address.
		///
		/// \param i The input stream (left parameter).
		/// \param a The MAC address (right parameter).
		///
		friend istream &operator>> (istream &i, EthernetMACAddress &a);
	}
	#ifndef DOXYSCAN
		__attribute__((packed))
	#endif
		;

	inline EthernetMACAddress::EthernetMACAddress (void)
	{
	}

	inline EthernetMACAddress::EthernetMACAddress (const EthernetMACAddress &e)
	{
		memcpy (addr, e.addr, 6);
	}

	inline EthernetMACAddress::EthernetMACAddress (const char *s)
	{
		fromString (s);
	}

	inline bool EthernetMACAddress::operator== (const EthernetMACAddress &a) const
	{
		return ! memcmp (addr, a.addr, 6);
	}

	inline bool EthernetMACAddress::operator!= (const EthernetMACAddress &a) const
	{
		return memcmp (addr, a.addr, 6);
	}

	inline EthernetMACAddress &EthernetMACAddress::operator= (const uint8_t *a)
	{
		memcpy (addr, a, 6);
		return *this;
	}

	inline ostream &operator<< (ostream &o, const EthernetMACAddress &a)
	{
		char s[EthernetMACAddress::STRINGLENGTH];
		return o << a.toString (s);
	}

	inline uint8_t &EthernetMACAddress::operator[] (int i)
	{
		return addr[i];
	}

	inline ostream &EthernetMACAddress::toBinaryStream (ostream &s) const
	{
		s.write (reinterpret_cast<const char *>(addr), 6);
		return s;
	}

	inline istream &EthernetMACAddress::fromBinaryStream (istream &s)
	{
		s.read (reinterpret_cast<char *>(addr), 6);
		return s;
	}

} // namespace Common

#endif
