/// \file exceptions.h
/// \author Mauro Brunato
///
/// \brief Exception objects.
///
/// $Id: exceptions.h,v 1.2 2005/06/16 13:22:22 cvs Exp $

/*##############################################################################
   
   Developed by NetMob Group, University of Trento, Italy

   University of Trento
   DIT - Department of Informatics and Telecommunications 
   Computer Networks and Mobility Research Program
   http://netmob.unitn.it/ 
   
   This file is part of the WilmaGate system, created as part
   of the WILMA Project, sponsored by the Autonomous
   Province of Trento and participated by ITC-irst, the University
   of Trento and Alpikom Spa. More info at

     http://www.wilmaproject.org/		

   To download the project:
      - in tarball format
        http://netmob.unitn.it/wilmagate.html
      - connect to CVS repository
        :pserver:cvs@rtm.unitn.it:/home/cvsroot
   In both previous cases, the download is password protected. To obtain
   the password you must send a mail to 

      wilmagate@dit.unitn.it

################################################################################

   The WilmaGate Licence of Use


   Copyright (c) 2003-2005
                - Universita` di Trento,
                  Dipartimento di Informatica e Telecomunicazioni (DIT)
                  Computer Networks and Mobility Research Program


   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (hereinafter
   the "WilmaGate") directly from the copyright owner, to use the WilmaGate
   without restriction of purpose or time, including without limitation the
   rights to use, copy, modify, and merge the WilmaGate, subject to the
   following conditions:

   The above copyright notice and this permission notice shall be included
   in all copies or substantial portions of the WilmaGate. 

   The WilmaGate shall not be sub-licenced either commercially or free of
   charge, as is or in part or as a part of any other package. 

   The licensee agrees to properly credit WilmaGate and its copyright 
   owners in any technical, scientific or divulgative publication, 
   whatever publishing means are used, that is based entirely or in part 
   on WilmaGate use, as well as in any booklet, web-site or brochure that 
   advertise services based on WilmaGate use. 

   If the WilmaGate is modified, the copyright owner shall be notified.

   THE WilmaGate IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
   OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
   IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
   CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
   TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
   SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 

##############################################################################*/

#ifndef _H_COMMON_EXCEPTIONS_
#define _H_COMMON_EXCEPTIONS_

#include <fstream>
#include <cstring>

namespace Common {

	///////////////////////////////////////////////////////////////////////////
	///
	/// \brief The base class for all exceptions.
	///
	class WilmaGateException {
		///
		/// \brief Output user-friendly description of exception
		///
		friend inline std::ostream &operator<< (std::ostream &o, const WilmaGateException &e);
	private:
		///
		/// \brief Small textual description of the exception causes.
		///
		const char * reason;
		///
		/// \brief Numerical code, just in case it were useful.
		///
		const int code;
	public:
		///
		/// \brief Empty constructor: should never happen and generates an unknown exception.
		///
		inline WilmaGateException (void);
		///
		/// \brief Constructor with explicit values for reason and code.
		///
		/// \param s Reason.
		/// \param n Code.
		///
		inline WilmaGateException (const char *s, int n);
		///
		/// \brief Copy constructor.
		///
		/// \param e Exception to be copied.
		///
		inline WilmaGateException (const WilmaGateException &e);
		///
		/// \brief Destructor: frees the description string.
		///
		inline ~WilmaGateException (void);
	};

	inline WilmaGateException::WilmaGateException (void):
		reason (strdup("Unknown exception")),
		code (0)
	{
	}

	inline WilmaGateException::WilmaGateException (const char *s, int n):
		reason (strdup(s)),
		code (n)
	{
	}

	inline WilmaGateException::WilmaGateException (const WilmaGateException &e):
		reason (strdup(e.reason)),
		code (e.code)
	{
	}

	inline WilmaGateException::~WilmaGateException (void)
	{
		delete [] reason;
	}

	inline std::ostream &operator<< (std::ostream &o, const WilmaGateException &e)
	{
		o << "Exception raised:\n\t"
			<< e.reason << " (opcode " << e.code << ").";
		return o;
	}

	////////////////////////////////////////////
	///
	/// Handles memory exception.
	///
	class OutOfMemoryException:
		public WilmaGateException {
	public:
		inline OutOfMemoryException (void);
	};

	inline OutOfMemoryException::OutOfMemoryException (void):
		WilmaGateException ("Out of memory", 1)
	{}

} // namespace Common

#endif
