/// \file packets.cpp
/// \author Mauro Brunato
///
/// \brief Member functions forpacket structures declared in packet.h.
///
/// $Id: packets.cpp,v 1.3 2005/06/23 14:35:59 cvs Exp $
///
/// Most functions are declared inline, only some of them are complex
/// enough to deserve a definition file.

/*##############################################################################
   
   Developed by NetMob Group, University of Trento, Italy

   University of Trento
   DIT - Department of Informatics and Telecommunications 
   Computer Networks and Mobility Research Program
   http://netmob.unitn.it/ 
   
   This file is part of the WilmaGate system, created as part
   of the WILMA Project, sponsored by the Autonomous
   Province of Trento and participated by ITC-irst, the University
   of Trento and Alpikom Spa. More info at

     http://www.wilmaproject.org/		

   To download the project:
      - in tarball format
        http://netmob.unitn.it/wilmagate.html
      - connect to CVS repository
        :pserver:cvs@rtm.unitn.it:/home/cvsroot
   In both previous cases, the download is password protected. To obtain
   the password you must send a mail to 

      wilmagate@dit.unitn.it

################################################################################

   The WilmaGate Licence of Use


   Copyright (c) 2003-2005
                - Universita` di Trento,
                  Dipartimento di Informatica e Telecomunicazioni (DIT)
                  Computer Networks and Mobility Research Program


   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (hereinafter
   the "WilmaGate") directly from the copyright owner, to use the WilmaGate
   without restriction of purpose or time, including without limitation the
   rights to use, copy, modify, and merge the WilmaGate, subject to the
   following conditions:

   The above copyright notice and this permission notice shall be included
   in all copies or substantial portions of the WilmaGate. 

   The WilmaGate shall not be sub-licenced either commercially or free of
   charge, as is or in part or as a part of any other package. 

   The licensee agrees to properly credit WilmaGate and its copyright 
   owners in any technical, scientific or divulgative publication, 
   whatever publishing means are used, that is based entirely or in part 
   on WilmaGate use, as well as in any booklet, web-site or brochure that 
   advertise services based on WilmaGate use. 

   If the WilmaGate is modified, the copyright owner shall be notified.

   THE WilmaGate IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
   OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
   IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
   CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
   TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
   SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 

##############################################################################*/

#include "addresses.h"
#include "packets.h"

// Commented in addresses.h
namespace Common {

	///
	/// \brief Local utility function.
	///
	/// Calculate the TCP/UDP-like checksum of a packet.
	///
	/// \param src Source IP address.
	/// \param dst Destination IP address.
	/// \param sz Packet size.
	/// \param pk Packet pointer.
	/// \param proto Packet protocol identifier.
	///
	/// \pre The checksum field of the packet must be zero when the function is called.
	///
	/// \return The packet checksum as required in TCP and UDP standards.
	///
	static uint16_t calculateChecksum (IPv4Address src, IPv4Address dst, int sz, uint16_t *pk, uint16_t proto)
	{
		uint32_t cs = sz + proto
				+ (src.toHost() & 0x0000ffff) + (src.toHost() >> 16)
				+ (dst.toHost() & 0x0000ffff) + (dst.toHost() >> 16);
		uint16_t pcs;
		for ( int i = 0; i < (sz+1)/2; i++ )
			cs += ntohs(pk[i]);
		pcs = (uint16_t)((cs & 0x0000ffff) + (cs >> 16));
		return htons(~pcs);
	}

	void UDPDatagram::setReplyHeader (UDPDatagram *d)
	{
		sourcePort = d->destinationPort;
		destinationPort = d->sourcePort;
	}

	void UDPDatagram::calculateChecksum (IPv4Address src, IPv4Address dst, int sz)
	{
		checksum = 0;
		checksum = Common::calculateChecksum (src, dst, sz, reinterpret_cast<uint16_t*>(this), IPPacket::UDP_PROTOCOL);
	}

	void TCPDatagram::setReplyHeader (TCPDatagram *d, int size /* = 0 */)
	{
		sourcePort = d->destinationPort;
		destinationPort = d->sourcePort;
		sequenceNumber = d->acknowledgmentNumber;
		acknowledgmentNumber = htonl (ntohl(d->sequenceNumber) + size - d->getHeaderLength());
		setOptionLength (0);
		windowSize = d->windowSize;
		urgentPointer = 0;
	}

	void TCPDatagram::calculateChecksum (IPv4Address src, IPv4Address dst, int sz)
	{
		checksum = 0;
		checksum = Common::calculateChecksum (src, dst, sz, reinterpret_cast<uint16_t*>(this), IPPacket::TCP_PROTOCOL);
	}

	void IPPacket::setReplyHeader (IPPacket *p)
	{
		version = p->version;
		setMinimumHeaderLength ();
		diffserv = p->diffserv;
		length = htons (sizeof(IPPacket));
		id = p->id;
		offset = 0;
		ttl = 127;
		protocol = p->protocol;
		source = p->destination;
		destination = p->source;
	}

	void IPPacket::calculateHeaderChecksum (void)
	{
		checksum = 0;
		unsigned short *pk = reinterpret_cast<unsigned short*>(this);
		unsigned long cs = 0;
		unsigned short pcs;
		for ( int i = 0; i < 10; i++ )
			cs += ntohs(pk[i]);
		pcs = static_cast<unsigned short>((cs & 0x0000ffff) + (cs >> 16));
		checksum = htons(~pcs);
	}

	void ARPPacket::forgeRequest (IPv4Address myIP, IPv4Address ip, const EthernetMACAddress &myMAC)
	{
		hardware_type = htons(0x0001);
		protocol_type = htons(EthernetFrame::IP_PROTOCOL);
		hardware_size = sizeof (EthernetMACAddress);
		protocol_size = sizeof (IPv4Address);
		opcode = htons(QUERY);
		senderMAC = myMAC;
		senderIP = myIP;
		receiverMAC = "00:00:00:00:00:00";
		receiverIP = ip;
	}

	void ARPPacket::forgeResponse (const ARPPacket &query, const EthernetMACAddress &myMAC)
	{
		memcpy (this, &query, reinterpret_cast<const uint8_t*>(&query.opcode) - reinterpret_cast<const uint8_t*>(&query));
		opcode = htons (ANSWER);
		senderMAC = myMAC;
		senderIP = query.receiverIP;
		receiverMAC = query.senderMAC;
		receiverIP = query.senderIP;
	}

	bool EthernetFrame::isProtocolStack (EthernetFrame::ProtocolType ethp,
						IPPacket::ProtocolType ipp /*= IPPacket::ANY_PROTOCOL*/,
						uint16_t srctpp /*= 0xFFFFU*/,
						uint16_t dsttpp /*= 0xFFFFU*/)
	{
		if ( protocol != ethp )
			return false;
		if ( ethp != htons(IP_PROTOCOL) )
			return true;
		IPPacket *ip = reinterpret_cast<IPPacket*>(payload);
		if ( ipp == IPPacket::ANY_PROTOCOL )
			return true;
		if ( ip->protocol != ipp )
			return false;
		if ( ipp == IPPacket::TCP_PROTOCOL ) {
			TCPDatagram *tcp = reinterpret_cast<TCPDatagram*>(ip->payload);
			return (srctpp == 0xFFFFU || tcp->sourcePort == srctpp)
				&& (dsttpp == 0xFFFFU || tcp->destinationPort == dsttpp);
		}
		if ( ipp == IPPacket::UDP_PROTOCOL ) {
			UDPDatagram *udp = reinterpret_cast<UDPDatagram*>(ip->payload);
			return (srctpp == 0xFFFFU || udp->sourcePort == srctpp)
				&& (dsttpp == 0xFFFFU || udp->destinationPort == dsttpp);
		}
		if ( ipp == IPPacket::ICMP_PROTOCOL )
			return true;
		return false;
	}

	void EthernetFrame::setReplyHeader (EthernetFrame *f)
	{
		sender = f->receiver;
		receiver = f->sender;
		protocol = f->protocol;
	}

} // namespace Common
