/// \file packets.h
/// \author Mauro Brunato
///
/// \brief Structures and utilities to deal with packets.
///
/// $Id: packets.h,v 1.4 2006/03/30 16:33:42 cvs Exp $
///
/// Definitions of all necessary frame, packet and datagram structures,
/// with simple utility functions that deal with network endianness.
///
/// Being very low-level structures, all members are public, so that
/// operations can be carried directly on fields.
/// Usage of proposed methods is suggested only if endianness issues
/// are involved.
/// Rather than structs, data types are declared as classes for compatibility
/// with modeling tools.
///
/// Packet headers have no separate definitions for simplicity, and because
/// in the real world a header is never separated from the packet body.
/// However, the size of each class is actually that of the corresponding header
/// (payload size is set to zero).

/*##############################################################################
   
   Developed by NetMob Group, University of Trento, Italy

   University of Trento
   DIT - Department of Informatics and Telecommunications 
   Computer Networks and Mobility Research Program
   http://netmob.unitn.it/ 
   
   This file is part of the WilmaGate system, created as part
   of the WILMA Project, sponsored by the Autonomous
   Province of Trento and participated by ITC-irst, the University
   of Trento and Alpikom Spa. More info at

     http://www.wilmaproject.org/		

   To download the project:
      - in tarball format
        http://netmob.unitn.it/wilmagate.html
      - connect to CVS repository
        :pserver:cvs@rtm.unitn.it:/home/cvsroot
   In both previous cases, the download is password protected. To obtain
   the password you must send a mail to 

      wilmagate@dit.unitn.it

################################################################################

   The WilmaGate Licence of Use


   Copyright (c) 2003-2005
                - Universita` di Trento,
                  Dipartimento di Informatica e Telecomunicazioni (DIT)
                  Computer Networks and Mobility Research Program


   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (hereinafter
   the "WilmaGate") directly from the copyright owner, to use the WilmaGate
   without restriction of purpose or time, including without limitation the
   rights to use, copy, modify, and merge the WilmaGate, subject to the
   following conditions:

   The above copyright notice and this permission notice shall be included
   in all copies or substantial portions of the WilmaGate. 

   The WilmaGate shall not be sub-licenced either commercially or free of
   charge, as is or in part or as a part of any other package. 

   The licensee agrees to properly credit WilmaGate and its copyright 
   owners in any technical, scientific or divulgative publication, 
   whatever publishing means are used, that is based entirely or in part 
   on WilmaGate use, as well as in any booklet, web-site or brochure that 
   advertise services based on WilmaGate use. 

   If the WilmaGate is modified, the copyright owner shall be notified.

   THE WilmaGate IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
   OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
   IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
   CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
   TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
   SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 

##############################################################################*/

#ifndef _PACKETS__H
#define _PACKETS__H

#include "exceptions.h"
#include "addresses.h"

// Commented in addresses.h
namespace Common {

	////////////////////////////////////////////////////////////////////////
	///
	/// \brief Base class for all packet classes
	///
	/// It overloads the new allocation operators. All packet classes with
	/// variable-size payload publicly inherit from it.
	///
	class VariablePayload {
	public:
		///
		/// \brief Allocate a packet with a payload of a given size.
		///
		/// \param s Implicit parameter - size of the header.
		/// \param l Length of the payload.
		///
		/// \return Pointer to the newly allocated packet.
		///
		inline void *operator new (size_t s, int l) throw (OutOfMemoryException);
		///
		/// \brief Allocate a packet with full payload.
		///
		/// \param s Implicit parameter - size of the header.
		///
		/// \return Pointer to the newly allocated packet.
		///
		inline void *operator new (size_t s) throw (OutOfMemoryException);
	};

	inline void *VariablePayload::operator new (size_t s, int l) throw (OutOfMemoryException)
	{
		void *p = reinterpret_cast<void*>(new uint8_t [s + l]);
		if ( p )
			return p;
		throw OutOfMemoryException ();
	}

	inline void *VariablePayload::operator new (size_t s) throw (OutOfMemoryException)
	{
		void *p = reinterpret_cast<void*>(new uint8_t [s+65536]);
		if ( p )
			return p;
		throw OutOfMemoryException ();
	}

	////////////////////////////////////////////////////
	///
	/// \brief DHCP message
	///
	class DHCPMessage:
		public VariablePayload {
	public:
		/////////////////////////////////////
		///
		/// \brief Type of message.
		///
		enum MessageType {
			QUERY = 1,	///< \brief Query message, normally issued by client.
			RESPONSE = 2	///< \brief response message, normally issued by server.
		};
		///
		/// \brief Message option type
		///
		enum OptionType {
			MESSAGE_TYPE = 53,
			REQUESTED_ADDRESS = 50,
			SERVER_ID = 54,
			SUBNET_MASK = 1,
			ROUTER = 3,
			DNS = 6,
			LEASE_TIME = 51,
			RENEWAL_TIME = 58,
			HOST_NAME = 12,
			DOMAIN_NAME = 15,
			WPAD_URL = 252,
			PARAMETER_REQUEST_LIST =55,
			SUBNET_SELECTION = 118,
			AGENT_OPTION = 82,
			END_OPTIONS = 0xff
		};
                ///
                /// \brief DHCP Message suboption type
                ///
                enum SubOptionType {
                        CIRCUIT_ID = 1,
                        LINK_SELECTION = 5
                };
		///
		/// \brief Option message type
		///
		enum OptionMessageType {
			NONE = 0,
			DISCOVER = 1,
			OFFER = 2,
			REQUEST = 3,
			DECLINE = 4,
			ACK = 5,
			NACK = 6,
			RELEASE = 7,
			INFORM = 8
		};
		///
		/// \brief Canonical packet length
		///
		static const int PACKET_LENGTH = 548;
		//
		///////////////////////////////////////
		///
		/// \brief Type of message (see DHCPMessage::MessageType).
		///
		uint8_t op;
		///
		/// \brief Hardware address type.
		///
		uint8_t htype;
		///
		/// \brief Hardware address length (usually 6).
		///
		uint8_t hlen;
		///
		/// \brief Number of DHCP relays traversed.
		///
		uint8_t hops;
		///
		/// \brief See RFC.
		///
		uint32_t xid;
		///
		/// \brief Time for response.
		///
		uint16_t secs;
		///
		/// \brief Flags.
		///
		uint16_t flags;
		///
		/// \brief Requested address.
		///
		IPv4Address ciaddr;
		///
		/// \brief Client address.
		///
		IPv4Address yiaddr;
		///
		/// \brief Server address.
		///
		IPv4Address siaddr;
		///
		/// \brief Gateway address.
		///
		IPv4Address giaddr;
		///
		/// \brief Client's MAC address.
		///
		EthernetMACAddress chaddr;
		///
		/// \brief Padding to reach the required 16-byte length of the DHCPMessage::chaddr field.
		///
		char padding [16 - sizeof(EthernetMACAddress)];
		///
		/// \brief Server name.
		///
		char sname[64];
		///
		/// \brief Setup file.
		///
		char file[128];
		///
		/// \brief Message cookie.
		///
		uint32_t cookie;
		///
		/// \brief Hardware address type.
		///
		uint8_t options [0];
	};

	////////////////////////////////////////////////////
	///
	/// \brief CAPWAP message
	///
	class CAPWAPMessage:
		public VariablePayload {
	public:
//----------------------------------------------------START OF CAPWAP HEADER
		///
		/// \brief F1, all fields contains in the first 32 bits.
		///
		uint32_t F1;
		//Version:4;RID:5;HLEN:5;WBID:5;T:1;F:1;L:1;W:1;M:1;Flags:8;

		///
		/// \brief F2, all fields contains in the last 32 bits.
		///
		uint32_t F2;
		//FragmentId:16;FragmentOffset:13;FragmentOffset:3;
//----------------------------------------------------END OF CAPWAP HEADER
//----------------------------------------------------START OF CAPWAP CONTROL HEADER
		///
		/// \brief Message Type.
		///
		uint32_t MessageType;

		/// \brief Field with SeqNum, MsgElementLength, Flags
		//uint32_t F3;

		///
		/// \brief Sequence Number.
		///
		uint8_t SeqNum;

		///
		/// \brief Flags.
		///
		uint8_t Flags;

		///
		/// \brief Message Element Length.
		///
		uint16_t MsgElementLength;

		///
		/// \brief Time Stamp.
		///
		uint32_t TimeStamp;
//----------------------------------------------------END OF CAPWAP CONTROL HEADER
		///
		/// \brief Message Element.
		///
		uint8_t MessageElement[0];
		
				/////////////////////////////////////
		///
		/// \brief Message option type
		///
		enum MessageTypes {
			NONE = 0,
			DISCOVERYREQUEST = 1,
			DISCOVERYRESPONSE = 2,
			JOINREQUEST = 3,
			JOINRESPONSE = 4,
			CONFIGURATIONSTATUSREQUEST = 5,
			CONFIGURATIONSTATUSRESPONSE = 6,
			CONFIGURATIONUPDATEREQUEST = 7,
			CONFIGURATIONUPDATERESPONSE = 8,
			RESETREQUEST = 17,
			RESETRESPONSE = 18,
			ECHOREQUEST = 13,
			ECHORESPONSE = 14,
			WIFIREQUEST = 30,
			WIFIRESPONSE = 31,
			UPDATECONFIGURATION = 32
		};
		///
		/// \brief Message Element type
		///
		enum MessageElementType {
			ME_NONE = 0,
			ACDESCRIPTOR = 1,
			ACIPV4LIST = 2,
			ACIPV6LIST  = 3,
			ACNAME = 4,
			CAPWAPTIMERS = 5,
			DISCOVERYTYPE = 6,
			LOCATIONDATA = 7,
			RADIOADMINISTRATIVESTATE = 8,
			RADIOOPERATIONALSTATE = 9,
			RESULTCODE = 10,
			SESSIONID = 11,
			STATISTICSTIMER = 12,
			WTPDESCRIPTOR = 13,
			WTPIPV4ADDRESS = 14,
			WTPNAME = 15,
			WTPSTATICIPADDRESSINFORMATION = 16,
			DIRECTSEQUENCECONTROL = 17,
			TXPOWER = 18,
			WTPRADIOINFORMATION = 19,
		};

		inline uint8_t getVersion(void){uint8_t ver; ver=((F1>>28)&0xF);return ver;}; // !!! Rivedi
	};

	////////////////////////////////////////////////////////////////////////
	///
	/// \brief UDP datagram with utility methods.
	///
	/// All members are public, in order to grant easy access to datagram fields
	///
	class UDPDatagram:
		public VariablePayload {
	public:
		///
		/// \brief Source port number, 16 bits.
		///
		uint16_t sourcePort;
		///
		/// \brief Destination port number, 16 bits.
		///
		uint16_t destinationPort;
		///
		/// \brief Datagram length, 16 bits.
		///
		uint16_t length;
		///
		/// \brief Datagram checksum.
		///
		/// It may be ignored, in this case it must be zero.
		/// 
		uint16_t checksum;
		///
		/// \brief Datagram payload.
		///
		uint8_t payload[0];
		///
		///////////////////////////////////////////////////////////
		///
		/// \brief Set a reply header.
		///
		/// Sets data fields in order to prepare a reply
		/// to the datagram passed as argument (basically
		/// it swaps the ports).
		///
		/// \param d The datagram to which this must be a reply.
		///
		void setReplyHeader (UDPDatagram *d);
		///
		/// \brief Compute the checksum for the UDP datagram.
		///
		/// \param src IP address of the source host.
		/// \param dst IP address of the destination host.
		/// \param sz Packet size as reported in the IP header.
		///
		/// \remarks All parameters must be borrowed from the IP header.
		///
		void calculateChecksum (IPv4Address src, IPv4Address dst, int sz);
		///
		/// \brief Interpret the payload as a CAPWAP MEssage.
		///
		/// \return Reference to the payload.
		///
		inline CAPWAPMessage &CAPWAPPayload(void);
	};
	
	inline CAPWAPMessage &UDPDatagram::CAPWAPPayload (void)
	{
		return *reinterpret_cast<CAPWAPMessage*>(payload);
	}
	
	////////////////////////////////////////////////////////////////////////
	///
	/// \brief TCP datagram with utility methods.
	///
	/// All members are public, in order to grant easy access to datagram fields.
	///
	class TCPDatagram:
		public VariablePayload {
	public:
		///
		/// \brief Datagram type flag masks.
		///
		enum FlagMask {
			FIN_FLAG = 0x01,	///< \brief TCP FIN flag mask
			SYN_FLAG = 0x02,	///< \brief TCP SYN flag mask
			RST_FLAG = 0x04,	///< \brief TCP RST flag mask
			PSH_FLAG = 0x08,	///< \brief TCP PSH flag mask
			ACK_FLAG = 0x10		///< \brief TCP ACK flag mask
		};
		///
		/// \brief Source port.
		///
		uint16_t sourcePort;
		///
		/// \brief Destination port.
		///
		uint16_t destinationPort;
		///
		/// \brief Sequence number.
		///
		uint32_t sequenceNumber;
		///
		/// \brief Expected sequence number of next datagram.
		///
		uint32_t acknowledgmentNumber;
		///
		/// \brief Size of the header (first four bits).
		///
		uint8_t headerLength;
		///
		/// \brief TCP Flags.
		///
		/// See constants declared as TCPDatagram::FlagMask.
		///
		uint8_t flags;
		///
		/// \brief Size of the transmission window.
		///
		uint16_t windowSize;
		///
		/// \brief Packet checksum.
		///
		/// Contrarily to UDP, this cannot be ignored.
		///
		uint16_t checksum;
		///
		/// \brief Urgent pointer.
		///
		uint16_t urgentPointer;
		///
		/// \brief Payload of the datagram.
		///
		/// It could be prefixed by some header options,
		/// better use function getPayload() to obtain the actual payload
		///
		uint8_t payload [0];
		///
		///////////////////////////////////////////
		///
		/// \brief Calculate the datagram checksum.
		///
		/// \param src IP address of the source host.
		/// \param dst IP address of the destination host.
		/// \param sz Packet size.
		///
		/// \remarks All parameters must be borrowed from the IP header.
		///
		void calculateChecksum (IPv4Address src, IPv4Address dst, int sz);
		///
		/// \brief Check whether a flag is set.
		///
		/// \param f The flag to be checked.
		///
		inline bool isFlag (FlagMask f) const;
		///
		/// \brief Set a flag.
		///
		/// \param f The flag to be set.
		///
		inline void setFlag (FlagMask f);
		///
		/// \brief Reset a flag.
		///
		/// \param f The flag to be reset.
		///
		inline void resetFlag (FlagMask f);
		///
		/// \brief Return header options address.
		///
		inline uint8_t *getOptions (void);
		///
		/// \brief Return length (in bytes) of the complete header.
		///
		inline int getHeaderLength (void);
		///
		/// \brief Return length (in bytes) of optional fields.
		///
		inline int getOptionLength (void);
		///
		/// \brief Adjust header length to accommodate l bytes of optional fields.
		///
		/// \param l Length in bytes of desired optional fields.
		///
		inline void setOptionLength (int l);
		///
		/// \brief Return the actual payload address by taking into account header options.
		///
		inline uint8_t *getPayload (void);
		///
		/// \brief Set a reply header.
		///
		/// Sets data fields in order to prepare a reply
		/// to the datagram passed as argument (basically
		/// it swaps the ports and accommodates sequence numbers).
		///
		/// \param d The datagram to which this must be a reply.
		/// \param size Size of the complete datagram (header included).
		///
		/// \remark Only some fields are prepared, length is set to a minimum,
		/// no flags are set and checksum is not computed.
		///
		void setReplyHeader (TCPDatagram *d, int size = 0);
	};

	inline bool TCPDatagram::isFlag (FlagMask f) const
	{
		return flags & f;
	}

	inline void TCPDatagram::setFlag (FlagMask f)
	{
		flags |= f;
	}

	inline void TCPDatagram::resetFlag (FlagMask f)
	{
		flags &= ~f;
	}

	inline uint8_t *TCPDatagram::getOptions (void)
	{
		return payload;
	}

	inline int TCPDatagram::getHeaderLength (void)
	{
		return ((headerLength >> 4) & 0x0f) * 4;
	}

	inline int TCPDatagram::getOptionLength (void)
	{
		return getHeaderLength() - sizeof(TCPDatagram);
	}

	inline uint8_t *TCPDatagram::getPayload (void)
	{
		return payload + getOptionLength();
	}

	inline void TCPDatagram::setOptionLength (int l)
	{
		headerLength = ((sizeof(TCPDatagram) + l) / 4) << 4;
	}

	////////////////////////////////////////////////////////////////////////
	///
	/// \brief IP packet with utility methods.
	///
	/// All members are public, in order to grant easy access to packet fields.
	///
	class IPPacket:
		public VariablePayload {
	public:
		///
		/// \brief Common protocol types.
		///
		enum ProtocolType {
			NO_PROTOCOL = 0x00,	///< \brief No protocol has been assigned to the packet yet.
			TCP_PROTOCOL = 0x06,	///< \brief Value for the TCP protocol.
			UDP_PROTOCOL = 0x11,	///< \brief Value for the UDP protocol.
			ICMP_PROTOCOL = 0x01,	///< \brief Value for the ICMP protocol.
			ANY_PROTOCOL = 0xFF	///< \brief The packet should match any protocol (for comparisons).
		};
		///
		/// \brief IP header version.
		///
		uint8_t version;
		///
		/// \brief Type of service field.
		///
		uint8_t diffserv;
		///
		/// \brief Size of the packet.
		///
		uint16_t length;
		///
		/// \brief Fragment identification number
		///
		uint16_t id;
		///
		/// \brief Fragment offset
		///
		uint16_t offset;
		///
		/// \brief Time to live.
		///
		uint8_t ttl;
		///
		/// \brief Protocol type.
		///
		/// May assume (among others) values declared in IPPacket::ProtocolType.
		///
		uint8_t protocol;
		///
		/// \brief Header checksum.
		///
		uint16_t checksum;
		///
		/// \brief Source address.
		///
		IPv4Address source;
		///
		/// \brief Destination address.
		///
		IPv4Address destination;
		///
		/// \brief Payload: in this library we shall only consider TCP and UDP packets.
		///
		uint8_t payload[0];
		///
		/////////////////////////////////////////////////////////////////////////
		///
		/// \brief Calculate the checksum of the header.
		///
		void calculateHeaderChecksum (void);
		///
		/// \brief Checks whether the payload belongs to a specific protocol.
		///
		/// \param p Protocol to be checked.
		///
		/// \remark This function is almost useless, since the IPPacket::protocol field
		/// is public, but it appears for uniformity with the EthernetFrame::isProtocol function.
		///
		inline bool isProtocol (ProtocolType p) const;
		///
		/// \brief Set a reply header.
		///
		/// Sets data fields in order to prepare a reply
		/// to the datagram passed as argument (basically
		/// it swaps the addresses).
		///
		/// \param p The packet to which this must be a reply.
		///
		void setReplyHeader (IPPacket *p);
		///
		/// \brief Extract the header length in bytes.
		///
		inline int getHeaderLength (void) const;
		///
		/// \brief Sets header length to its minimum
		/// (no options present).
		///
		inline void setMinimumHeaderLength (void);
		///
		/// \brief Interpret the payload as a TCP datagram.
		///
		/// \return Reference to the payload.
		///
		inline TCPDatagram &TCPPayload (void);
		///
		/// \brief Interpret the payload as a UDP datagram.
		///
		/// \return Reference to the payload.
		///
		inline UDPDatagram &UDPPayload (void);
	};

	inline bool IPPacket::isProtocol (ProtocolType p) const
	{
		return protocol == p;
	}

	inline int IPPacket::getHeaderLength (void) const
	{
		return (version & 0x0f) << 2;
	}

	inline void IPPacket::setMinimumHeaderLength (void)
	{
		version = (version & 0xf0) | (sizeof(IPPacket) >> 2);
	}

	inline TCPDatagram &IPPacket::TCPPayload (void)
	{
		return *reinterpret_cast<TCPDatagram*>(payload);
	}

	inline UDPDatagram &IPPacket::UDPPayload (void)
	{
		return *reinterpret_cast<UDPDatagram*>(payload);
	}

	////////////////////////////////////////////////////////////////////////
	///
	/// \brief ARP packet with utility methods.
	///
	/// All members are public, in order to grant easy access to packet fields
	/// The structure is packed because the Ethernet address is 6 bytes long,
	/// and 32-bit word alignment would cause padding between fields.
	///
	class ARPPacket {
	public:
		////////////////////////////////////////
		///
		/// Types of ARP messages.
		///
		enum MessageType {
			QUERY = 0x0001,	///< \brief The message is a query.
			ANSWER = 0x0002	///< \brief The message is a response.
		};
		///
		////////////////////////////////////////
		///
		/// \brief Code for the type of hardware (layer-2 protocol); we always assume Ethernet type
		///
		uint16_t hardware_type;
		///
		/// \brief Code for the Layer-3 protocol; here we assume IP.
		///
		uint16_t protocol_type;
		///
		/// \brief Size for the Layer-2 protocol (should be 6).
		///
		uint8_t hardware_size;
		///
		/// \brief Size for the Layer-3 protocol (should be 4).
		///
		uint8_t protocol_size;
		///
		/// \brief Type of message.
		///
		/// The type as from ARPPacket::MessageType.
		///
		uint16_t opcode;
		///
		/// \brief MAC address of the sender (should replicate that of Ethernet frame).
		///
		EthernetMACAddress senderMAC;
		///
		/// \brief IP address of the sender.
		///
		IPv4Address senderIP;
		///
		/// \brief MAC address of the receiver (usually a broadcast in queries).
		///
		EthernetMACAddress receiverMAC;
		///
		/// \brief IP address of the receiver.
		///
		IPv4Address receiverIP;
		///
		///////////////////////////////////////////////////////////////////////////////
		///
		/// \brief Return the hardware type in host order.
		///
		inline uint16_t getHardwareType (void) const;
		///
		/// \brief Return the layer-3 protocol type in host order.
		///
		inline uint16_t getProtocolType (void) const;
		///
		/// \brief Create a valid request packet.
		///
		/// \param myIP IP address of the sender.
		/// \param ip IP address being queried.
		/// \param myMAC MAC address of the sender.
		///
		void forgeRequest (IPv4Address myIP, IPv4Address ip, const EthernetMACAddress &myMAC);
		///
		/// \brief Create a valid response packet.
		///
		/// \param query Query ARP packet to be answered.
		/// \param myMAC MAC address of the destination.
		///
		void forgeResponse (const ARPPacket &query, const EthernetMACAddress &myMAC);
	}
	#ifndef DOXYSCAN
		__attribute__((packed))
	#endif
		;

	inline uint16_t ARPPacket::getHardwareType (void) const
	{
		return ntohs (hardware_type);
	}

	inline uint16_t ARPPacket::getProtocolType (void) const
	{
		return ntohs (protocol_type);
	}


	////////////////////////////////////////////////////////////////////////
	///
	/// \brief Ethernet frame with utility methods.
	///
	/// All members are public, in order to grant easy access to frame fields.
	/// The structure is packed because the Ethernet address is 6 bytes long,
	/// and 32-bit word alignment would cause padding between fields.
	///
	/// \remark The size of a complete Ethernet frame is rather large,
	/// because it must accommodate for the largest possible
	/// IP packet. Therefore, declaring an EthernetFrame object
	/// should only be done in special cases, and never as a
	/// stack object. It is advisable to use the overloaded
	/// VariablePayload::operator new provided by the
	/// base class (remember to delete the object
	/// afterwards!).
	///
	class EthernetFrame:
		public VariablePayload {
	public:
		///
		/// \brief Possible types of protocol contained in the payload.
		///
		enum ProtocolType {
			NO_PROTOCOL = 0x0000,	///< \brief The packet protocol has not been assigned yet.
			IP_PROTOCOL = 0x0800,	///< \brief An IP packet.
			ARP_PROTOCOL = 0x0806,	///< \brief An ARP packet.
			ANY_PROTOCOL = 0xFFFF	///< \brief Comparisons only: the frame may have any protocol.
		};
		///
		/// \brief The receiver's address.
		///
		EthernetMACAddress receiver;
		///
		/// \brief The sender's address.
		///
		EthernetMACAddress sender;
		///
		/// \brief The protocol.
		///
		/// Some of the possible values are listed in the EthernetFrame::ProtocolType declaration.
		///
		uint16_t protocol;
		///
		/// \brief The payload (usually an IP packet, but could also be an ARP or something else).
		///
		/// Note that this field accounts for a zero-length payload; an EthernetFrame structure
		/// should be allocated using one of the overloaded EthernetFrame::operator new
		/// methods.
		///
		uint8_t payload [0];
		///
		////////////////////////////////////////////////////////
		///
		/// \brief Get the protocol in host order.
		///
		inline uint16_t getProtocol (void) const;
		///
		/// \brief Checks whether the payload belongs to a specific protocol.
		///
		/// \param p Protocol to be checked.
		///
		/// \remark This function is a useless remake of EthernetFrame::getProtocol,
		/// but it can be used to optimize checks, because htons() is applied to a
		/// constant value, rather than to a structure member.
		///
		inline bool isProtocol (ProtocolType p) const;
		///
		/// \brief Test the protocol stack of a packet.
		///
		/// Used by rule tables to check whether a packet matches a protocol rule. Note the default values
		/// of most parameters.
		///
		/// \param ethp Ethernet protocol word (EthernetPacket::ANY_PROTOCOL to match any).
		/// \param ipp IP protocol byte (IPPacket::ANY_PROTOCOL to match any, in this case the ports will not be tested).
		/// \param srctpp Transport protocol source port (0xFFFF to match any).
		/// \param dsttpp Transport protocol destination port (0xFFFF to match any).
		///
		/// \return True if and only if frame matches parameters.
		///
		/// \remark All data must be provided in network byte order (big endian) for faster operation.
		///
		bool isProtocolStack (ProtocolType ethp, IPPacket::ProtocolType ipp = IPPacket::ANY_PROTOCOL,
					uint16_t srctpp = 0xFFFFU, uint16_t dsttpp = 0xFFFFU);
		///
		/// \brief Set a reply header.
		///
		/// Sets data fields in order to prepare a reply
		/// to the datagram passed as argument (basically
		/// it swaps the addresses).
		///
		/// \param f The frame to which this must be a reply.
		///
		void setReplyHeader (EthernetFrame *f);
		///
		/// \brief Interpret the payload as an IP packet.
		///
		/// \return Reference to the payload.
		///
		inline IPPacket &IPPayload (void);
		///
		/// \brief Interpret the payload as an ARP packet.
		///
		/// \return Reference to the payload.
		///
		inline ARPPacket &ARPPayload (void);
	}
	#ifndef DOXYSCAN
		__attribute__((packed))
	#endif
		;

	inline uint16_t EthernetFrame::getProtocol (void) const
	{
		return ntohs (protocol);
	}

	inline bool EthernetFrame::isProtocol (ProtocolType p) const
	{
		return protocol == htons(p);
	}

	inline IPPacket &EthernetFrame::IPPayload (void)
	{
		return *reinterpret_cast<IPPacket*>(payload);
	}

	inline ARPPacket &EthernetFrame::ARPPayload (void)
	{
		return *reinterpret_cast<ARPPacket*>(payload);
	}
} // namespace Common

#endif
