/// \file parameters.h
/// \author Mauro Brunato
///
/// \brief Structures and utilities to read a parameters file.
///
/// $Id: parameters.h,v 1.2 2005/06/16 13:22:22 cvs Exp $

/*##############################################################################
   
   Developed by NetMob Group, University of Trento, Italy

   University of Trento
   DIT - Department of Informatics and Telecommunications 
   Computer Networks and Mobility Research Program
   http://netmob.unitn.it/ 
   
   This file is part of the WilmaGate system, created as part
   of the WILMA Project, sponsored by the Autonomous
   Province of Trento and participated by ITC-irst, the University
   of Trento and Alpikom Spa. More info at

     http://www.wilmaproject.org/		

   To download the project:
      - in tarball format
        http://netmob.unitn.it/wilmagate.html
      - connect to CVS repository
        :pserver:cvs@rtm.unitn.it:/home/cvsroot
   In both previous cases, the download is password protected. To obtain
   the password you must send a mail to 

      wilmagate@dit.unitn.it

################################################################################

   The WilmaGate Licence of Use


   Copyright (c) 2003-2005
                - Universita` di Trento,
                  Dipartimento di Informatica e Telecomunicazioni (DIT)
                  Computer Networks and Mobility Research Program


   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (hereinafter
   the "WilmaGate") directly from the copyright owner, to use the WilmaGate
   without restriction of purpose or time, including without limitation the
   rights to use, copy, modify, and merge the WilmaGate, subject to the
   following conditions:

   The above copyright notice and this permission notice shall be included
   in all copies or substantial portions of the WilmaGate. 

   The WilmaGate shall not be sub-licenced either commercially or free of
   charge, as is or in part or as a part of any other package. 

   The licensee agrees to properly credit WilmaGate and its copyright 
   owners in any technical, scientific or divulgative publication, 
   whatever publishing means are used, that is based entirely or in part 
   on WilmaGate use, as well as in any booklet, web-site or brochure that 
   advertise services based on WilmaGate use. 

   If the WilmaGate is modified, the copyright owner shall be notified.

   THE WilmaGate IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
   OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
   IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
   CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
   TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
   SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 

##############################################################################*/

#ifndef _PARAMETERS_H_
#define _PARAMETERS_H_

#include <string>
#include <fstream>

#include "addresses.h"
#include "exceptions.h"

using namespace std;

namespace Common {

	///////////////////////////////////////////////////////////////////////
	///
	/// \brief Exception launched by the Common::Parameters class if anomalous
	/// conditions are verified.
	///
	class ParameterException:
		public WilmaGateException {
	public:
		///
		/// \brief Constructor.
		///
		/// \param code Number that specifies the source of the error.
		///
		inline ParameterException (int code);
	};

	inline ParameterException::ParameterException (int code):
		WilmaGateException ("Parameter error", code)
	{
	}

	/////////////////////////////////////////////////////////////////////////
	///
	/// \brief Extraction of parameters from a file and from the command line
	///
	class Parameters {
	private:
		///
		/// \brief Configuration file to be read.
		///
		ifstream ifs;
		///
		/// \brief Buffer for file lines.
		///
		char line [200];
		///
		/// \brief Current command line argument pointer.
		///
		int currentArg;
		///
		/// \brief Number of available command line arguments.
		///
		const int argNo;
		///
		/// \brief Array of available arguments.
		///
		const char * const * const argList;
		///
		/// \brief Read next configuration string.
		///
		/// Internal function for reading the next configuration string.
		/// Use the extraction operators for actual parameter values.
		///
		const char *readParameter (void) throw (ParameterException);
	public:
		///
		/// \brief Constructor.
		///
		/// \param argc parameter count from the main() function.
		/// \param argv parameter array from the main() function.
		///
		Parameters (int argc, char **argv) throw (ParameterException);
		///
		/// \brief Destructor: close the parameters file.
		///
		~Parameters (void);
		///
		/// \brief Extract an integer value from the file.
		///
		Parameters& operator>> (int&) throw (ParameterException);
		///
		/// \brief Extract a float value from the file.
		///
		Parameters& operator>> (float&) throw (ParameterException);
		///
		/// \brief Extract a double value from the file.
		///
		Parameters& operator>> (double&) throw (ParameterException);
		///
		/// \brief Extract a string from the file.
		///
		Parameters& operator>> (string&) throw (ParameterException);
		///
		/// \brief Extract an IP address from the file.
		///
		Parameters& operator>> (IPv4Address&) throw (ParameterException);
		///
		/// \brief Extract an Ethernet address from the file.
		///
		Parameters& operator>> (EthernetMACAddress&) throw (ParameterException);
	};

} // namespace Common

#endif
