/// \file socket.cpp
/// \author Mauro Brunato
///
/// \brief Implementation of classes for socket management.
///
/// $Id: socket.cpp,v 1.2 2005/06/16 13:22:22 cvs Exp $

/*##############################################################################
   
   Developed by NetMob Group, University of Trento, Italy

   University of Trento
   DIT - Department of Informatics and Telecommunications 
   Computer Networks and Mobility Research Program
   http://netmob.unitn.it/ 
   
   This file is part of the WilmaGate system, created as part
   of the WILMA Project, sponsored by the Autonomous
   Province of Trento and participated by ITC-irst, the University
   of Trento and Alpikom Spa. More info at

     http://www.wilmaproject.org/		

   To download the project:
      - in tarball format
        http://netmob.unitn.it/wilmagate.html
      - connect to CVS repository
        :pserver:cvs@rtm.unitn.it:/home/cvsroot
   In both previous cases, the download is password protected. To obtain
   the password you must send a mail to 

      wilmagate@dit.unitn.it

################################################################################

   The WilmaGate Licence of Use


   Copyright (c) 2003-2005
                - Universita` di Trento,
                  Dipartimento di Informatica e Telecomunicazioni (DIT)
                  Computer Networks and Mobility Research Program


   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (hereinafter
   the "WilmaGate") directly from the copyright owner, to use the WilmaGate
   without restriction of purpose or time, including without limitation the
   rights to use, copy, modify, and merge the WilmaGate, subject to the
   following conditions:

   The above copyright notice and this permission notice shall be included
   in all copies or substantial portions of the WilmaGate. 

   The WilmaGate shall not be sub-licenced either commercially or free of
   charge, as is or in part or as a part of any other package. 

   The licensee agrees to properly credit WilmaGate and its copyright 
   owners in any technical, scientific or divulgative publication, 
   whatever publishing means are used, that is based entirely or in part 
   on WilmaGate use, as well as in any booklet, web-site or brochure that 
   advertise services based on WilmaGate use. 

   If the WilmaGate is modified, the copyright owner shall be notified.

   THE WilmaGate IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
   OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
   IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
   CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
   TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
   SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 

##############################################################################*/

#include <sys/socket.h>
#include <arpa/inet.h>
#include <netinet/in.h>
#include <unistd.h>

#include "socket.h"

namespace Common {

	static int lingeringClose (int s)
	{
		linger l;
		l.l_onoff = 1;
		l.l_linger = 0;
		setsockopt (s, SOL_SOCKET, SO_LINGER, reinterpret_cast<char*>(&l), sizeof(l));
		int i = ::close (s);
		return i;
	}

	UDPSocket::UDPSocket (void)
		throw (SocketException)
	{
		socket = ::socket (AF_INET, SOCK_DGRAM, 0);
		thread = 0;
		listener = NULL;
		id = 0;
		if ( socket < 0 )
			throw SocketException (1);
	}

	Socket::~Socket (void)
		throw (SocketException)
	{
		listener = NULL;
		if ( socket && lingeringClose (socket) )
			throw SocketException (50+id);
	}

	void Socket::registerListener (PacketListener &l, int n)
	{
		listener = &l;
		id = n;
	}

	bool Socket::bind (IPv4Address ip, uint16_t port)
	{
		sockaddr_in laddr;
		localIP = ip;
		localPort = port;
		memset (&laddr, 0, sizeof(laddr));
		laddr.sin_family = AF_INET;
		laddr.sin_addr.s_addr = ip.asInteger ();
		laddr.sin_port = htons(port);
		return ::bind (socket, reinterpret_cast<sockaddr*>(&laddr), sizeof(laddr)) >= 0;
	}

	void Socket::setDestination (IPv4Address ip, uint16_t port)
	{
		remoteIP = ip;
		remotePort = port;
		memset (&raddr, 0, sizeof(raddr));
		raddr.sin_family = AF_INET;
		raddr.sin_addr.s_addr = ip.asInteger ();
		raddr.sin_port = htons(port);
	}

	bool UDPSocket::send (const void *message, int length)
	{
		return sendto (socket, message, length, 0, reinterpret_cast<sockaddr*>(&raddr), sizeof (raddr)) >= 0;
	}

	void *UDPRecvThread (void *sk)
	{
		UDPSocket *sock = reinterpret_cast<UDPSocket*>(sk);
		uint8_t *buffer = new uint8_t [65536];
		sockaddr_in addr;
		memset (&addr, 0, sizeof(addr));
		addr.sin_family = AF_INET;
		SocketListener::SuggestedAction action = SocketListener::CONTINUE;
		for ( ; ; ) {
			socklen_t l = sizeof(addr);
			int res = recvfrom (sock->socket, buffer, 65536, 0, reinterpret_cast<sockaddr*>(&addr), &l);
			if ( !sock->listener )
				break;
			if ( res < 0 ) {
				action = sock->listener->failed (sock->id);
				if ( action != SocketListener::CONTINUE )
					break;
			}
			action = sock->listener->receivedPacket (buffer, res, sock->id, addr.sin_addr, ntohs(addr.sin_port));
			if ( action != SocketListener::CONTINUE )
				break;
		}
		delete buffer;
		if ( action == SocketListener::DELETE ) {
			sock->close ();
			delete sock;
		}
		return NULL;
	}

	void UDPSocket::startReceiving (void)
		throw (SocketException)
	{
		if ( pthread_create (&thread, NULL, UDPRecvThread, this) )
			throw SocketException (8);
	}

	void UDPSocket::stopReceiving (void)
		throw (SocketException)
	{
	}

	TCPSocket::TCPSocket (void)
		throw (SocketException)
	{
		socket = ::socket (AF_INET, SOCK_STREAM, 0);
		thread = 0;
		listener = NULL;
		id = 0;
		clistener = NULL;
		if ( socket < 0 )
			throw SocketException (1);
	}

	TCPSocket::TCPSocket (int s, sockaddr_in *addr)
		throw (SocketException)
	{
		socket = s;
		thread = 0;
		listener = NULL;
		clistener = NULL;
		remoteIP = addr->sin_addr;
		remotePort = ntohs(addr->sin_port);
		raddr = *addr;
	}

	TCPSocket::~TCPSocket (void)
		throw (SocketException)
	{
		clistener = NULL;
		slistener = NULL;
	}

	void TCPSocket::registerConnectionListener (ConnectionListener &l, int n)
	{
		clistener = &l;
		id = n;
	}

	void TCPSocket::registerStreamListener (StreamListener &l)
	{
		slistener = &l;
	}

	bool TCPSocket::setDestination (IPv4Address ip, uint16_t port)
	{
		Socket::setDestination (ip, port);
		return connect (socket, reinterpret_cast<sockaddr*>(&raddr), sizeof (raddr)) >= 0;
	}

	bool TCPSocket::send (const void *message, int length)
	{
		return ::send (socket, message, length, 0) >= 0;
	}

	static int getString (int skt, uint8_t *buffer)
	{
		int l = 0;
		while ( recv (skt, buffer+l, 1, 0) > 0 ) {
			if ( buffer[l] == '\n' ) {
				buffer[l] = '\0';
				return l;
			}
			if ( buffer[l] != '\r' )
				l++;
		}
		return -1;
	}

	void *TCPRecvThread (void *sk)
	{
		TCPSocket *sock = reinterpret_cast<TCPSocket*>(sk);
		uint8_t *buffer = new uint8_t [65536];
		SocketListener::SuggestedAction action = SocketListener::CONTINUE;
		for ( ; ; ) {
			int l = getString (sock->socket, buffer);
			if ( !sock->listener )
				break;
			if ( l < 0 ) {
				action = sock->listener->failed (sock->id);
				if ( action != SocketListener::CONTINUE )
					break;
			}
			action = sock->listener->receivedPacket (buffer, l, sock->id, sock->remoteIP, sock->remotePort);
			if ( action != SocketListener::CONTINUE )
				break;
		}
		delete buffer;
		if ( action == SocketListener::DELETE ) {
			sock->close ();
			sock->socket = 0;
			delete sock;
		}
		return NULL;
	}

	bool TCPSocket::readString (string &s)
	{
		char buffer[65536];
		int l = getString (socket, reinterpret_cast<uint8_t*>(buffer));
		if ( l < 0 )
			return false;
		s = buffer;
		return true;
	}

	void *TCPSyncRecvThread (void *sk)
	{
		TCPSocket *&sock = reinterpret_cast<TCPSocket*&>(sk);
		sock->slistener->connectionOpen (sock);
		sock->close ();
		delete sock;
		return NULL;
	}

	void TCPSocket::startReceiving (void)
		throw (SocketException)
	{
		pthread_attr_t tha;
		pthread_attr_init (&tha);
		pthread_attr_setdetachstate (&tha, PTHREAD_CREATE_DETACHED);
		if ( pthread_create (&thread, &tha, TCPRecvThread, this) )
			throw SocketException (8);
	}

	void TCPSocket::startSyncReceiving (void)
		throw (SocketException)
	{
		pthread_attr_t tha;
		pthread_attr_init (&tha);
		pthread_attr_setdetachstate (&tha, PTHREAD_CREATE_DETACHED);
		if ( pthread_create (&thread, &tha, TCPSyncRecvThread, this) )
			throw SocketException (9);
	}

	void *TCPListenThread (void *sk)
	{
		TCPSocket *sock = reinterpret_cast<TCPSocket*>(sk);
		sockaddr_in addr;
		memset (&addr, 0, sizeof(addr));
		addr.sin_family = AF_INET;
		SocketListener::SuggestedAction action = SocketListener::CONTINUE;
		for ( ; ; ) {
			socklen_t l = sizeof(addr);
			int res = accept (sock->socket, reinterpret_cast<sockaddr*>(&addr), &l);
			if ( !sock->clistener )
				break;
			if ( res == -1 ) {
				action = sock->clistener->failed (sock->id);
				if ( action != SocketListener::CONTINUE )
					break;
			}
			try {
				if ( !sock->clistener )
					break;
				TCPSocket *s = new TCPSocket (res, &addr);
				action = sock->clistener->incomingConnection (s, sock->id);
				if ( action != SocketListener::CONTINUE )
					delete s;
				if ( action == SocketListener::STOP || action == SocketListener::DELETE )
					break;
			}
			catch ( SocketException& ) {				
				if ( !sock->clistener )
					break;
				action = sock->clistener->failed (sock->id);
				if ( action != SocketListener::CONTINUE )
					break;
			}
		}
		if ( action == SocketListener::DELETE )
			delete sock;
		return NULL;

	}


	void TCPSocket::startListening (void)
		throw (SocketException)
	{
		if ( ::listen (socket, 5) < 0 )
			throw SocketException (8000);
		if ( pthread_create (&thread, NULL, TCPListenThread, this) )
			throw SocketException (8001);
	}

	void TCPSocket::stopListening (void)
		throw (SocketException)
	{
	}

	void Socket::close (void)
		throw (SocketException)
	{
		if ( socket && lingeringClose (socket) )
			throw SocketException (10010);
		socket = 0;
	}

	void Socket::setReceiveBuffer (uint32_t s)
		throw (SocketException)
	{
		int sz = sizeof (s);
		if ( setsockopt (socket, SOL_SOCKET, SO_RCVBUF, &s, sz) )
			throw SocketException (1023);
	}


} // namespace Common
