/// \file socket.h
/// \author Mauro Brunato
///
/// \brief All classes for socket management.
///
/// $Id: socket.h,v 1.3 2005/12/13 17:18:53 cvs Exp $
///
/// Definition of classes and interfaces for using TCP and UDP sockets.
/// All sockets work with the "Listener class" method for event notification: they
/// provide registering methods for classes that extend the appropriate "listener"
/// abstraction. Receiving and listening operations are asynchronous and executed on a separate
/// thread.
///
/// \remark Do not confuse the "listening" approach implemented by programming callbacks and
/// the "listening" of passive TCP sockets waiting for incoming connection requests.

/*##############################################################################
   
   Developed by NetMob Group, University of Trento, Italy

   University of Trento
   DIT - Department of Informatics and Telecommunications 
   Computer Networks and Mobility Research Program
   http://netmob.unitn.it/ 
   
   This file is part of the WilmaGate system, created as part
   of the WILMA Project, sponsored by the Autonomous
   Province of Trento and participated by ITC-irst, the University
   of Trento and Alpikom Spa. More info at

     http://www.wilmaproject.org/		

   To download the project:
      - in tarball format
        http://netmob.unitn.it/wilmagate.html
      - connect to CVS repository
        :pserver:cvs@rtm.unitn.it:/home/cvsroot
   In both previous cases, the download is password protected. To obtain
   the password you must send a mail to 

      wilmagate@dit.unitn.it

################################################################################

   The WilmaGate Licence of Use


   Copyright (c) 2003-2005
                - Universita` di Trento,
                  Dipartimento di Informatica e Telecomunicazioni (DIT)
                  Computer Networks and Mobility Research Program


   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (hereinafter
   the "WilmaGate") directly from the copyright owner, to use the WilmaGate
   without restriction of purpose or time, including without limitation the
   rights to use, copy, modify, and merge the WilmaGate, subject to the
   following conditions:

   The above copyright notice and this permission notice shall be included
   in all copies or substantial portions of the WilmaGate. 

   The WilmaGate shall not be sub-licenced either commercially or free of
   charge, as is or in part or as a part of any other package. 

   The licensee agrees to properly credit WilmaGate and its copyright 
   owners in any technical, scientific or divulgative publication, 
   whatever publishing means are used, that is based entirely or in part 
   on WilmaGate use, as well as in any booklet, web-site or brochure that 
   advertise services based on WilmaGate use. 

   If the WilmaGate is modified, the copyright owner shall be notified.

   THE WilmaGate IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
   OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
   IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
   CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
   TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
   SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 

##############################################################################*/

#ifndef _H_SOCKET_H_
#define _H_SOCKET_H_

#include <arpa/inet.h>

#include <pthread.h>

#include "exceptions.h"
#include "addresses.h"

// Commented in addresses.h
namespace Common {

	///////////////////////////////////////////////////////////////////////
	///
	/// \brief Exception launched by all socket operations if anomalous
	/// conditions are verified.
	///
	class SocketException:
		public WilmaGateException {
	public:
		///
		/// \brief Constructor.
		///
		/// \param code Number that specifies the source of the error.
		///
		inline SocketException (int code);
	};

	inline SocketException::SocketException (int code):
		WilmaGateException ("Socket error", code)
	{
	}

	//////////////////////////////////////////////////////////////////////////
	///
	/// \brief Base class for interfaces that listen to socket events.
	///
	/// \remark This class should not be directly extended, because it only
	/// defines types and callbacks that are common to all listeners.
	/// Use Common::PacketListener and Common::ConnectionListener
	/// as base classes.
	///
	class SocketListener {
	public:
		///
		/// \brief Return values of callbacks.
		///
		/// The value determines what the socket thread should do
		/// when the callback returns.
		///
		enum SuggestedAction {
			CONTINUE,	///< \brief The socket thread should move on to the next packet.
			STOP,		///< \brief The socket thread should stop.
			REFUSE,		///< \brief Data or connection is refused, but the thread must go on.
			DELETE		///< \brief The socket object should delete itself
					///< (be careful: the object must have been dynamically allocated).
		};
		///
		/// \brief Virtual destructor.
		///
		virtual ~SocketListener (void) {}
		///
		/// \brief Called in case of failure in the socket thread.
		///
		/// \param socketID ID of the calling socket (see Common::UDPSocket::registerListener for details).
		///
		/// \return Action to be performed by the socket thread.
		///
		virtual SuggestedAction failed (int socketID) = 0;
	};

	/////////////////////////////////////////////////////////////////////////////////////
	///
	/// \brief Packet listener interface.
	///
	/// A class that wants to be notified when a packet is received should
	/// extend this interface and register itself to the socket object.
	///
	class PacketListener:
		public SocketListener {
	public:
		///
		/// \brief Called when a packet has been received.
		///
		/// \param message Pointer to the packet.
		/// \param size Size of the packet.
		/// \param socketID ID of the calling socket (see Common::UDPSocket::registerListener for details).
		/// \param senderIP IP address of the packet sender.
		/// \param senderPort Transport layer port of the packet sender.
		///
		/// \return Action to be performed by the socket thread.
		///
		virtual SuggestedAction receivedPacket (uint8_t *message, int size, int socketID, IPv4Address senderIP,
							uint16_t senderPort) = 0;
	};

	//////////////////////////////////////////////////////////////////////////
	///
	/// \brief Base class for BSD-style sockets
	///
	class Socket {
	protected:
		///
		/// \brief Local IP address (if the socket has been bound to any).
		///
		IPv4Address localIP;
		///
		/// \brief Local port (if the socket has been bound to any).
		///
		uint16_t localPort;
		///
		/// \brief Remote IP address (must be set prior to sending a packet).
		///
		IPv4Address remoteIP;
		///
		/// \brief Remote port (must be set prior to sending a packet).
		///
		uint16_t remotePort;
		///
		/// \brief Remote address of outgoing packets, same information as the two fields above
		/// in a Berkeley-compliant structure.
		///
		sockaddr_in raddr;
		///
		/// \brief Socket handle.
		///
		int socket;
		///
		/// \brief Listening or receiving thread handle.
		///
		pthread_t thread;
		///
		/// \brief Pointer to the registered listener object.
		///
		PacketListener *listener;
		///
		/// \brief ID of the socket with respect to the listener object (see Common::Socket::registerListener for details).
		///
		int id;
	public:
		///
		/// \brief Destructor.
		///
		/// Closes the socket and halts an ongoing receiving thread.
		///
		~Socket (void) throw (SocketException);
		///
		/// \brief Listener registration.
		///
		/// Called by the listener object to register itself to the socket.
		/// From this moment on, if the receiving thread is started,
		/// all received packets will cause a call to the PacketListener::ReceivedPacket method
		/// of the registered object.
		///
		/// Since a listener can register itself to many sockets, it will also
		/// pass an arbitrary ID value which shall be used in all subsequent callback communications
		/// by the socket.
		///
		/// \param l Reference to the listener object.
		/// \param n Socket ID with respect to the listener object.
		///
		void registerListener (PacketListener &l, int n);
		///
		/// \brief Bind the socket to a specific IP/port address.
		///
		/// \param ip IP address.
		/// \param port Port.
		///
		/// \return True if OK, false if error.
		///
		bool bind (IPv4Address ip, uint16_t port);
		///
		/// \brief Set the destination of subsequent packets.
		///
		/// \param ip IP address of the destination.
		/// \param port Port of the destination.
		///
		void setDestination (IPv4Address ip, uint16_t port);
		///
		/// \brief Return the remote IP address.
		///
		inline IPv4Address getRemoteIP (void) const;
		///
		/// \brief return the remote port.
		///
		inline uint16_t getRemotePort (void) const;
		///
		/// \brief Wait for completion of the receiving thread.
		///
		inline void wait (void);
		///
		/// \brief Close the connection
		///
		void close (void) throw (SocketException);
		///
		/// \brief Set the receive buffer size
		///
		void setReceiveBuffer (uint32_t) throw (SocketException);
	};

	inline IPv4Address Socket::getRemoteIP (void) const
	{
		return remoteIP;
	}

	inline uint16_t Socket::getRemotePort (void) const
	{
		return remotePort;
	}

	inline void Socket::wait (void)
	{
		pthread_join (thread, NULL);
	}

	//////////////////////////////////////////////////////////////////////////////////////////
	///
	/// \brief UDP socket wrapper class.
	///
	/// Provides binding and sending functions as the common Berkeley UDP sockets,
	/// and provides an asynchronous thread function with a callback mechanism for
	/// receiving packets.
	///
	class UDPSocket:
			public Socket {
		//
		// See socket.cpp for details.
		//
		friend void *UDPRecvThread (void *sk);
	protected:
	public:
		///
		/// \brief Constructor.
		///
		/// Instantiates a new socket of class AF_INET, type SOCK_DGRAM, default protocol (UDP).
		///
		UDPSocket (void) throw (SocketException);
		///
		/// \brief Send a packet.
		///
		/// The packet shall be sent to the destination address specified by UDPSocket::setDestination.
		///
		/// \param message Pointer to the packet payload.
		/// \param length Size of the payload.
		///
		/// \return True if OK, false if error.
		///
		bool send (const void *message, int length);
		///
		/// \brief Start the receiving thread.
		///
		/// All received packets shall be sent to the registered listener via its
		/// PacketListener::receivedPacket method.
		///
		void startReceiving (void) throw (SocketException);
		///
		/// \brief Stop the receiving thread.
		///
		void stopReceiving (void) throw (SocketException);
	};

	//
	// Forward declaration: see below.
	//
	class TCPSocket;

	/////////////////////////////////////////////////////////////////////////////////////
	///
	/// \brief Connection listener interface.
	///
	/// For passive TCP sockets, the interface provides a callback method to
	/// notify incoming connections.
	///
	class ConnectionListener:
		public SocketListener {
	public:
		///
		/// \brief Incoming connection.
		///
		/// This method is called when the passive TCP socket receives and accepts an incoming
		/// TCP connection.
		///
		/// \param s Pointer to the newly created TCP socket handling the new connection.
		/// \param socketID ID of the passive socket performing the callback.
		///
		/// \return Action to be performed by the calling thread.
		///
		virtual SuggestedAction incomingConnection (TCPSocket *s, int socketID) = 0;
	};

	/////////////////////////////////////////////////////////////////////////////////////
	///
	/// \brief Stream listener interface.
	///
	/// A class that wants to implement an active socket I/O method
	/// must implement this interface.
	///
	class StreamListener:
		public SocketListener {
	public:
		///
		/// \brief Called when a TCP socket is ready for communication.
		///
		/// \param socket Pointer to the TCP socket.
		///
		virtual void connectionOpen (TCPSocket *socket) = 0;
	};

	////////////////////////////////////////////////////////////////////////////////////
	///
	/// \brief TCP socket wrapper class.
	///
	class TCPSocket:
			public Socket {
		//
		// Thread functions: see socket.cpp for comments.
		//
		friend void *TCPRecvThread (void *sk);
		friend void *TCPSyncRecvThread (void *sk);
		friend void *TCPListenThread (void *sk);
	private:
		///
		/// \brief Pointer to a connection listener object.
		///
		ConnectionListener *clistener;
		///
		/// \brief Pointer to a stream listener object.
		///
		StreamListener *slistener;
		///
		/// \brief Constructor, called when the TCP socket is already created and connected.
		///
		/// \param s Handle of the TCP socket.
		/// \param addr Pointer to the address of the remote endpoint.
		///
		TCPSocket (int s, sockaddr_in *addr) throw (SocketException);
	public:
		///
		/// \brief Empty constructor.
		///
		/// Instantiates an unbound and unconnected TCP socket.
		///
		TCPSocket (void) throw (SocketException);
		///
		/// \brief Destructor.
		///
		/// Closes the socket.
		///
		~TCPSocket (void) throw (SocketException);
		///
		/// \brief Set the remote endpoint and connect.
		///
		/// Differently from its UDPSocket counterpart,
		/// this function also tries to connect to the destination
		/// and throws a SocketException if connection fails.
		///
		/// \param ip IP address of the remote endpoint.
		/// \param port Remote port.
		///
		/// \return True if OK, false if error.
		///
		bool setDestination (IPv4Address ip, uint16_t port);
		///
		/// \brief Send a message to the connected endpoint.
		///
		/// \param message Pointer to the message.
		/// \param length Length of the message.
		///
		/// \return True if OK, false otherwise.
		///
		bool send (const void *message, int length);
		///
		/// \brief Start the asynchronous receiving thread in a connected socket.
		///
		/// All incoming messages shall be treated via the registered listener object.
		///
		void startReceiving (void) throw (SocketException);
		///
		/// \brief Start the synchronous receiving thread in a connected socket.
		///
		/// All incoming messages shall be treated via the registered listener object.
		///
		void startSyncReceiving (void) throw (SocketException);
		///
		/// \brief Read a string from the socket (in synchronous mode).
		///
		/// \param s Reference to the string.
		///
		/// \return true if a string has been read, false if socket closed or error.
		///
		bool readString (string &s);
		///
		/// \brief Register a connection event listener to a passive socket.
		///
		/// Since a listener can register itself to many sockets, it will also
		/// pass an arbitrary ID value which shall be used in all subsequent callback communications
		/// by the socket.
		///
		/// \param l Reference to the connection listener object.
		/// \param n ID of the socket with respect to the listener.
		///
		void registerConnectionListener (ConnectionListener &l, int n);
		///
		/// \brief Register a stream listener to an active and open socket.
		///
		/// \param l Reference to the stream listener object.
		///
		void registerStreamListener (StreamListener &l);
		///
		/// \brief Start a passive listening thread.
		///
		void startListening (void) throw (SocketException);
		///
		/// \brief Stop the passive connection listening thread.
		///
		void stopListening (void) throw (SocketException);
	};

} // namespace Common

#endif
