/// \file Common/test.cpp
/// \author Mauro Brunato
///
/// \brief Small program for testing functions.
///
/// $Id: test.cpp,v 1.2 2005/06/16 13:22:22 cvs Exp $
///
/// The main purpose of this program is to see whether packed structures
/// are properly treated, lest word alignment spoils packet structures.
/// Also network interfaces are checked.

/*##############################################################################
   
   Developed by NetMob Group, University of Trento, Italy

   University of Trento
   DIT - Department of Informatics and Telecommunications 
   Computer Networks and Mobility Research Program
   http://netmob.unitn.it/ 
   
   This file is part of the WilmaGate system, created as part
   of the WILMA Project, sponsored by the Autonomous
   Province of Trento and participated by ITC-irst, the University
   of Trento and Alpikom Spa. More info at

     http://www.wilmaproject.org/		

   To download the project:
      - in tarball format
        http://netmob.unitn.it/wilmagate.html
      - connect to CVS repository
        :pserver:cvs@rtm.unitn.it:/home/cvsroot
   In both previous cases, the download is password protected. To obtain
   the password you must send a mail to 

      wilmagate@dit.unitn.it

################################################################################

   The WilmaGate Licence of Use


   Copyright (c) 2003-2005
                - Universita` di Trento,
                  Dipartimento di Informatica e Telecomunicazioni (DIT)
                  Computer Networks and Mobility Research Program


   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (hereinafter
   the "WilmaGate") directly from the copyright owner, to use the WilmaGate
   without restriction of purpose or time, including without limitation the
   rights to use, copy, modify, and merge the WilmaGate, subject to the
   following conditions:

   The above copyright notice and this permission notice shall be included
   in all copies or substantial portions of the WilmaGate. 

   The WilmaGate shall not be sub-licenced either commercially or free of
   charge, as is or in part or as a part of any other package. 

   The licensee agrees to properly credit WilmaGate and its copyright 
   owners in any technical, scientific or divulgative publication, 
   whatever publishing means are used, that is based entirely or in part 
   on WilmaGate use, as well as in any booklet, web-site or brochure that 
   advertise services based on WilmaGate use. 

   If the WilmaGate is modified, the copyright owner shall be notified.

   THE WilmaGate IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
   OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
   IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
   CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
   TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
   SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 

##############################################################################*/

#include <iostream>
#include <iomanip>

#include "exceptions.h"
#include "addresses.h"
#include "packets.h"
#include "interfaces.h"

using namespace std;
using namespace Common;

//const char * const IFNAME = "eth1";
const char * const IFNAME = "eth2";

///
/// Global variable, becomes false at the first error.
///
static bool ok = true;

///
/// \brief Test returned values.
///
/// \param s Short descriptive string.
/// \param a Actual value.
/// \param b Expected value.
///
/// \Remarks The global variable ok will be irrevocably set to false at the first error.
///
void test (const char *s, int a, int b)
{
	ok &= a==b;
	cout << setw(20) << s << setw(4) << a;
	if ( a == b )
		cout << " OK";
	else
		cout << " NO: should be " << b;
	cout << endl;
}

int main (void)
{
	try {
		EthernetFrame *eth = new (1000) EthernetFrame;

		IPPacket *ip = &(eth->IPPayload());
		TCPDatagram *tcp = &(ip->TCPPayload());
		UDPDatagram *udp = &(ip->UDPPayload());

		test ("IPv4Address",		sizeof(IPv4Address),								 4);
		test ("EthernetMACAddress",	sizeof(EthernetMACAddress),							 6);
		test ("Ethernet header",	reinterpret_cast<char*>(&(eth->payload)) - reinterpret_cast<char*>(eth),	14);
		test ("struct EthernetFrame",	sizeof (EthernetFrame),								14);
		test ("ARP packet",		sizeof(ARPPacket),								28);
		test ("IP header",		reinterpret_cast<char*>(&(ip->payload)) - reinterpret_cast<char*>(ip),		20);
		test ("TCP header",		reinterpret_cast<char*>(&(tcp->payload)) - reinterpret_cast<char*>(tcp),	20);
		test ("UDP header",		reinterpret_cast<char*>(&(udp->payload)) - reinterpret_cast<char*>(udp),	 8);

		delete eth;

		cerr << (ok ? "All tests ok\n" : "Test failed\n");

		IPv4Address ipaddr = "192.168.195.55";
		EthernetMACAddress macaddr;
		int res;

		cout << "Fake address: " << ipaddr << endl;

		res = getSettings (IFNAME, ipaddr, macaddr);

		if ( res < 0 ) {
			cerr << "Interface test not passed.\n";
			ok = false;
		}
		else
			cout << "IP address: " << ipaddr
				<< "\nMAC address: " << macaddr
				<< endl;
	}
	catch ( WilmaGateException &ex ) {
		cerr << ex << '\n';
		ok = false;
	}

	return !ok;
}
