/// \file timers.h
/// \author Mauro Brunato
///
/// \brief Structures and utilities to deal with timing.
///
/// $Id: timers.h,v 1.3 2005/12/13 17:18:53 cvs Exp $
///

/*##############################################################################
   
   Developed by NetMob Group, University of Trento, Italy

   University of Trento
   DIT - Department of Informatics and Telecommunications 
   Computer Networks and Mobility Research Program
   http://netmob.unitn.it/ 
   
   This file is part of the WilmaGate system, created as part
   of the WILMA Project, sponsored by the Autonomous
   Province of Trento and participated by ITC-irst, the University
   of Trento and Alpikom Spa. More info at

     http://www.wilmaproject.org/		

   To download the project:
      - in tarball format
        http://netmob.unitn.it/wilmagate.html
      - connect to CVS repository
        :pserver:cvs@rtm.unitn.it:/home/cvsroot
   In both previous cases, the download is password protected. To obtain
   the password you must send a mail to 

      wilmagate@dit.unitn.it

################################################################################

   The WilmaGate Licence of Use


   Copyright (c) 2003-2005
                - Universita` di Trento,
                  Dipartimento di Informatica e Telecomunicazioni (DIT)
                  Computer Networks and Mobility Research Program


   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (hereinafter
   the "WilmaGate") directly from the copyright owner, to use the WilmaGate
   without restriction of purpose or time, including without limitation the
   rights to use, copy, modify, and merge the WilmaGate, subject to the
   following conditions:

   The above copyright notice and this permission notice shall be included
   in all copies or substantial portions of the WilmaGate. 

   The WilmaGate shall not be sub-licenced either commercially or free of
   charge, as is or in part or as a part of any other package. 

   The licensee agrees to properly credit WilmaGate and its copyright 
   owners in any technical, scientific or divulgative publication, 
   whatever publishing means are used, that is based entirely or in part 
   on WilmaGate use, as well as in any booklet, web-site or brochure that 
   advertise services based on WilmaGate use. 

   If the WilmaGate is modified, the copyright owner shall be notified.

   THE WilmaGate IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
   OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
   IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
   CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
   TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
   SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 

##############################################################################*/

#ifndef _TIMERS__H
#define _TIMERS__H

#include "exceptions.h"

// Commented in addresses.h
namespace Common {

	////////////////////////////////////////////////////////////////////////
	///
	/// \brief Timer listener class
	///
	class TimerListener {
	public:
		///
		/// \brief Virtual destructor.
		///
		virtual ~TimerListener (void) {}
		///
		/// \brief Timer tick callback.
		///
		/// \return True to continue timing, false to stop.
		///
		virtual bool timerTick (void) = 0;
	};

	//////////////////////////////////////////////////////////
	///
	/// \brief Coarse timing class
	///
	/// Pauses are in seconds and are obtained by the sleep
	/// function without any correction.
	///
	class CoarseTimer {
		friend void *timerThread (void*);
	private:
		///
		/// \brief Timer period (argument of sleep in the thread).
		///
		int period;
		///
		/// \timer Pointer to the callback object.
		///
		TimerListener *listener;
		///
		/// \timer ID of the looping thread.
		///
		pthread_t threadID;
	public:
		///
		/// \brief Empty constructor; all members are initialized to null values.
		///
		CoarseTimer ();
		///
		/// \brief Register the listening object.
		///
		/// The listening object must provide the appropriate callback method(see Common::TimerListener)
		///
		/// \param l Pointer to the listening object.
		///
		void registerListener (TimerListener *l);
		///
		/// \brief Start timing.
		///
		/// \param p Initial timing period.
		///
		void start (int p);
		///
		/// \brief Stop timing.
		///
		void stop (void);
		///
		/// \brief Wait until the timing thread is over (timer callback answers false).
		///
		void wait (void);
		///
		/// \brief Set the period of current timing.
		///
		/// \param p Period in seconds.
		///
		/// \remark Works only for running timers, otherwise the start method will
		/// overwrite the period value.
		///
		void setPeriod (int p);
	};

} // namespace Common

#endif
