///
/// \file authentication.h
///	\author Danilo Severina
///
///	$Id: authentication.h,v 1.2 2005/06/16 13:22:22 cvs Exp $
///
/// \brief Manage the handshake between remote provider authenticator and %Gatekeeper.
/// 
///	The file contains the class that allows th communication between the GateKeeeper 
///	and remote provider. The object detects the packets recived by a remote provider,
///	process it and does the proper actions (sends packets, modifies the Status Table, 
///	changes the token or updates its lease time...).
///

/*##############################################################################
   
   Developed by NetMob Group, University of Trento, Italy

   University of Trento
   DIT - Department of Informatics and Telecommunications 
   Computer Networks and Mobility Research Program
   http://netmob.unitn.it/ 
   
   This file is part of the WilmaGate system, created as part
   of the WILMA Project, sponsored by the Autonomous
   Province of Trento and participated by ITC-irst, the University
   of Trento and Alpikom Spa. More info at

     http://www.wilmaproject.org/		

   To download the project:
      - in tarball format
        http://netmob.unitn.it/wilmagate.html
      - connect to CVS repository
        :pserver:cvs@rtm.unitn.it:/home/cvsroot
   In both previous cases, the download is password protected. To obtain
   the password you must send a mail to 

      wilmagate@dit.unitn.it

################################################################################

   The WilmaGate Licence of Use


   Copyright (c) 2003-2005
                - Universita` di Trento,
                  Dipartimento di Informatica e Telecomunicazioni (DIT)
                  Computer Networks and Mobility Research Program


   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (hereinafter
   the "WilmaGate") directly from the copyright owner, to use the WilmaGate
   without restriction of purpose or time, including without limitation the
   rights to use, copy, modify, and merge the WilmaGate, subject to the
   following conditions:

   The above copyright notice and this permission notice shall be included
   in all copies or substantial portions of the WilmaGate. 

   The WilmaGate shall not be sub-licenced either commercially or free of
   charge, as is or in part or as a part of any other package. 

   The licensee agrees to properly credit WilmaGate and its copyright 
   owners in any technical, scientific or divulgative publication, 
   whatever publishing means are used, that is based entirely or in part 
   on WilmaGate use, as well as in any booklet, web-site or brochure that 
   advertise services based on WilmaGate use. 

   If the WilmaGate is modified, the copyright owner shall be notified.

   THE WilmaGate IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
   OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
   IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
   CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
   TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
   SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 

##############################################################################*/

#ifndef _AUTHENTICATION_
#define _AUTHENTICATION_

#include <iostream>
#include "../Common/addresses.h"
//#include "../Common/packets.h"
#include "../Common/socket.h"
#include "../Common/exceptions.h"
#include "auth_statustable_interface.h"
#include "providerlist_interface.h"

using namespace std;
using namespace Common;


namespace Gatekeeper {

///
/// \brief Manage the handshake between remote provider authenticator and %Gatekeeper.
/// 
///	The class contains the definitions of variable and method used by
///	the Authenticator. This object senses its interface with the network and
///	waits for packets from remote provider.
///	When it receives a packet it can be a packet to authentication of a user
///	or to renew the lease of the user, when his lease time is expired.
///	In both case it must contacts the ProviderList to check if the authentication
///	provider is in the list of allowed provider and then renews the lease time of the 
///	user if the parameters in the requested form are correct.
///
class Authentication :	public ConnectionListener, 
						public PacketListener{
private:

	///
	/// \brief Refence to the interface between %Authenticator and %StatusTable.
	/// 
	/// This private element is a pointer to the interface between the
	///	Authentication class
	///	and the StatusTable class.
	///
	AuthStatusTableInterface &asti;

	///
	/// \brief Reference to the interface between %Authenticator and %ProviderList.
	/// 
	/// This private element is a reference to the interface between this class
	///	and the ProviderList class. Really the variable that is passed by reference to the
	///	constructor is the ProviderList object.
	///
	ProviderListInterface &pli;

	///
	///	\brief TCP socket for incoming calling.
	///
	TCPSocket skTCPin;

	///
	///	\brief Array of ongoing connections
	///
	TCPSocket *tcpconn[256];

	///
	///	\brief Authorization renewal time.
	///
	const int renewalTime;

public:

	///
	/// \brief Constructor. Inizialise reference to used interfaces.
	/// 
	///	The constructor sets the private variables asti as reference to the Status
	///	Table object and pli to the ProviderList object.
	/// \param authStatusTableInterfaceObject The reference to the StatusTable object.
	///	\param providerListInterfaceObject The reference to the ProviderList object.
	///	\param listenIP Ip address of the local machine for UDP socket.
	///	\param listenPort Port address of the local port for UDP socket.
	///	\param ren Authoriztion renewal time.
	///
	Authentication (AuthStatusTableInterface &authStatusTableInterfaceObject,
					ProviderListInterface &providerListInterfaceObject,
					IPv4Address listenIP, uint16_t listenPort, int ren)
			throw (WilmaGateException);
	///
	///	\brief Virtual destructor.
	///
	virtual ~Authentication();


	///
	/// \brief Called when a packet has been received.
	///
	/// \param message Pointer to the packet.
	/// \param size Size of the packet.
	/// \param socketID ID of the calling socket (see Common::UDPSocket::registerListener for details).
	/// \param senderIP IP address of the packet sender.
	/// \param senderPort Transport layer port of the packet sender.
	///
	/// \return Action to be performed by the socket thread.
	///
	SuggestedAction receivedPacket (uint8_t *message, int size, int socketID,
								IPv4Address senderIP, uint16_t senderPort);

	///
	/// \brief Incoming connection.
	///
	/// This method is called when the passive TCP socket receives and accepts an incoming
	/// TCP connection.
	///
	/// \param s Pointer to the newly created TCP socket handling the new connection.
	/// \param socketID ID of the passive socket performing the callback.
	///
	/// \return Action to be performed by the calling thread.
	///
	SuggestedAction incomingConnection (TCPSocket *s, int socketID);


	///
	/// \brief Called in case of failure in the socket thread.
	///
	/// \param socketID ID of the calling socket (see Common::UDPSocket::registerListener for details).
	///
	/// \return Action to be performed by the socket thread.
	///
	SuggestedAction failed (int socketID);

	///
	///	\brief Start listening for TCP connection.
	///
	inline void start (void) {this->skTCPin.startListening();};

	///
	///	\brief Stop listening for TCP connection.
	///
	inline void stop (void) {this->skTCPin.stopListening();};

	
	//function to send packet to the network (to the remote provier)

};

}

#endif


