///
/// \file capwap_plugin_A.h
///	\author Nicolo Pernigo
///
/// \brief %Plugin to manage Discovery and Join Request packets.
/// 
/// The file contains the definition of elements and methods to process Discovery and Join Request packets
///	other packets are processed by capwap_plugin_B.
///
/*##############################################################################
   
   Developed by NetMob Group, University of Trento, Italy

   University of Trento
   DIT - Department of Informatics and Telecommunications 
   Computer Networks and Mobility Research Program
   http://netmob.unitn.it/ 
   
   This file is part of the WilmaGate system, created as part
   of the WILMA Project, sponsored by the Autonomous
   Province of Trento and participated by ITC-irst, the University
   of Trento and Alpikom Spa. More info at

     http://www.wilmaproject.org/		

   To download the project:
      - in tarball format
        http://netmob.unitn.it/wilmagate.html
      - connect to CVS repository
        :pserver:cvs@rtm.unitn.it:/home/cvsroot
   In both previous cases, the download is password protected. To obtain
   the password you must send a mail to 

      wilmagate@dit.unitn.it

################################################################################

   The WilmaGate Licence of Use


   Copyright (c) 2003-2005
                - Universita` di Trento,
                  Dipartimento di Informatica e Telecomunicazioni (DIT)
                  Computer Networks and Mobility Research Program


   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (hereinafter
   the "WilmaGate") directly from the copyright owner, to use the WilmaGate
   without restriction of purpose or time, including without limitation the
   rights to use, copy, modify, and merge the WilmaGate, subject to the
   following conditions:

   The above copyright notice and this permission notice shall be included
   in all copies or substantial portions of the WilmaGate. 

   The WilmaGate shall not be sub-licenced either commercially or free of
   charge, as is or in part or as a part of any other package. 

   The licensee agrees to properly credit WilmaGate and its copyright 
   owners in any technical, scientific or divulgative publication, 
   whatever publishing means are used, that is based entirely or in part 
   on WilmaGate use, as well as in any booklet, web-site or brochure that 
   advertise services based on WilmaGate use. 

   If the WilmaGate is modified, the copyright owner shall be notified.

   THE WilmaGate IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
   OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
   IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
   CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
   TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
   SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 

##############################################################################*/

#ifndef _CAPWAP_PLUGIN_A
#define _CAPWAP_PLUGIN_A

#include <bitset>

#include <iostream>
#include <string>
#include <ctime>

extern "C" {
	#include <inttypes.h>
}

#include "plugin.h"
#include "capwap_plugin_interface.h"

#define ETHERMAXSIZE 10000
#define IPv4 69
#define TTL 64

using namespace std;

namespace Gatekeeper {

/// \brief %Plugin to manage Discovery and Join Request packets.
/// 
/// The file contains the definition of elements and methods to process Discovery and Join Request packets
///	other packets are processed by capwap_plugin_B.
///	The class must have only one public function: the processPacket() function
///	because it must be invoked by the Dispatcher. Obviusly it has also a constructr
///	that inzialise the private variable.
///

class CapwapPluginA : public Plugin {

private:

///
	/// \brief IP address of the GateKeeper.
	/// 
	/// This private element is the variable that contains the IP address
	///	of the GateKeeper.
	///
	const IPv4Address gkIP;

	///
	/// \brief IP address of the GateWay.
	/// 
	/// This private element is the variable that contains the IP address
	///	of the GateWay.
	///
	const IPv4Address gwIP;

	///
	/// \brief Domain name;
	///
	const string domain;
	
	
	int inudp;
	int outudp;
	
	///
	/// \brief Reference to interface between Plugins and %StatusTable.
	/// 
	/// This private element is a pointer to the interface between the Plugin class
	///	and the Status Table class.
	///
	PluginStatusTableInterface &psti;

	///
	/// \brief Reference to interface between Plugins and %WTPTable.
	/// 
	/// This private element is a pointer to the interface between the Plugin class
	///	and the WTP Table class.
	///
	WTPTableInterface &pwtpti;

	///
	/// \brief Check if the received packet is a CAPWAP packet, only Discovery Request e Join Request are processed.
	/// 
	/// The private function checks if the received frame is a CAPWAP packet.
	/// If the packet is a CAPWAP packet, it checks if the frame is a Discovery
	/// Request Join Request, all other packets are processed by capwap_plugin_B
	/// \param ethf Incoming Ethernet frame
	///	\return CAPWAPMessage::DISCOVERYREQUEST if the packet is a CAPWAP Discovery Request packet.
	///	\return CAPWAPMessage::JOINREQUEST if the packet is a CAPWAP Join Request packet.
	static CAPWAPMessage::MessageTypes check(EthernetFrame *ethf);

//-------------------------------------------------------------------------------------------------START Message Elemnt
	///
	/// \brief Set the AC Descriptor Message Element.
	///
	void setACDescriptor (uint8_t** data, int* len);

	///
	/// \brief Set the AC Name  Message Element.
	///
	void setACName (uint8_t** data, int* len);

	///
	/// \brief Set the Result Code Message Element.
	///
	void setResultCode (uint8_t** data, int* len , int code);

	///
	/// \brief Set the ession ID Message Element.
	///
	void setSessionID (uint8_t** data, int* len, uint32_t SID);
//-------------------------------------------------------------------------------------------------END Message Elemnt
//-------------------------------------------------------------------------------------------------START CAPWAP CONTROL PCK
	///
	/// \brief Set the frames DISCOVERY RESPONSE.
	///
	/// The private function sets the array of EthernetFrames with the packets
	/// that the Dispatcher should send to the network.
	///
	void setDISCOVERYRESPONSE(EthernetFrame* ethf, EthernetFrame* e);

	///
	/// \brief Set the frames JOIN RESPONSE.
	///
	/// The private function sets the array of EthernetFrames with the packets
	/// that the Dispatcher should send to the network.
	///
	void setJOINRESPONSE(EthernetFrame* ethf, EthernetFrame* e, uint32_t code, uint32_t SID);
//-------------------------------------------------------------------------------------------------END CAPWAP CONTROL PCK

	///
	/// \brief Set all fields of CAPWAP HEADER with Default values of this implementation.
	///
	void setCAPWAPHdr(CAPWAPMessage *m);

	///
	/// \brief Set Version Field in the CAPWAP HEADER.
	///
	void setVersion(CAPWAPMessage *m, uint8_t Version);

	///
	/// \brief Set Radio ID Field in the CAPWAP HEADER.
	///
	void setRID(CAPWAPMessage *m, uint8_t RID);

	///
	/// \brief Set HLEN Field in the CAPWAP HEADER.
	///
	void setHLEN(CAPWAPMessage *m, uint8_t HLEN);

	///
	/// \brief Set WBID Field in the CAPWAP HEADER.
	///
	void setWBID(CAPWAPMessage *m, uint8_t WBID);

	///
	/// \brief Set T Field in the CAPWAP HEADER.
	///
	void setT(CAPWAPMessage *m, uint8_t T);

	///
	/// \brief Set F Field in the CAPWAP HEADER.
	///
	void setF(CAPWAPMessage *m, uint8_t F);

	///
	/// \brief Set L Field in the CAPWAP HEADER.
	///
	void setL(CAPWAPMessage *m, uint8_t L);

	///
	/// \brief Set W Field in the CAPWAP HEADER.
	///
	void setW(CAPWAPMessage *m, uint8_t W);

	///
	/// \brief Set M Field in the CAPWAP HEADER.
	///
	void setM(CAPWAPMessage *m, uint8_t M);

	///
	/// \brief Set Flags Field in the CAPWAP HEADER.
	///
	void setFlags(CAPWAPMessage *m, uint8_t Flags);

	///
	/// \brief Set FragmentId Field in the CAPWAP HEADER.
	///
	void setFragmentId(CAPWAPMessage *m, uint16_t FragmentId);

	///
	/// \brief Set FragmentOffset Field in the CAPWAP HEADER.
	///
	void setFragmentOffset(CAPWAPMessage *m, uint16_t FragmentOffset);

	///
	/// \brief Set Rsvd Field in the CAPWAP HEADER.
	///
	void setRsvd(CAPWAPMessage *m, uint8_t Rsvd);

	///
	/// \brief Get Session ID.
	///
	uint32_t getSID(uint8_t *me, uint16_t maxlen);

	///
	/// \brief Get WTP Name.
	///
	string getWTPName(uint8_t *me, uint16_t maxlen);

	///
	/// \brief Get Location.
	///
	string getLocation(uint8_t *me, uint16_t maxlen);

	///
	/// \brief Add WTP on WTP Table with Join Request Information.
	///
	bool AddJoinRequestInformation(IPv4Address ip,EthernetMACAddress mac,uint32_t newsid);
public:

	///
	///	\brief Constructor.
	///
	///	Costructor of the class: it inizialises the references of the interfaces and
	///	the paraemters of the DHCP server (the GateKeeper itself).
	///	\param pluginStatusTableInterfaceObject used to inizialise the private reference
	///	to the interface between plugins and status table in the Plugin class.
	///	\param pluginWTPTableInterfaceObject used to inizialise the private reference
	///	to the interface between plugins and WTP table in the Plugin class.
	///	\param gateIpAddress IP address of the WilmaGate that, in this case,
	///	works as a DHCP server.
	///	\param wayIpAddress is the IP address of the gateway (to be used as gateway router by the client)
	///	\param domainName name of the domain.
	///
	CapwapPluginA(	PluginStatusTableInterface &pluginStatusTableInterfaceObject, 
			WTPTableInterface &WTPTableInterfaceObject,
			IPv4Address gateIpAddress,
			IPv4Address wayIpAddress,
			const char *domainName);

	///
	/// \brief Virtual destructor
	///
	~CapwapPluginA (void);

	///
	///	\brief Process and manage the received packet.
	///
	///	The function is the implemetation of pure virtual function defined in the
	///	Plugin class.
	///	The function
	///	receives a packet and processes it to know if it can be processed.
	///	In affermetive case, the function processes it: it contact the status table
	/// and AP table and defines the packets that must send to user.
	///	\param pointerToPacket is a pointer to the packet for
	/// analysing
	///	\param instructionVector is a vector of EthernetFrame elements and 
	///	stores the frame that the Dispatcher sends to the users.
	///	\param sense Direction in which the packet in the corresponding  instructionVector element is expected to be sent.
	///	\return The function returns a boolean variable: TRUE if the packet that is passed 
	///	by parameter can be processed by the called plugin sub-class, FALSE if the 
	/// packet cannot be processed.
	///
	bool processPacket (EthernetFrame *pointerToPacket, EthernetFrame* instructionVector[], PacketSense sense[]);

	//bool SendCONFIGURATIONUPDATEREQUEST(IPv4Address newwtp);
};

}

#endif
