///
/// \file dispatcher.h
///	\author Danilo Severina
///
/// \brief Manage the packets that are received by unauthorized user.
///
///	The file contains the definition of function of dispatcher element.
///	This element senses the interface and receives packet from it. It doesn't know
///	neither the information in the packet nor the protocol used to send the packet.
///	The dispatcher receives the packet and interrogates the different plugin to know 
///	which of the can process the packet; the interrogation is similar to a 
///	daisy chain or polling process. 
///	The request is sent to all the %Plugin
///	classes and only one of them answers affirmatively and processes the packet.
///	The %Plugin that processes the packet sends information to the dispatcher
///	about the actions that it has to do: basically the dispatcher should send 
///	some packets to the user or to the network.
///
///	The disaptcher does not analyse the packet: when it receives a packet, it
///	starts a polling procedure to look for the plugin that can analyse the packet.
///
///	$Id: dispatcher.h,v 1.3 2006/03/30 13:35:23 cvs Exp $
///

/*##############################################################################
   
   Developed by NetMob Group, University of Trento, Italy

   University of Trento
   DIT - Department of Informatics and Telecommunications 
   Computer Networks and Mobility Research Program
   http://netmob.unitn.it/ 
   
   This file is part of the WilmaGate system, created as part
   of the WILMA Project, sponsored by the Autonomous
   Province of Trento and participated by ITC-irst, the University
   of Trento and Alpikom Spa. More info at

     http://www.wilmaproject.org/		

   To download the project:
      - in tarball format
        http://netmob.unitn.it/wilmagate.html
      - connect to CVS repository
        :pserver:cvs@rtm.unitn.it:/home/cvsroot
   In both previous cases, the download is password protected. To obtain
   the password you must send a mail to 

      wilmagate@dit.unitn.it

################################################################################

   The WilmaGate Licence of Use


   Copyright (c) 2003-2005
                - Universita` di Trento,
                  Dipartimento di Informatica e Telecomunicazioni (DIT)
                  Computer Networks and Mobility Research Program


   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (hereinafter
   the "WilmaGate") directly from the copyright owner, to use the WilmaGate
   without restriction of purpose or time, including without limitation the
   rights to use, copy, modify, and merge the WilmaGate, subject to the
   following conditions:

   The above copyright notice and this permission notice shall be included
   in all copies or substantial portions of the WilmaGate. 

   The WilmaGate shall not be sub-licenced either commercially or free of
   charge, as is or in part or as a part of any other package. 

   The licensee agrees to properly credit WilmaGate and its copyright 
   owners in any technical, scientific or divulgative publication, 
   whatever publishing means are used, that is based entirely or in part 
   on WilmaGate use, as well as in any booklet, web-site or brochure that 
   advertise services based on WilmaGate use. 

   If the WilmaGate is modified, the copyright owner shall be notified.

   THE WilmaGate IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
   OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
   IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
   CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
   TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
   SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 

##############################################################################*/

#ifndef _DISPATCHER_
#define _DISPATCHER_

#include <iostream>
#include "../Common/packets.h"
#include "plugin.h"
#include "../Common/socket.h"
#include "../Common/packetproto.h"

//For DHCP relay plugin
#include "relay_listener.h"
#include "relay_plugin.h"
#include "capwap_plugin_B.h"


using namespace std;
using namespace Common;

///
/// \brief Gatekeeper-related entities.
///
namespace Gatekeeper {

///
/// \brief Manage the packets that are received by unauthorized users.
///
///	The class contain the definition of the Dispatcher element: it
///	senses the interface and receives from it the packets
///	that arrivers from user that are not allowed to browse. The %Dispatcher
///	doesn't know
///	neither the information in the packet nor the protocol used to send the packet
///	because it doesn't analyse the packet:
///	it receives the packet and interrogates the different plugin to know 
///	which of them can process it; the interrogation is similar to a 
///	daisy chain or polling process. 
///	The request is sent to all the %Plugin
///	classes and only one of them answers affirmatively and processes the packet.
///	The request to the %Plugin is done using an array tha contains a reference to
///	the %Plugins: the array is a private element of the %Dispatcher, but 
///	the main() function can introduce %Plugin element using the public function 
///	insertPlugin(). When this function is invoked, the parameter is a sub-plugin
///	object, but the %Dispatcher considers it as a generic Plugin element.
///
///	The %Plugin that processes the packet sends information to the dispatcher
///	about the actions that it has to do: basically the dispatcher should send 
///	some packets to the user or to the network. The packets (EtehrnetFrame) that 
///	the %Dispatcher must send are in the private vector frames (defined in this class).
///
///	The %Dispatcher class is simple: it receives and sends packets, even if it
///	doesn't know what the packet contains and what it means.
///
/// A more elaborated description of class Dispatcher, methods and elements follows.
///
class Dispatcher : public PacketListener ,
		   public RelayListener { //For DHCP relay plugin


private:
    //For DHCP relay plugin 
   	RelayPlugin* pToRelay;
	
	//pointer to capwap Plugin B 
	CapwapPluginB* cpb;

	///
	///	\brief Maximum number of Plugin element in the array.
	/// 
	static const int MAX_PLUGIN = 10;

	///
	/// \brief Pointers to the registrated Plugins.
	///
	/// Private array that contains pointers to the 
	///	plugins. The class dispatcher inlcudes only the pure virtual class Plugin,
	///	and doesn't know about the presence of different Plugin sub-class that 
	///	analyse packets that contains different information and are based on different
	///	protocol.
	///	For this class each element that are pointed by the array is a
	///	generic plugin, but when the method to process the packet is invoked, it
	///	has to be associated with a Plugin sub-class.
	///
	Plugin *element[MAX_PLUGIN];

	///
	/// \brief The number of %Plugin sub-class elements in the private array.
	///
	int l;

	///
	/// \brief maximum number of ethernet frames in an answer sequence.
    ///
    static const int MAXFRAMES = 10;

	///
    /// \brief Ethernet frame answer sequence.
	///
	///	The private element \e ethernetPacketToSend is an array of
	///	EthernetFrame elements and it contains the packets that the dispatcher
	///	must send to the user or to the network after the packet process.
	///	The vector is built by the plugin that processes the packet.
    /// First unused frame will contain 0x0000U in ethernet protocol field.
    ///
    EthernetFrame* frames[MAXFRAMES];

	//MB
	///
	/// \brief Direction of the packet.
	///
	/// Coupled with Dispatcher::frames, says whether each frame built by the Plugin
	/// is intended to be returned to the sender or forwarded to the destination.
	///
	PacketSense sense[MAXFRAMES];

	///
	///	\brief Socket for UDP connection.
	///
	UDPSocket skudp;


public:

	///
	///	\brief Constructor. Inizilise Plugin and EthernetFrames arrays.
	///
	/// The constructor of the class set to zero the length of the array of Plugin
	///	elements. The array is not initilized because the use of the index that
	///	contains the length of the array allows to check only the inserted Plugin
	///	during the forwarding of the requests of packet processes.
	///	The first element of the array of pointer to EthernetFrame is set to NULL.
	///	In the array the first element at NULL, means that the previous packet is the last one
	///	to send.
	///	\param listenIP Ip address of the local machine for UDP socket
	///	\param listenPort Port address of the local port for UDP socket
	///	\param destIP Ip address of the remote machine for UDP socket
	///	\param destPort Port address of the remote port for UDP socket
	///
	Dispatcher(
     RelayPlugin* pointerToRelay, //For DHCP relay plugin
	 IPv4Address listenIP, uint16_t listenPort, 
	 IPv4Address destIP, uint16_t destPort);

	 Dispatcher(
     RelayPlugin* pointerToRelay, //For DHCP relay plugin
	CapwapPluginB* poitercpb, //For CAPWAP Plugin B     
	 IPv4Address listenIP, uint16_t listenPort, 
	 IPv4Address destIP, uint16_t destPort);
	///
	///	\brief Destructor.
	///
	virtual ~Dispatcher();

	///
	///	\brief Insert a %Plugin in the private list.
	///
	///	The function allows to insert a new Plugin pointer in the array.
	/// Also in this case the function considers the element Plugin as a 
	///	 generic Plugin element and doesn't know the right Plugin sub-class.
	///	\param pointerToPluginElement is pointer to the plugin to insert.
	///
	void insertPlugin(Plugin *pointerToPluginElement);

	///
	///	\brief Invoked when a new Ethernet Frame is received by the %Dispatcher.
	///
	///	The function inquires the Plugins to process the packet. It inquires all the %Plugin
	///	until one of them anwsers affirmatively: in this case the packet has processed and the 
	///	vector of %EthernetFrame contains the packets to send to the network and/or user.
	///	\param message Pointer to the received framefrom the Gatway.
	///	\param size Size of the message.
	///	\param socketID Identifier of the socket (see Common::UDPSocket::registerListener for details).
	///	\param senderIP IP address of the sender of the packet.
	///	\param senderPort Port number of the sender of the packet.
	///	\return Action to be performed by the socket thread.
	///	
	virtual SuggestedAction receivedPacket (uint8_t *message, int size, int socketID, IPv4Address senderIP, uint16_t senderPort);

	///
	/// \brief Called in case of failure in the socket thread.
	///
	/// \param socketID ID of the calling socket (see Common::UDPSocket::registerListener for details).
	/// \return Action to be performed by the socket thread.
	///
	virtual SuggestedAction failed (int socketID);

	///
	///	\brief Build the packets to send to the user and/or to the network.
	///
	///	The function sends packets towards the interface for remote hosts.
	///	The packets that should be sent are received by the Plugin sub-class that 
	///	analysed the recevied packet and informs the Dispatcher which actions it has 
	///	to do. 
	///
	/// \param e Original Ethernet frame.
	/// \param intf Gateway interface that shall process the packet.
	///
	void sendPacket(EthernetFrame *e, int intf);
		//forse private invia packets. ma non deve esssere invocata da classi esterne.

	inline void printL() {cout<<l; return;};

	//MB
	///
	/// \brief Start receiving
	///
	inline void start (void) { skudp.startReceiving (); }

	//MB
	///
	/// \brief Stop receiving
	///
	inline void stop (void) { skudp.stopReceiving (); }

	//MB
	///
	/// \brief Hang waiting for receiving thread completion.
	///
	inline void wait (void) { skudp.wait (); }


	SocketListener::SuggestedAction sendResponseFrame(EthernetFrame* ethf, int size);

};

}

#endif
