///
/// \file interceptor_plugin.h
///	\author Albino Scalet
///
///

/*##############################################################################
   
   Developed by NetMob Group, University of Trento, Italy

   University of Trento
   DIT - Department of Informatics and Telecommunications 
   Computer Networks and Mobility Research Program
   http://netmob.unitn.it/ 
   
   This file is part of the WilmaGate system, created as part
   of the WILMA Project, sponsored by the Autonomous
   Province of Trento and participated by ITC-irst, the University
   of Trento and Alpikom Spa. More info at

     http://www.wilmaproject.org/		

   To download the project:
      - in tarball format
        http://netmob.unitn.it/wilmagate.html
      - connect to CVS repository
        :pserver:cvs@rtm.unitn.it:/home/cvsroot
   In both previous cases, the download is password protected. To obtain
   the password you must send a mail to 

      wilmagate@dit.unitn.it

################################################################################

   The WilmaGate Licence of Use


   Copyright (c) 2003-2005
                - Universita` di Trento,
                  Dipartimento di Informatica e Telecomunicazioni (DIT)
                  Computer Networks and Mobility Research Program


   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (hereinafter
   the "WilmaGate") directly from the copyright owner, to use the WilmaGate
   without restriction of purpose or time, including without limitation the
   rights to use, copy, modify, and merge the WilmaGate, subject to the
   following conditions:


   The above copyright notice and this permission notice shall be included
   in all copies or substantial portions of the WilmaGate. 

   The WilmaGate shall not be sub-licenced either commercially or free of
   charge, as is or in part or as a part of any other package. 

   The licensee agrees to properly credit WilmaGate and its copyright 
   owners in any technical, scientific or divulgative publication, 
   whatever publishing means are used, that is based entirely or in part 
   on WilmaGate use, as well as in any booklet, web-site or brochure that 
   advertise services based on WilmaGate use. 

   If the WilmaGate is modified, the copyright owner shall be notified.

   THE WilmaGate IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
   OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
   IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
   CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
   TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
   SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 

##############################################################################*/

#ifndef _INTERCEPTOR_PLUGIN_
#define _INTERCEPTOR_PLUGIN_

#include <iostream>
#include <string>
#include <ctime>

extern "C" {
	#include <inttypes.h>
}

#include "plugin.h"

using namespace std;

namespace Gatekeeper {

///
/// \brief %Plugin to manage DHCP packets.
/// 
/// The class defines elements and methods to process packets
///	that use DHCP protocol.
///	The classs must have only one public function: the processPacket() function
///	because it must be invoked by the Dispatcher. Obviusly it has also a constructr
///	that inzialise the private variable.
///
///	The private variables gateIP and gateMAc need to generate the EthernetFrame that
///	the %Dispatcher will send: the packets must contains the MAC and IP address of
///	the DHCP server, that in this case is the same WilmaGate.
///	The private varible psti is needed by the %DhcpPlugin because the class must contact
///	the Status Table to check the status of the user identified by the parameters
///	in the received packets.
///
/// The %DhcpPlugin can receive TWO kind of packets:
///		- a DHCP-Discovery packet: when the user connets itself to the network the
///	first time it has not an IP address (static IP addesses are not allowed) so it 
///	requests an address to the network sending a packet. The 
///	DHCP-Discovery packet is a broadcast packet (destination address 255.255.255.255) and the
///	source Ip is 0.0.0.0: the only useful information in the packet is the MAC address of the 
///	user. The DHCP server answers with a DHCP-Offer packet that contains the offered IP address
///	to the user.
///		- a DHCP-Request packet: when the user receives a DHCP-Offer packet, if it accepts the offered
///	IP (there is no reason that the user should refuse the offer if in the system there is only one
///	DHCP server), it sends a DHCP-Request packet. When it is received by the DHCP server, the previous
///	offered Ip is considered associated with the user and a DHCP-Acknoledge packet is sent to the user.
///
///	A more detailed descritption of actions that the %DhcpPlugin does when receive one of the two kind 
///	of packets described above follows.
///
///	\par DHCP-Discovery.
///	When the %DhcpPlugin receives this kind of packets, it contacts the Status Table 
///	to obtain a IP address for sending to the user. The first thing it must do is to look for in the
///	Status Table if the MAC address of the user is just authenticated in the past (searchIP()): if the MAC address
///	is in the Status Table and the Status of IP address associated with this user is FREE, this IP is
///	the offered address to send to the user. If the MAC address is not in the Status Table an IP
///	address is required (getIP()). Then the staus of IP must be changed from FREE to FIRST_SEEN (setStatus())
///	and finally the expiration time of the IP address must be set (setExpIP). In this case the lease
///	has a short expiration time.
///	The offered Ip address is used to generate the %EthernetFrames that the %Dispatcher 
///	will send. Three EtherentFrames are generated:
///	- the first one is the DHCP-Offer packet and is send to the user to offer the IP address (this one
///	contains also the information about the MAC and IP address of the WilmaGate, that in this moment
///	is the DHCP server.
///	- the second and the third are necessary to stop the DHCP-Disocvery process, started by the user;
///	one packet is sent to the user and the other one is sent to the network.
///
///	\par DHCP-Request.
///	When the %DhcpPlugin receives this kind of packets, it contacts the Status Table to notify
///	to the Status Table that the user has been accepted the offered IP address: the StatusTable is
///	forced to change the status of IP address from FIRST_SEEN to TEMORARY_LEASE (setStatus()).
///	Then the token that is generated by the StatuaTable for the IP address is gotten (getToken()).
///	the last thing is the generation of a %EthernetFrame that the %Dispatcher will send as a DHCP-Aknowledge
///	packet.
///	
///	In both previous cases, when the processPacket() function is invoked by the Dispatcher, it 
///	returns a true boolean variable.
///
class interceptorPlugin : public Plugin {

private:

	
	///
	/// \brief Reference to interface between Plugins and %StatusTable.
	/// 
	/// This private element is a pointer to the interface between the Plugin class
	///	and the Status Table class.
	///
	PluginStatusTableInterface &psti;

  IPv4Address inIP;
  
  IPv4Address netmask;
  
  int shortLease;

public:

	///
	///	\brief Constructor.
	///
	///	Costructor of the class: it inizialises the references of the interfaces and
	///	the paraemters of the DHCP server (the GateKeeper itself).
	///	\param pluginStatusTableInterfaceObject used to inizialise the private reference
	///	to the interface between plugins and status table in the Plugin class.
	///	\param gateIpAddress IP address of the WilmaGate that, in this case,
	///	works as a DHCP server.
	///	\param wayIpAddress is the IP address of the gateway (to be used as gateway router by the client)
	///	\param DNSIPAddress IP address of the proposed DNS server.
	///	\param clientNetmask Netmask of the proposed IP address.
	///	\param domainName name of the domain.
	///	\param wfile URL of the WPAD file (empty string if no file).
	///	\param shortLease Short lease time for unauthorized clients
	///	\param longLease Long lease time for authorized clients
	///

	interceptorPlugin(PluginStatusTableInterface &pluginStatusTableInterfaceObject,   IPv4Address innerInterfaceIP,
  IPv4Address netm, int sl);

	///
	/// \brief Virtual destructor
	///
	virtual ~interceptorPlugin (void);

	///
	///	\brief Process and manage the received packet.
	///
	///	The function is the implemetation of pure virtual function defined in the

	///	Plugin class.
	///	The function
	///	receives a packet and processes it to know if it can be processed.
	///	In affermetive case, the function processes it: it modifies the status table
	/// and defines the packets that must send to user.
	///	\param pointerToPacket is a pointer to the packet for
	/// analysing
	///	\param instructionVector is a vector of EthernetFrame elements and 
	///	stores the frame that the Dispatcher sends to the users.
	///	\param sense Direction in which the packet in the corresponding  instructionVector element is expected to be sent.
	///	\return The function returns a boolean variable: TRUE if the packet that is passed 
	///	by parameter can be processed by the called plugin sub-class, FALSE if the 
	/// packet cannot be processed.
	///
	bool processPacket (EthernetFrame *pointerToPacket, EthernetFrame* instructionVector[], PacketSense sense[]);

	//inline void printEthFrame(EthernetFrame* ethf);

};



}

#endif
