///
/// \file plugin.h
///	\author Danilo Severina
///
///	\brief Implement the manage of received packets form unauthorized users.
///
///	This file contains the definition of a pure virtual class. It
///	contains the definition of a class with undefined function
///	and it groups different sub-classes. 
///	This class is extended by classes that process differnt kind 
///	of packet: each of them must recognise the packets that it can analyse
///	and overload tha virtual function of this class to process packets.
///	This pure virtual class have only one method (pure virtual) and does not have the 
///	respective cpp file.
///
///	$Id: plugin.h,v 1.4 2006/03/30 16:33:42 cvs Exp $
///

/*##############################################################################
   
   Developed by NetMob Group, University of Trento, Italy

   University of Trento
   DIT - Department of Informatics and Telecommunications 
   Computer Networks and Mobility Research Program
   http://netmob.unitn.it/ 
   
   This file is part of the WilmaGate system, created as part
   of the WILMA Project, sponsored by the Autonomous
   Province of Trento and participated by ITC-irst, the University
   of Trento and Alpikom Spa. More info at

     http://www.wilmaproject.org/		

   To download the project:
      - in tarball format
        http://netmob.unitn.it/wilmagate.html
      - connect to CVS repository
        :pserver:cvs@rtm.unitn.it:/home/cvsroot
   In both previous cases, the download is password protected. To obtain
   the password you must send a mail to 

      wilmagate@dit.unitn.it

################################################################################

   The WilmaGate Licence of Use


   Copyright (c) 2003-2005
                - Universita` di Trento,
                  Dipartimento di Informatica e Telecomunicazioni (DIT)
                  Computer Networks and Mobility Research Program


   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (hereinafter
   the "WilmaGate") directly from the copyright owner, to use the WilmaGate
   without restriction of purpose or time, including without limitation the
   rights to use, copy, modify, and merge the WilmaGate, subject to the
   following conditions:

   The above copyright notice and this permission notice shall be included
   in all copies or substantial portions of the WilmaGate. 

   The WilmaGate shall not be sub-licenced either commercially or free of
   charge, as is or in part or as a part of any other package. 

   The licensee agrees to properly credit WilmaGate and its copyright 
   owners in any technical, scientific or divulgative publication, 
   whatever publishing means are used, that is based entirely or in part 
   on WilmaGate use, as well as in any booklet, web-site or brochure that 
   advertise services based on WilmaGate use. 

   If the WilmaGate is modified, the copyright owner shall be notified.

   THE WilmaGate IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
   OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
   IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
   CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
   TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
   SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 

##############################################################################*/

#ifndef _PLUGIN_
#define _PLUGIN_

#include <iostream>
#include "../Common/packets.h"
#include "plugin_statustable_interface.h"
#include "../Common/addresses.h"
#include "providerlist_interface.h"
#include "../Common/packetproto.h"
#include "wtptable_interface.h"

using namespace std;
using namespace Common;

namespace Gatekeeper {

class Dispatcher;

///
///	\brief Implement the manage of received packets from unauthorized users.
///
///	This class contains the pure virtual function to analyse the packets.
/// The method processPacket() is invoked by the Dispatcher when it receives a packet
///	from its own interface.
///	The Plugin class defines the above function as pure virtual, because its implementation
///	is in the classes that derive from this (HttpPlugin, DhcpPlugin...). The implementation
///	of this functionin the different class will be different, because it must analyse
///	packet that has different informations and uses different protocols.
///	The function returns to the Disptcher a boolean value to notify a successful or
///	unsuccessful process. Moreover, if there is a successful process, the %Plugin 
///	fills the %EthernetFrame vector with the %EthernetFrame objects: each of this object
///	contains a packet that the %Dispatcehr must send to the user and/or to the network.
///	
///	This pure virtual class is an interface between the %Dispatcher and the 
///	%Plugin class; it contains only the above mentioned function and has no
///	implementation for its own function. No other elements or methods are needed in
///	the class.
///

class Plugin{

private:

	/// \brief Pointer to the dispatcher
	Dispatcher *rtl;

public:

		///
		/// \brief Virtual destructor.
		///
		virtual ~Plugin (void) {}

	///
	///	\brief Process and manage the received packets.
	///
	///	This processPacket function is a pure virtual function. It will be
	///	implemented by plugin sub-classes in different ways for right processing.
	/// Although the differents implementation, the function has the same scope: it
	///	receives a packet and processes it to know if it can be processed.
	///	In affermetive case, the function processes it: it modifies the status table
	/// and defines the packets that must send to user.
	///	\param pointerToPacket is a pointer to the packet for
	/// analysing
	///	\param instructionVector is a vector of EthernetFrame elements and 
	///	stores the frame that the Dispatcher sends to the users.
	///	\param sense Direction of the packet once it has been returned to the gateway.
	///	\return The function returns a boolean variable: TRUE if the packet that is passed 
	///	by parameter can be processed by the called plugin sub-class, FALSE if the 
	/// packet cannot be processed.
	///
	virtual bool processPacket (EthernetFrame *pointerToPacket, EthernetFrame* instructionVector[],
			PacketSense sense[]) = 0;

	/// \brief Register the dispatcher to the plugin (called by the dispatcher in response
	/// to the opposite registration)
	///
	/// \param p Pointer to the dispatcher
	///
	inline void registerDispatcher (Dispatcher* p);

};

inline void Plugin::registerDispatcher (Dispatcher* p)
{
	rtl = p;
}


}

#endif
