///
/// \file provider_list.h
///	\author Danilo Severina
///
///	\brief List of allowed providers for authentication.
///
///	The file contains the structure for storing the list of the authorized
///	provider for the authentication of the user and useful method to access 
///	to read or to write the data in the list.
///
///	$Id: provider_list.h,v 1.4 2005/12/14 16:25:32 cvs Exp $
///	

/*##############################################################################
   
   Developed by NetMob Group, University of Trento, Italy

   University of Trento
   DIT - Department of Informatics and Telecommunications 
   Computer Networks and Mobility Research Program
   http://netmob.unitn.it/ 
   
   This file is part of the WilmaGate system, created as part
   of the WILMA Project, sponsored by the Autonomous
   Province of Trento and participated by ITC-irst, the University
   of Trento and Alpikom Spa. More info at

     http://www.wilmaproject.org/		

   To download the project:
      - in tarball format
        http://netmob.unitn.it/wilmagate.html
      - connect to CVS repository
        :pserver:cvs@rtm.unitn.it:/home/cvsroot
   In both previous cases, the download is password protected. To obtain
   the password you must send a mail to 

      wilmagate@dit.unitn.it

################################################################################

   The WilmaGate Licence of Use


   Copyright (c) 2003-2005
                - Universita` di Trento,
                  Dipartimento di Informatica e Telecomunicazioni (DIT)
                  Computer Networks and Mobility Research Program


   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (hereinafter
   the "WilmaGate") directly from the copyright owner, to use the WilmaGate
   without restriction of purpose or time, including without limitation the
   rights to use, copy, modify, and merge the WilmaGate, subject to the
   following conditions:

   The above copyright notice and this permission notice shall be included
   in all copies or substantial portions of the WilmaGate. 

   The WilmaGate shall not be sub-licenced either commercially or free of
   charge, as is or in part or as a part of any other package. 

   The licensee agrees to properly credit WilmaGate and its copyright 
   owners in any technical, scientific or divulgative publication, 
   whatever publishing means are used, that is based entirely or in part 
   on WilmaGate use, as well as in any booklet, web-site or brochure that 
   advertise services based on WilmaGate use. 

   If the WilmaGate is modified, the copyright owner shall be notified.

   THE WilmaGate IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
   OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
   IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
   CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
   TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
   SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 

##############################################################################*/

#ifndef _PROVIDER_LIST_
#define _PROVIDER_LIST_


#include <iostream>
#include "providerlist_interface.h"

/*
extern "C"{
	#include "netinet/in.h"
	#include "arpa/inet.h"
	#include "inttypes.h"
}
*/

using namespace std;

namespace Gatekeeper {

///
///	\brief Maximum numer of provider that can be stored.
///
const int MAX_PROV=10;	

///
///	\brief List of providers for authentication.
///
/// The ProviderList class contains the list of the authorized
///	provider for the authentication of the user and useful method to access 
///	to read or to write the data in the list.
///	
///	The class derives from PluginProviderListInterface where some functions are
/// defined \e virtual.
///
class ProviderList : public ProviderListInterface {

private:

	///
	///	\brief Parameters that identify a single providers.
	///
	/// The stucture identifies the authorized authentication provider. Each
	///	provider is associated to a name, an IP address and some data about
	///	the identification process.
	///
	struct providerElement{
		///
		/// Name of the provider.
		///
		char* name;
		///
		/// IP address of the Provider
		///
		IPv4Address ip;
		///
		/// Domain name of the Provider
		///
		char *domain;
		///
		/// URL of the authentication provider (NULL if access provider).
		///
		char* url;
		///
		///	Prefix of allowed URLs.
		///
		char* prefix;
		// altri dati x idenitifcazione del processo 
	};


	///
	///	\brief List of the remote provider.
	///
	///	The private element is the list of possible authentication provider. The system
	///	must support different remote provider.
	///
	providerElement provider[MAX_PROV];

	///
	///	\brief Number of allowed authentication provider.
	///
	/// Private integer variable that contains
	///	the number of authentication provider that are storaged in the private array.
	///
	int l;


public:

	///
	///	\brief Constructor.
	///
	///	Constuctor. It reads parameters about the providers from a configuration file
	///	and inizialises the list of provider with these parameters.
	///
	ProviderList(void);

	///
	///	\brief Insert of a provider in the list.
	///
	///	The function inserts a new provider in the private list of the class.
	///	\param name is the name of the provider
	///	\param url is the URL address of the provider
	///	\param prefix Prefix of URLs that are allowed to pass.
	///	\param domain Domain of URLs that are allowed to pass.
	///	\param ipAddress is the IP address of the authentication provider.
	///	\return \true if the insertion is successfull, \false otherwise
	///
	bool insertProvider(const char* name, const char* url, const char* prefix, const char *domain, IPv4Address ipAddress);

	// inherited
	const char *getDomainFromName(const char* name);
	const char* getNameFromDomain(const char *domain);
	const char* getPrefixFromName(const char* name);
	const char* getPrefixFromDomain(const char *domain);
	const char* getURLFromName(const char* name);
	const char* getURLFromDomain(const char *domain);
	bool isPrefixPresent(const char* url);
	bool isDomainPresent(const char *domain);
	bool isIPPresent(const IPv4Address IP) const;
	virtual void listProviders (ostream &o);
	virtual IPv4Address getIPFromName(const char* name);
};

}

#endif



