///
/// \file providerlist_interface.h
///	\author Danilo Severina
///
///	The file contains tha definition of an interface for the ProviderList class.
///	Some classses must only read the information in the list, and should not be
///	able to modify the fields.
///
///	$Id: providerlist_interface.h,v 1.5 2005/12/14 16:25:32 cvs Exp $
///	

/*##############################################################################
   
   Developed by NetMob Group, University of Trento, Italy

   University of Trento
   DIT - Department of Informatics and Telecommunications 
   Computer Networks and Mobility Research Program
   http://netmob.unitn.it/ 
   
   This file is part of the WilmaGate system, created as part
   of the WILMA Project, sponsored by the Autonomous
   Province of Trento and participated by ITC-irst, the University
   of Trento and Alpikom Spa. More info at

     http://www.wilmaproject.org/		

   To download the project:
      - in tarball format
        http://netmob.unitn.it/wilmagate.html
      - connect to CVS repository
        :pserver:cvs@rtm.unitn.it:/home/cvsroot
   In both previous cases, the download is password protected. To obtain
   the password you must send a mail to 

      wilmagate@dit.unitn.it

################################################################################

   The WilmaGate Licence of Use


   Copyright (c) 2003-2005
                - Universita` di Trento,
                  Dipartimento di Informatica e Telecomunicazioni (DIT)
                  Computer Networks and Mobility Research Program


   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (hereinafter
   the "WilmaGate") directly from the copyright owner, to use the WilmaGate
   without restriction of purpose or time, including without limitation the
   rights to use, copy, modify, and merge the WilmaGate, subject to the
   following conditions:

   The above copyright notice and this permission notice shall be included
   in all copies or substantial portions of the WilmaGate. 

   The WilmaGate shall not be sub-licenced either commercially or free of
   charge, as is or in part or as a part of any other package. 

   The licensee agrees to properly credit WilmaGate and its copyright 
   owners in any technical, scientific or divulgative publication, 
   whatever publishing means are used, that is based entirely or in part 
   on WilmaGate use, as well as in any booklet, web-site or brochure that 
   advertise services based on WilmaGate use. 

   If the WilmaGate is modified, the copyright owner shall be notified.

   THE WilmaGate IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
   OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
   IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
   CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
   TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
   SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 

##############################################################################*/

#ifndef _PROVIDER_LIST_INTERFACE_
#define _PROVIDER_LIST_INTERFACE_


#include <iostream>
#include <fstream>
#include "../Common/addresses.h"

using namespace std;
using namespace Common;

namespace Gatekeeper {

///
///	The class is tha definition of an interface for the ProviderList class.
///	Some classses must only read the information in the list, and should not be
///	able to modify the fields: so these classes interact with the Provider List only
///	through the interface.
///	This interface allows only to read the information about the provider, e.g. the name, 
///	the URL, the IP address and other data for interacting with the remote authentication
///	provider.
///
/// The class doesn't implement the method, and they are implemented in ProviderList class.
///
class ProviderListInterface{

private:

public:

		///
		/// \brief Virtual destructor.
		///
		virtual ~ProviderListInterface (void) {}

	///
	///	\brief Get the name of a provider.
	///
	/// The function gets the name of the provider identified by its IP address.
	///	\param domain Domain of the provider.
	///	\return The name of the provider.
	///
	virtual  const char* getNameFromDomain (const char *domain) = 0;

	///
	///	\brief Get the allowed URL prefix of a provider.
	///
	///	\param name The name of the provider.
	///
	///	\return pointer to a char array, the prefix.
	///
	virtual const char* getPrefixFromName (const char* name) = 0;

	///
	///	\brief Get the allowed URL prefix of a provider.
	///
	///	\param domain The domain of the provider.
	///
	///	\return pointer to a char array, the prefix.
	///
	virtual const char* getPrefixFromDomain (const char *domain) = 0;

	///
	///	\brief Get the URL of a provider.
	///
	/// The function gets the URL of the provider identified by its name.
	///	\param name The name of the provider.
	///	\return pointer to a char array, the URL of the provider.
	///
	virtual const char* getURLFromName(const char* name) = 0;

	///
	///	\brief Get the URL of a provider.
	///
	/// The function gets the URL of the provider identified by its name.
	///	\param domain Domain of the provider.
	///	\return pointer to a char array, the URL of the provider.
	///
	virtual const char* getURLFromDomain(const char *domain) = 0;

	///
	///	\brief Check if the URL is a valid authentication provider.
	///
	/// The function checks if the URL passed by parameter is a URL of
	///	an authentication provider.
	///	\param url is the received URL
	///	\return \e true if the URL is present in the list of authentication provider (actually, a prefix of it)
	///	\e false otherwise.
	///
	virtual bool isPrefixPresent(const char* url) = 0;

	///
	///	\brief Check if the provider is in the list.
	///
	/// The function checks if the domain passed as parameter belongs to
	///	an authentication provider.
	///	\param domain is the domain (or IP address, in case of numeric domain) to search for
	///	\return \e true if the IP address is present in the list of authentication provider,
	///	\e false otherwise.
	///
	virtual bool isDomainPresent(const char *domain) = 0;

	///
	///	\brief Check if the provider's IP is in the list.
	///
	/// The function checks if the IP address passed as parameter belongs to
	///	an authentication provider.
	///	\param IP is the IP address to search for
	///	\return \e true if the IP address is present in the list of authentication provider,
	///	\e false otherwise.
	///
	virtual bool isIPPresent(const IPv4Address IP) const = 0;

	///
	/// \brief List all providers on an output stream.
	///
	virtual void listProviders (ostream &o) = 0;

	///
	///	\brief Get the IP of a provider.
	///
	///	\param name The name of the provider.
	///	\return the IP address of the provider.
	///
	virtual IPv4Address getIPFromName(const char* name) = 0;

};

}

#endif



