///
/// \file proxy_plugin.h
///	\author Mauro Brunato
///
///	\brief %Plugin to manage HTTP packet from unathorized users.
///
/// The file contains the definition of elements and methods to process packets
///	that use HTTP protocol.
///	There is the implementation of processPacket function defined in Plugin
///	class.
///
///	$Id: proxy_plugin.h,v 1.4 2006/03/23 13:31:18 cvs Exp $
///

/*##############################################################################
   
   Developed by NetMob Group, University of Trento, Italy

   University of Trento
   DIT - Department of Informatics and Telecommunications 
   Computer Networks and Mobility Research Program
   http://netmob.unitn.it/ 
   
   This file is part of the WilmaGate system, created as part
   of the WILMA Project, sponsored by the Autonomous
   Province of Trento and participated by ITC-irst, the University
   of Trento and Alpikom Spa. More info at

     http://www.wilmaproject.org/		

   To download the project:
      - in tarball format
        http://netmob.unitn.it/wilmagate.html
      - connect to CVS repository
        :pserver:cvs@rtm.unitn.it:/home/cvsroot
   In both previous cases, the download is password protected. To obtain
   the password you must send a mail to 

      wilmagate@dit.unitn.it

################################################################################

   The WilmaGate Licence of Use


   Copyright (c) 2003-2005
                - Universita` di Trento,
                  Dipartimento di Informatica e Telecomunicazioni (DIT)
                  Computer Networks and Mobility Research Program


   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (hereinafter
   the "WilmaGate") directly from the copyright owner, to use the WilmaGate
   without restriction of purpose or time, including without limitation the
   rights to use, copy, modify, and merge the WilmaGate, subject to the
   following conditions:

   The above copyright notice and this permission notice shall be included
   in all copies or substantial portions of the WilmaGate. 

   The WilmaGate shall not be sub-licenced either commercially or free of
   charge, as is or in part or as a part of any other package. 

   The licensee agrees to properly credit WilmaGate and its copyright 
   owners in any technical, scientific or divulgative publication, 
   whatever publishing means are used, that is based entirely or in part 
   on WilmaGate use, as well as in any booklet, web-site or brochure that 
   advertise services based on WilmaGate use. 

   If the WilmaGate is modified, the copyright owner shall be notified.

   THE WilmaGate IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
   OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
   IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
   CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
   TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
   SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 

##############################################################################*/

#ifndef _PROXY_PLUGIN_
#define _PROXY_PLUGIN_

#include <iostream>
#include <string>
#include "plugin.h"
//#include "proxy_info.h"

using namespace std;

namespace Gatekeeper {

class ProxyPlugin : public Plugin {

private:

	///
	///	\brief Number that identify the %Plugin.
	///
	///	The number is the ID of the %Plugin and is the index of the array element
	///	userElement::info that contains the pointer to the information of the 
	///	user.
	///
	int idPlugin;

	///
	///	\brief Reference to the interface between Plugins and %StatusTable.
	///
	/// This private element is a pointer to the interface between the Plugin class
	///	and the Status Table class.
	///
	PluginStatusTableInterface &psti;

	///
	///	\brief Reference to the interface between the Plugins and the %ProviderList.
	///
	/// This private element is a reference to the interface between this class
	///	and the ProviderList class. Really the variable that is passed by reference to the
	///	constructor is the ProviderList object.
	///
	ProviderListInterface &pli;

	///
	///	\brief IP address of the report service.
	///
	///	This address shall be inserted in responses as the server address (that of the Gatekeeper);
	///	It must be contacted by the report server.
	///
	const IPv4Address reportIP;

	///
	///	\brief Port of the report service.
	///
	///	This port shall be inserted in responses as the server address (that of the Gatekeeper);
	///	It must be contacted by the report server.
	///
	const uint16_t reportPort;

	///
	///	\brief IP address of the authentication service.
	///
	///	This address shall be inserted in responses as the server address (that of the Gatekeeper);
	///	It must be contacted by the authentication server.
	///
	const IPv4Address authIP;

	///
	///	\brief Port of the authentication service.
	///
	///	This port shall be inserted in responses as the server address (that of the Gatekeeper);
	///	It must be contacted by the authentication server.
	///
	const uint16_t authPort;

	///
	///	\brief Redirection URL.
	///
	///	Initial part of the redirection URL; the remaining part is a parametric string
	///	hardcoded in http_plugin.cpp
	///
	string redirectURL;

public:

	///
	///	\brief Constructor.
	///
	///	Costructor of the class: it inizialises the references to the interfaces.
	///	\param pluginStatusTableInterfaceObject is used to inizialise the private reference
	///	to the interface between plugins and status table in the Plugin class.
	///	\param providerListInterfaceObject is used to inizialise the private reference
	///	to the interface between the plugin and the provider list.
	///	\param rip Reporter IP address.
	///	\param rport Reporter port.
	///	\param aip Authenticator IP address.
	///	\param aport Authenticator port.
	///	\param red Redirection URL.
	///
	
	ProxyPlugin(PluginStatusTableInterface &pluginStatusTableInterfaceObject, 
			ProviderListInterface &providerListInterfaceObject,	
			IPv4Address rip, uint16_t rport,
			IPv4Address aip, uint16_t aport,
			const char *red);

	///
	/// \brief Virtual destructor.
	///
	virtual ~ProxyPlugin (void);

	///
	///	\brief Process and manage the received packet.
	///
	///	The function is the implemetation of pure virtual function defined in the
	///	Plugin class.
	///	The function
	///	receives a packet and processes it to know if it can be processed.
	///	In affermetive case, the function processes it: it modifies the status table
	/// and defines the packets that must send to user.
	///	to store the frame that the Dispatcher sends to the users.
	///	\param pointerToPacket is a pointer to the packet for
	/// analysing
	///	\param instructionVector is a vector of EthernetFrame elements and 
	///	stores the frame that the Dispatcher sends to the users.
	///	\param sense Direction in which the packet in the corresponding  instructionVector element is expected to be sent.
	///	\return \e true if the packet that is passed 
	///	by parameter can be processed by the called plugin sub-class, \e false if the 
	/// packet cannot be processed.
	///
	bool processPacket (EthernetFrame *pointerToPacket, EthernetFrame* instructionVector[], PacketSense sense[]);


};

}

#endif
