/// \file relay_plugin.h
/// \author Albino Scalet
///
/// \brief Class that implements DHCP relay features.
///
/// $Id: relay_plugin.h,v 1.1 2004/11/09 15:25:18 cvs Exp $

#ifndef _RELAY_PLUGIN_H_
#define _RELAY_PLUGIN_H_


#include "plugin.h"
#include "../Common/packets.h"
#include "../Common/socket.h"
#include "../Common/exceptions.h"
#include "../Common/addresses.h"
#include "../Common/packetproto.h"
#include "../Common/utilities.h"

////////////////////////////////////
#include <arpa/inet.h>
#include <sys/socket.h>
#include <features.h>
#include <netpacket/packet.h>
#include <net/if_arp.h>
#include <net/ethernet.h>
#include <unistd.h>

//To interact with status table
#include "plugin_statustable_interface.h"
#include "relay_listener.h"


using namespace Common;
using namespace std;

namespace Gatekeeper {


	enum Various {
			ETHHEADERSIZE=14,
			IPHEADERSIZE=20,
			UDPHEADERSIZE=8,
			HEADERSIZE = 42,
			PADDINGSIZE = 18,
			DHCPSTATICSIZE =240,
			DHCPOPTIONSMAXSIZE = 1000,
			ETHERMAXSIZE = 10000,
			RAIOSIZE =6,
			IPv4 =69,
			TTL = 64,
			CHADDR = 16,
			SFILE = 128,
			SNAME = 64,
			RESPONSEISOFFER =1,
			RESPONSEISACK= 2

	};


	////////////////////////////////////////////////////////////////////
	/// \brief Relay object
	////////////////////////////////////////////////////////////////////
	class RelayPlugin:
		public StatusTableListener, 
		public Plugin,
		public PacketListener {
	private:
		
		
		RelayListener *rtl;
	
	
		int shortLease;
			
		int longLease;
		
		
		///
		/// \brief Pointer to Dispatcher object.
		///

		const string * wpad;
		
		///
		/// \brief Reference to interface between Relay and %StatusTable.
		///
		/// This private element is a pointer to the interface between the Plugin class
		///	and the Status Table class.
		///
		//ReportStatusTableInterface &rsti;
		///
		/// \brief Reference to interface between Plugins and %StatusTable.
		///
		/// This private element is a pointer to the interface between the Plugin class
		///	and the Status Table class.
		///
		PluginStatusTableInterface &psti;
		///
		///\brief Array of pointers to registered listeners.
		///
		PacketListener * interface[1];
		///
		/// \brief UDP socket for communication with dhcp server.
		///
		UDPSocket relay;
		///
		/// \brief Pointer to a memory area devoted to an ethernet frame from INNER segment.
		///
		EthernetFrame *ethin;
		///
		/// \brief Pointer to a memory area devoted to a IP message from INNER segment.
		///
		IPPacket *ipin;
		///
		/// \brief Pointer to a memory area devoted to a UDP message from INNER segment.
		///
		UDPDatagram *udpin;
		///
		/// \brief Pointer to a memory area devoted to a DHCP message from INNER segment.
		///
		DHCPMessage *dhcpin;
		///
		///
		/// \brief Pointer to a memory area devoted to an ethernet frame from OUTER segment.
		///
		EthernetFrame *ethout;
		///
		/// \brief Pointer to a memory area devoted to a IP message from OUTER segment.
		///
		IPPacket *ipout;
		///
		/// \brief Pointer to a memory area devoted to a UDP message from OUTER segment.
		///
		UDPDatagram *udpout;
		///
		/// \brief Pointer to a memory area devoted to a DHCP message from OUTER segment.
		///
		DHCPMessage *dhcpout;
		///
		///
		/// \brief This structure contains the IP address, the name and the MAC of the interface that is oriented towards the client 			subnet.
		 struct SubnetInfo {
			/// \brief Interface name in system style.
			string	InterfaceName;
			/// \brief Interface IP address.
			IPv4Address InterfaceIP;
			/// \brief Interface MAC address.
			EthernetMACAddress  InterfaceMAC;
			/// \brief Subnet netmask.
			IPv4Address netmask;
		};
		///
		///\brief This structure contains name and IP of Relay's interface that is oriented towards the client Subnet.
		struct SubnetInfo toClientSubnet;
		///
		///\brief This structure contains name and IP of Relay's interface that is oriented towards the server Subnet.
		struct SubnetInfo toServerSubnet;
		///
		///\brief This is the IP of DHCP server.
		IPv4Address  serverIP;
		///
		///\brief This is the Public IP of NAT.
		IPv4Address NatPublicIP;
		///\brief This is the port of DHCP server.
		uint16_t serverPort;
		///
		///\brief This is the standard Server port.
		uint16_t clientPort;
		///
		///\brief This is the IP of DHCP server.
		IPv4Address  dnsIP;
		///
		///
		///\brief This private method is used to retrieve  the name of the interface which is oriented towards the client subnet.
		string getClientInterfaceName();
		///
		///\brief This private method is used to retrieve  the IP of the interface which is oriented towards the client subnet.
		IPv4Address  getClientInterfaceIP();
		///
		///\brief This private method is used to retrieve  the netmask of the interface which is oriented towards the client 				subnet.
		IPv4Address  getClientSubnetMask();
		///
		///\brief This private method is used to retrieve  the IP of the interface which is oriented towards the client 				subnet.
		IPv4Address  getServerInterfaceIP();
		///
		///\brief This private method is used to retrieve  the MAC address of the interface which is oriented towards the client subnet.
		EthernetMACAddress  getClientInterfaceMAC();
		///
		///\brief A method to obtain the IP of the DHCP server or intermediary relay.
		IPv4Address getServerIP();
		///
		///\brief A method to obtain the Port of the DHCP server.
		uint16_t getServerPort();
		///\brief A method to obtain the Port of the DHCP Client.
		uint16_t getClientPort();
		///	
		///\brief A method to obtain the IP of the DHCP server or intermediary relay.
		IPv4Address getDnsIP();
		///\brief A method to obtain NAT Public Interface IP .
		IPv4Address getNatIP();
		///
		///\brief This private method is used to build the RAI option appended to the end of option field.
		void raioBuild(DHCPMessage*  & myd, int & optlen);
		///
		///\brief This routine is useful in debug mode to print out built ethernet frames .
		void showETHmessage(EthernetFrame * myeth);
		///
		///\brief This routine is useful in debug mode to print out built IP packets .
		void showIPmessage(IPPacket * myip);
		///
		///\brief This routine is useful in debug mode to print out built UDP datagrams .
		void showUDPmessage(UDPDatagram * myudp);
		///
		///\brief This routine is useful in debug mode to print out built DHCP packets .
		void showDHCPmessage(DHCPMessage* mydhcp);
		
		
		public:
		///
		/// \brief Constructor.
		///
		/// \param sip  Server or intermediary relay IP address.
		/// \param sport Server  or intermediary relay UDP port.
		/// \param interface3IP IP address of the interface oriented towards client subnet.
		/// \param interface3Name System name of the interface oriented towards client subnet.
		///
		RelayPlugin (
			PluginStatusTableInterface &p,
			IPv4Address sip,
			uint16_t sport,
			uint16_t cport,
			IPv4Address interface3IP,
			string  interface3Name,
			IPv4Address interface2IP,
			IPv4Address NatIP,
			IPv4Address mask,
			int shortl,
			int longl,
			IPv4Address dns,
			const char* wpadf
		);
		
        ///
		/// \brief Virtual destructor.
		///
		virtual ~RelayPlugin (void);
		///
		/// \brief Inherited from Plugin.
		///
		bool processPacket (EthernetFrame *pointerToPacket, EthernetFrame* instructionVector[], PacketSense sense[]);
		///
		/// \brief Start the UDP socket receiver.
		///
		inline void start (void);
		//
		// Inherited from PacketListener.
		//
		virtual SuggestedAction failed (int socketID);
		virtual SuggestedAction receivedPacket (uint8_t *message, int size, int socketID, IPv4Address senderIP,
		uint16_t senderPort);
		//
		// Inherited from RouterInterface.
		//
		virtual void sendFrame (EthernetFrame *eth, int l, PacketSense sense);
		virtual void registerListener (PacketListener *t);
	
		//
		// Invoked from status table when a short dhcp lease expires
		//
		virtual void sendRelease(IPv4Address ip, EthernetMACAddress &mac);
		//
		// Never invoked and never implemented from relay class
		//
		virtual void statusChanged (IPv4Address ip, EthernetMACAddress &mac, StatusTableInterface::statusType oldStatus,StatusTableInterface::statusType newStatus);
	
 		void registerDHCPService(RelayListener *pointerToDispatcher);
	
	};

	inline void RelayPlugin::start (void)
	{
		logger() << "Starting socket \n"; 
		relay.startReceiving ();
	}
	
	
	
	
	
	
	
	
	
	
	
	

} // namespace Gatekeeper

#endif
