///
///	\file report.h
///	\author Danilo Severina
///
///	The fiel contains the interface with the Gateway block. It is used by the Gateway
///	when it needs to update or read the Status Table. It sends also informations to the Gateway
///	when a new user is authenticated, so the configuration in the Gateway Status Table must be 
///	set to allow the new user to browse.
///
///	$Id: report.h,v 1.3 2005/06/16 13:22:22 cvs Exp $
///

/*##############################################################################
   
   Developed by NetMob Group, University of Trento, Italy

   University of Trento
   DIT - Department of Informatics and Telecommunications 
   Computer Networks and Mobility Research Program
   http://netmob.unitn.it/ 
   
   This file is part of the WilmaGate system, created as part
   of the WILMA Project, sponsored by the Autonomous
   Province of Trento and participated by ITC-irst, the University
   of Trento and Alpikom Spa. More info at

     http://www.wilmaproject.org/		

   To download the project:
      - in tarball format
        http://netmob.unitn.it/wilmagate.html
      - connect to CVS repository
        :pserver:cvs@rtm.unitn.it:/home/cvsroot
   In both previous cases, the download is password protected. To obtain
   the password you must send a mail to 

      wilmagate@dit.unitn.it

################################################################################

   The WilmaGate Licence of Use


   Copyright (c) 2003-2005
                - Universita` di Trento,
                  Dipartimento di Informatica e Telecomunicazioni (DIT)
                  Computer Networks and Mobility Research Program


   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (hereinafter
   the "WilmaGate") directly from the copyright owner, to use the WilmaGate
   without restriction of purpose or time, including without limitation the
   rights to use, copy, modify, and merge the WilmaGate, subject to the
   following conditions:

   The above copyright notice and this permission notice shall be included
   in all copies or substantial portions of the WilmaGate. 

   The WilmaGate shall not be sub-licenced either commercially or free of
   charge, as is or in part or as a part of any other package. 

   The licensee agrees to properly credit WilmaGate and its copyright 
   owners in any technical, scientific or divulgative publication, 
   whatever publishing means are used, that is based entirely or in part 
   on WilmaGate use, as well as in any booklet, web-site or brochure that 
   advertise services based on WilmaGate use. 

   If the WilmaGate is modified, the copyright owner shall be notified.

   THE WilmaGate IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
   OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
   IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
   CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
   TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
   SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 

##############################################################################*/

#ifndef _REPORT_
#define _REPORT_

#include <iostream>
#include "../Common/addresses.h"
//#include "../Common/packets.h"
#include "../Common/socket.h"
#include "../Common/exceptions.h"
#include "report_statustable_interface.h"
#include "providerlist_interface.h"
#include "statustable_listener.h"

using namespace std;

namespace Gatekeeper {

///
///	\brief Interface towards %Gateway to update user status in %Gateway.
///
/// The class is an interface with the Gateway.
///	The class must be able to recive query from the %Gateway about refresh of the Status Table
///	and must also be able to send to the %Gateway each time a new user is authorized to access
///	to the network.
///
class Report :	public StatusTableListener, 
				public ConnectionListener, 
				public PacketListener {
private:

	///
	///	\brief Reference to StatusTable.
	///
	/// This private element is a reference to the interface between the
	///	Authentication class
	///	and the StatusTable class.
	///
	ReportStatusTableInterface &rsti;


	///
	///	\brief Reference to StatusTable.
	///
	/// This private element is a reference to the interface between the
	///	Authentication class
	///	and the StatusTable class.
	///
	ProviderListInterface &pli;

	///
	///	\brief TCP socket for incoming calling.
	///
	TCPSocket skTCPin;

	///
	///	\brief Array of ongoing connections
	///
	TCPSocket *tcpconn[256];

	///
	///	\brief IP address of the gateway.
	///
	const IPv4Address gatewayIP;

	///
	///	\brief Listening port of the gateway.
	///
	const uint16_t gatewayPort;

public:

	///
	///	\brief Constructor. Inizialize socket and reference to StatusTable.
	///
	///	The constructor sets the reference to the Status Table object and inizializes
	///	the listener for the TCP connection.
	///	\param reportStatusTableInterfaceObject The reference to the Status Table object.
	///	\param providerListInterfaceObject The reference to the Provider List object.
	///	\param listenIP Ip address of the local machine for TCP socket
	///	\param listenPort Port address of the local port for TCP socket
	///	\param gwIP Ip address of the gateway
	///	\param gwPort Port of the gateway
	///
	Report (ReportStatusTableInterface &reportStatusTableInterfaceObject,
			ProviderListInterface &providerListInterfaceObject,
			IPv4Address listenIP, uint16_t listenPort,
			IPv4Address gwIP, uint16_t gwPort)
		throw (WilmaGateException);

	///
	///	\brief Virtual destructor.
	///
	virtual ~Report();

	///
	///	\brief Start listening for TCP connection.
	///
	inline void start (void) { skTCPin.startListening(); }

	///
	///	\brief Stop listening for TCP connection.
	///
	inline void stop (void) { skTCPin.stopListening(); }

	///////////////////////////////////////////////////////////////
	//
	// Inherited from SocketListener
	//
	SuggestedAction receivedPacket (uint8_t *message, int size, int socketID,
						IPv4Address senderIP, uint16_t senderPort);
	SuggestedAction failed (int socketID);

	//
	// Inherited from ConnectionListener
	//
	SuggestedAction incomingConnection (TCPSocket *s, int socketID);

	//
	// Inherited from StatusTableListener
	//
	virtual void statusChanged (IPv4Address ip, EthernetMACAddress &mac, StatusTableInterface::statusType oldStatus,
					StatusTableInterface::statusType newStatus);
					
	//For DHCP relay plugin
	virtual void Report::sendRelease (IPv4Address ip, EthernetMACAddress &mac);



};

}

#endif
