/*********************************************************************************/
/// \file sip_plugin.h
///	\author Edoardo Bello e Flavio Pompermaier
///
///	\brief %Plugin to manage SIP packet from unauthorized users.
///
/// The file contains the definition of elements and methods to process packets
///	that use SIP protocol.
///	There is the implementation of processPacket function defined in Plugin
///	class.
///
///	$Id: sip_plugin.h,v 1.4 2006/10/11 13:10:18 cvs Exp $
///

/*##############################################################################
   
   Developed by NetMob Group, University of Trento, Italy

   University of Trento
   DIT - Department of Informatics and Telecommunications 
   Computer Networks and Mobility Research Program
   http://netmob.unitn.it/ 
   
   This file is part of the WilmaGate system, created as part
   of the WILMA Project, sponsored by the Autonomous
   Province of Trento and participated by ITC-irst, the University
   of Trento and Alpikom Spa. More info at

   http://www.wilmaproject.org/		

   To download the project:
      - in tarball format
        http://netmob.unitn.it/wilmagate.html
      - connect to CVS repository
        :pserver:cvs@rtm.unitn.it:/home/cvsroot
   In both previous cases, the download is password protected. To obtain
   the password you must send a mail to 

      wilmagate@dit.unitn.it

################################################################################

   The WilmaGate Licence of Use


   Copyright (c) 2003-2005
                - Universita` di Trento,
                  Dipartimento di Informatica e Telecomunicazioni (DIT)
                  Computer Networks and Mobility Research Program


   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (hereinafter
   the "WilmaGate") directly from the copyright owner, to use the WilmaGate
   without restriction of purpose or time, including without limitation the
   rights to use, copy, modify, and merge the WilmaGate, subject to the
   following conditions:

   The above copyright notice and this permission notice shall be included
   in all copies or substantial portions of the WilmaGate. 

   The WilmaGate shall not be sub-licenced either commercially or free of
   charge, as is or in part or as a part of any other package. 

   The licensee agrees to properly credit WilmaGate and its copyright 
   owners in any technical, scientific or divulgative publication, 
   whatever publishing means are used, that is based entirely or in part 
   on WilmaGate use, as well as in any booklet, web-site or brochure that 
   advertise services based on WilmaGate use. 

   If the WilmaGate is modified, the copyright owner shall be notified.

   THE WilmaGate IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
   OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
   IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
   CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
   TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
   SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 

##############################################################################*/

#ifndef SIP_PLUGIN_H_
#define SIP_PLUGIN_H_

#include <iostream>
#include <string>
#include "plugin.h"
#include "sip_providers.h"

using namespace std;

namespace Gatekeeper {

///
///	\brief %Plugin to manage SIP packet from unauthorized users.
///
/// The class defines elements and methods to process packets
///	that use SIP protocol.
///	The class must have only one public function: the processPacket() function
///	because it must be invoked by the Dispatcher. Obviously it has also a constructor
///	that initialize the private variable.
///
///	The private variable psti is needed by the %SipPlugin because the class must contact
///	the Status Table to check the status of the user identified by the parameters
///	in the received packets.
///	The private variable pli is needed by the %SipPlugin because the class must contact
///	the Provider List to check if the required URL in the received packet is th URL 
///	of an authentication SIP provider or not.
///	
///	When a packet is received by the %SipPlugin and it is an HTTP packet that 
///	requires a URL, the %HttpPlugin must contact the Status Table using the setURL() 
///	function with the MAC and IP addresses of the user and the required URL as 
///	parameters. The Status Table checks if the user has a right IP address (with a
///	token) and in affirmatively case it stores the required URL. In this case the 
///	user has a right Ip address but it cannot browse to the network because it is not 
///	authenticated yet, so the %HttpPlugin must generate an %EthernetFrame that the 
/// %Dispatcher will send to the user. The %EthernetFrame is a packet that redirects
///	the browser of the user to the authentication page of the provider (if there 
///	is only one authentication provider in the system) or to the web page where the
///	user can chose the preferred authentication provider (if there are more then one
///	authentication provider in the system).
///
/// The function setURL() returns a Token that identify the user in the next process
///	of the authentication: for a right authentication procedure, the remote 
///	authentication provider must identify the user with his name and the temporary token
///	that the system has given him during the HTTP capture procedure.
///
 

class SipPlugin : public Plugin {

private:
	///
	///	\brief Number that identify the %Plugin.
	///
	///	The number is the ID of the %Plugin and is the index of the array element
	///	userElement::info that contains the pointer to the information of the 
	///	user.
	///
	int idPlugin;
	
	///
	///	\brief Reference to the interface between Plugins and %StatusTable.
	///
	/// This private element is a pointer to the interface between the Plugin class
	///	and the Status Table class.
	///
	PluginStatusTableInterface &psti;

	///
	///	\brief Reference to the interface between the Plugins and the %ProviderList.
	///
	/// This private element is a reference to the interface between this class
	///	and the SipProviders class. Really the variable that is passed by reference to the
	///	constructor is the ProviderList object.
	///
	SipProviders &sp;	
	
	///
	/// \brief  enum indicating some type of messages, those that are interesting
	/// from an authentication point of view 
	enum method_t{
		DUMMY,		// others
		REGISTER,	// REGISTER message	
		SIP_2_0		// Response message
	};
	
	
	///
	///	\brief Verbosity of the %Plugin.
	///
	///	If this field is greater than 0, the %Plugin will show many details of INVITE and REGISTER
	///     SIP messages passing through it. If instead debug variable is equal to 0, normal
	/// 	logging verbosity will apply.
	///
	const int debug;
	
	///
	/// \brief This utility function return a pointer to the next line, 
	/// starting to parse 'offset' characters after the beginning of the payload 
	///
	/// \param payload A pointer to the beginning of the payload (see as a string)
	/// \param length  The total number of characters in the payload. Serves as a limit for parsing
	/// \param offset  A pointer indicating how many characters skip (from the 
	/// beginning of the payload) before start parsing
	/// \param delim   A character that indicates when the current line ends
	char* readline(char* payload, int length, int *offset, char delim);
	
	
	///
	/// \brief This utility function checks if in the passed header is present 
	/// in variable line, and in that case, puts parameters contained in 
	/// the params list into buf. It is important to note also
	/// that if params parameter passed is 0, then this function doesn't scan parameters
	/// of the header we are interested in, but put into buf[0] the value of that
	/// header field
	///
	/// \param line A pointer to the beginning of a single line of the SIP message
	/// \param header  The header we're looking for
	/// \param params  A list of parameters (WHICH LAST ELEMENT MUST BE 0!) 
	/// we are interested in. If found their values are pushed into buf array
	//  (e.g. if params={"branch",0} then for example buf[0]="xd4eqr12" and buf[1]=0)
	/// \param buf  a list containing the values of the parameters contained in
	/// params list (if present)
	bool paramscomp(char *line, char *header, char **params, char **buf);
	
	///
	/// \brief This utility function is designed to retrieve user information, such as Username and SIP account.
	/// Some strange User name can cause parsing error in the server side; but a typical SIP server vendor answer 
 	/// about that is that this is an application problem, because following SIP RFC this is illegal
	///
	/// In fact, the following answer is retrieved from OpenSer forum:
	///
	///>>>>> Question:
	///
	/// Hi all,
	/// I have noticed that when a user tries to register with a Username which 
	/// contains spaces, such as 'Enter your number here' (which is not present 
	/// in the subscriber table of the openser schema, thus one cannot 
	/// eventually register), Openser doesn't just send a 401 - Unauthorized 
	/// message back to the user, but on the other hand it will log the 
	/// following errors in /var/log/messages:
	///
	/// ERROR: get_hdr_field: bad to header
	/// ERROR: bad header field
	/// find_first_route: Error while parsing headers
	/// xl_get_to_attr: ERROR cannot parse TO header
	/// find_credentials(): Error while parsing headers
	/// pre_auth(): Error while looking for credentials
	/// ERROR: build_res_buf_from_sip_req: alas, parse_headers failed
    ///
	/// These errors are repeated over and over again, and I'm afraid that if 
	/// there are a number of users with such a username OpenSER might crash. 
	/// Could you  please tell me which part in the config file is sending out 
	/// such errors, or whether it is the development part of openser 
	/// itself which is creating  such errors?
	///
	///>>>> Answer:
	///
	/// " It is illegal as part of rfc 3261 to have non-escaped whitespace, 
	///   and as such, when the message is first parsed by openser, it undergoes
	///   some basic sanity checks. Don't worry, this shouldn't cause undue stress on 
	///   the application and openser's performance wont be affected. "
	///
	bool getUserInfo(char *line, char *header, char** destination, char chrstart, char chrend, int* retPosition);
	
public:

	///
	/// \brief Constructor.
	///
	/// Costructor of the class: it initializes the references to the interfaces.
	/// \param pluginStatusTableInterfaceObject is used to initialize the private reference
	/// to the interface between plugins and status table in the Plugin class.
	/// \param sipProviders: a list of all trusted Sip Authentication Server
	/// \param d: verbosity of logs (normal or very verbose).
	
	SipPlugin(PluginStatusTableInterface &pluginStatusTableInterfaceObject, 
			SipProviders &sipProviders, int d);

	///
	/// \brief Virtual destructor.
	///
	virtual ~SipPlugin (void);

	///
	///	\brief Process and manage the received packet.
	///
	///	The function is the implementation of pure virtual function defined in the
	///	Plugin class.
	///	The function
	///	receives a packet and processes it to know if it can be processed.
	///	In affirmative case, the function processes it: it modifies the status table
	/// and defines the packets that must send to user.
	///	to store the frame that the Dispatcher sends to the users.
	///	\param pointerToPacket is a pointer to the packet for
	/// analyzing
	///	\param instructionVector is a vector of EthernetFrame elements and 
	///	stores the frame that the Dispatcher sends to the users.
	///	\param sense Direction in which the packet in the corresponding  instructionVector element is expected to be sent.
	///	\return \e true if the packet that is passed 
	///	by parameter can be processed by the called plugin sub-class, \e false if the 
	/// packet cannot be processed.
	///
	bool processPacket (EthernetFrame *pointerToPacket, EthernetFrame* instructionVector[], PacketSense sense[]);


};

}

#endif /*SIP_PLUGIN_H_*/
