///
/// \file status_table.h
///	\author Danilo Severina
///
///	\brief Implementation and management for allowing the users.
///
///	The file status_table.h contains the definition of the parameters 
///	that identify the status of the network controlled by the 
///	authentication system. The used structure is an array of element
/// and each element contains information about an IP address: the array has
///	a static length because it is initialized with the 256 possible ip
///	address of a class "C" of IP addresses.
///
///	The structure contains the following information:
///		- IP address
///		- mac of the user
///		- identifier of the user
///		- token associated to the user
///		- the status of the IP address (free or used)
///		- the url required by the user when it is not authenticated
///		and authorized yet
///
///	The status of IP address are more complicated than the two case \e free or
///	\e used. The possible statuses are:
///		- FORBIDDEN: the IP address cannot be used by a user;
///		- FREE: the IP address is free and can be associated to a new client 
///		upon a DHCP request;
///		- FIRST_SEEN: the IP address has been offered to anew client during
///		the discovery process;
///		- TEMPORARY_LEASE: th e IP address is leased to the client with a short
///		expiration time;
///		- AUTHORIZED: the client has been identified and the gateway services are
///		granted. Subsequent DHCP lease will confirm the current IP address with a
///		longer expiration time.
///
///	$Id: status_table.h,v 1.6 2006/03/30 13:35:23 cvs Exp $
///	

/*##############################################################################
   
   Developed by NetMob Group, University of Trento, Italy

   University of Trento
   DIT - Department of Informatics and Telecommunications 
   Computer Networks and Mobility Research Program
   http://netmob.unitn.it/ 
   
   This file is part of the WilmaGate system, created as part
   of the WILMA Project, sponsored by the Autonomous
   Province of Trento and participated by ITC-irst, the University
   of Trento and Alpikom Spa. More info at

     http://www.wilmaproject.org/		

   To download the project:
      - in tarball format
        http://netmob.unitn.it/wilmagate.html
      - connect to CVS repository
        :pserver:cvs@rtm.unitn.it:/home/cvsroot
   In both previous cases, the download is password protected. To obtain
   the password you must send a mail to 

      wilmagate@dit.unitn.it

################################################################################

   The WilmaGate Licence of Use


   Copyright (c) 2003-2005
                - Universita` di Trento,
                  Dipartimento di Informatica e Telecomunicazioni (DIT)
                  Computer Networks and Mobility Research Program


   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (hereinafter
   the "WilmaGate") directly from the copyright owner, to use the WilmaGate
   without restriction of purpose or time, including without limitation the
   rights to use, copy, modify, and merge the WilmaGate, subject to the
   following conditions:

   The above copyright notice and this permission notice shall be included
   in all copies or substantial portions of the WilmaGate. 

   The WilmaGate shall not be sub-licenced either commercially or free of
   charge, as is or in part or as a part of any other package. 

   The licensee agrees to properly credit WilmaGate and its copyright 
   owners in any technical, scientific or divulgative publication, 
   whatever publishing means are used, that is based entirely or in part 
   on WilmaGate use, as well as in any booklet, web-site or brochure that 
   advertise services based on WilmaGate use. 

   If the WilmaGate is modified, the copyright owner shall be notified.

   THE WilmaGate IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
   OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
   IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
   CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
   TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
   SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 

##############################################################################*/

#ifndef _STATUS_TABLE_
#define _STATUS_TABLE_

#include <iostream>
#include "plugin_statustable_interface.h"
#include "auth_statustable_interface.h"
#include "report_statustable_interface.h"
#include "../Common/access.h"
#include "../Common/timers.h"
#include "plugin_info.h"
#include <ctime>
extern "C"{
	#include "netinet/in.h"
	#include "arpa/inet.h"
	#include "inttypes.h"
}
 
using namespace std;
using namespace Common;




namespace Gatekeeper {


//	The class has a derivated class (StatusTableInterface). Maybe some functions
//	should be implemented as protected methods, so only the first level derived class
//	can access to them. Other solution with "friends" class (but so you can access
//	also to private elements) or method.
//

/**
*	The constant defines the maximum number of IP address that the system uses.
*	Also FORBIDDEN Ip addresses (used by the system) are considered.
*/
const int MAX_USER=256;

const int MAX_LEN_TOKEN=20;

class PluginInfo;

///
///	\brief Table of the status of users.
///
/// The class StatusTable identifies the status of the user in the network.
///	It contains an array of structures ordered 
///	by IP address, and each element of the array has also information about:
///		- IP address
///		- mac of the user
///		- identifier of the user
///		- token associated to the user
///		- the status of the IP address
///		- the url required by the user when it is not authenticated and authorized yet.
///		- token that identifies the user during the authentication procedures and
///	during the renewal procedure for the validity of IP address.
///		- lease times that identify the validity of IP address or authentication
///   		- type of authorization achieved by the user (NONE, PERSISTENT, UNKNOWN, HTTP or SIP)
///
///	The Status Table manages the system because it contains the informations about
///	which user can access to the network and which cannot. The status of Status Table
///	can change frequently and the changes can be invoked by extern action (e.g. for
///	request of IP addresses or for authentication by remote server) or can be managed
///	by the same %StatusTable class (e.g. when a lease time expires and a renewal procedure
///	must be invoked).
///
/// Description of operations that the %StatusTable must do follows. The involved functions
///	are described briefly, a mode detailed description can be found in the following 
///	sections.
///
///	The function %registerListener() is used only to have a pointer to the Report class: it needs
///	when the status of an IP address changes and reaches the AUTHORIZED status, because in this case
///	the Gateway (contacted throughout the Report object) must know the parameters of the new user
///	that is allowed to access to the network. To avoid the Report inquires cyclically the
///	Status Table to know the status of the IP, an interface on the %Report is used: the Report
///	will be informed when important changes are done.
///
///	There are also other general functions that are defined in the StatusTableInterface: these 
///	functions are defined in the other class as virtual public ones. The StatusTable does not 
///	include the StatusTableInterface class because the last one is inherited by the other
///	interfaces that are inherited by the StatusTable.
///
class StatusTable : public PluginStatusTableInterface, 
					public AuthStatusTableInterface,
					public ReportStatusTableInterface,
					public TimerListener {

private:

	///
	///	\brief Pointer to StatusTableListener
	///
	///	The private element is a pointer to the StatusTableListener. Really the element
	///	points to the Report object, but you can modify the type of pointer because the 
	///	Report class derives from StatusTableListenerInterface.
	///	When a StatusType in the Status Table changes in the AUTHORIZED status, the Gateway must
	///	know the changes and to inform it, the call() function defined in 
	///	StatusTableListenerInterface must be invoked.
	///
	StatusTableListener *stl;

        ///     \brief Pointer to StatusTableListener activated on relay plugin
        ///
        StatusTableListener *dtl;

	///
	///	\brief Parameters referred to an IP address.
	///	
	///	The structure that identifies the status of the network is a vector of
	///	elements and each element contains information about the user that is
	///	associated to an IP address and the status of the IP address 
	///	itself.
	///	The elements are
	///		- IP address:
	///		- mac of the user: it is an array of char of 8 character;
	///		- token associated to the user: it an object of the class Token;
	///		- the status of the IP address: it can assume one of the status
	///	concerned by the previous list;
	///		- the url required by the user when it is not authenticated
	///		and authorized yet: it is a pointer to a string of character;
	///		- authentication type: designed to allow flexible management of authorizations
	///	achieved by users
	struct userElement{
		// DATA FOR A SINGLE USER
		bool changed;
		IPv4Address ip;			// IP address of the user
		EthernetMACAddress mac;		// MAC address of the user
		string url;			// Required URL during the HTTP capture
		Token* token;			// Token identifies an authenticated user
		statusType status;		// Status of IP address
		time_t expireIP;		// Expiration time of IP address
		time_t expireAuth;		// Expiration time of Authentication
		bool dhcpRenewsAuth; 		// Authorization is renewed as long as
						// lease is kept valid
		char *name;
		char *email;
		PluginInfo* *info;		// Array of data for Plugins.
						// Each columns refers to a %Plugin.
		bool IPMode;	//false manual releaved from interceptor..
                        //true with dhcp
  		//  time_t presenceCheck;
		
		//variable authType: Added for SIP plugin
		authTypes authType;//intended as a flag-array: at the moment, 
		//bit-0 equals to 1 says that the associated IP is not authenticated 
		//bit-1 equals to 1 says that an UNKNOWN authentication has been performed
		//bit-2 equals to 1 says that a  PERSISTENT authentication has been performed
		//bit-3 equals to 1 says that an HTTP authentication has been performed, 
		//bit-4 equals to 1 says that a  SIP  authentication has been performed	

	};

	//albino here
	struct userAuth{
		EthernetMACAddress mac;		// MAC address of the user
		char *name;
		char *email;
		time_t expireAuth;
	};

  
  	//albino
	time_t timeWindow;
	
	//albino here
	userAuth authUser[MAX_USER];

	
	//albino here
	int currAuth;
	


	///
	///	\brief Array of IP address supported by the system.
	///	
	///	The array that contains pointer to elements that describe the status of each
	///	IP address and the information about the user associated to it, has a
	///	static dimension, because we suppose that the network controlled by the
	///	system has IP addresses of "C" class (x.x.x.y/24).
	///
	userElement user[MAX_USER];

	///
	///	\brief Variable to control the access to the StatusTable
	///
	///	The class AccesRight allows to user to access to the Status Table with shared access
	///	exclusive access. The shared one allows to other classes to access the same Status
	///	Table with a shared access, but not in a exclusive mode. Instead, the exclusive 
	///	access forbids any other access (shared or exclusive).

	///
	AccessRights ar[MAX_USER];

	///
	///	\brief Number of registered Plugins.
	///
	///	The number of registered Plugins to the %Dispatcher is used to create the array of
	///	pointer to PluginInfo to storage the information about each used by each Plugins.
	///
	int nPlugin;

	///
	///	\brief Timer object for automatic periodic maintenance.
	///
	CoarseTimer timer;

	///
	///	\brief Persistence file, where table is stored between program executions.
	///
	fstream persistenceFile;
	
	///
	/// \brief Remove all authentication type associated to the IP address passed
	///
	/// \param ip The IP address who will reset to NONE authorization type
	/// 
	//Added for SIP Plugin purposes
	bool clearAuthType(IPv4Address ip);
	
public:

	///
	///	\brief Constructor. Inizialise array of userElement.
	///	
	///	The constructor initialize the private array users: each element of the array
	///	is a pointer to a struct userElement. The each userElement is initialized 
	///	with the IP address and the status is set to FREE or FORBIDDEN for reserved 
	///	IP address.
	///
	//MB
	///	\param ipClass Net address (at present we always store a class C subnet).
	///	\param minIP First FREE IP. All addresses prior to this are marked as FORBIDDEN.
	///	\param maxIP Last FREE IP. All addresses next to this are marked as FORBIDDEN.
	///	\param fname Name of file storing persistent info.
	///
	StatusTable(IPv4Address ipClass, IPv4Address minIP, IPv4Address maxIP, const char *fname,time_t timeWin);

	///
	///	\brief Destructor.
	///
	virtual ~StatusTable();

	///
	///	\brief Add an unrevocable authorization
	///
	bool addAuthorization (const char *name, const char *email, IPv4Address ip, EthernetMACAddress &mac);

	// Inherited
	bool setURL(IPv4Address ip, const char *urlRequired);
	bool setStatus(IPv4Address ip, statusType status, bool newToken);
	bool setExpirationIP(IPv4Address ip, int sec);
	bool setExpirationAuth(IPv4Address ip, int sec, bool dhcpRenews);
	StatusTableInterface::statusType getStatus(IPv4Address ip);
	IPv4Address getFreeIP(EthernetMACAddress &macUser);
	IPv4Address getIP(EthernetMACAddress macUser);
	const char* getURL(IPv4Address ip);
	Token* getToken(IPv4Address ip);
	bool checkIP (EthernetMACAddress &mac, IPv4Address ip);
	//>>>>>>>>>>>>>>>>>>>>>>>>>> always inherited but added after SIP Plugin insertion <<<<<<<<<<<<<<<<<
	bool authorizeUserFor(IPv4Address ip,authTypes authType,const char *name, const char *email);
	bool addAuthType(IPv4Address ip, authTypes auth);
	bool removeAuthType(IPv4Address ip, authTypes auth);
	bool isAuthenticated(IPv4Address ip);
	bool isHTTPauthenticated(IPv4Address ip);
	bool isSIPauthenticated(IPv4Address ip);
	//>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>


	///
	///	\brief Get the number of users. The dimension of the Status Table
	///
	inline int getNumberUser() {return MAX_USER;};

	///
	///	\brief Get the first IP of the Status Table.
	///
	IPv4Address getFirstIP();


	bool validExpirationIP(IPv4Address ip);
	bool validExpirationAuth(IPv4Address ip);

	

	///
	///	\brief Registers the listener for the Report.
	///	
	///	The function is used by the StatusTable to notify changes in the Status Table to the Report.
	///	When there is a change of the status of an IP address the Report must be informed, because it
	///	must send the new informations to the Gateway which updates its own list of users.
	///	The main task of the function is to send to the StatusTabel a pointer to the 
	///	StatusTableListener. Really parameter is a the pointer to the Report object created in 
	///	main function.
	///	The function is implemented in the StatusTable class.
	///	\param pointerToReport is a pointer to Report object, but it is defined as a pointer to
	///	the interface between StatusTable and Report.
	///	
	virtual void registerListener(StatusTableListener *pointerToReport);

	///
	///	\brief Create the array for information for the Plugins.
	///
	bool createInfo();

	///
	///	\brief Notify the addition of a %Plugin in the system.
	///
	///	The function is invoked by the Plugin when it is initializes. In this way the Status Table
	///	can know how many Plugins are in the system and can create an array (userElement::*info)
	///	with right dimensions.
	///	The %StatusTable doesn't know the type of the new %Plugin.
	///	\return The index of the column of the array reserved for the added %Plugin.
	///
	inline int registerPlugin() {return nPlugin++;};


	///
	///	\brief Add a pointer to Plugin.
	///
	///	The function adds the pointer to a %PluginInfo object where the %Plugin store
	///	information about a user.
	///	\param IDPlugin The ID of the %Plugin.
	///	\param ip The IP address of the user.
	///	\param pi Pointer to the pluginInfo object to be added.
	///	\return \e true if the insertion was successful, \e false otherwise.
	///
	bool addPluginInfo(int IDPlugin, IPv4Address ip, PluginInfo* pi);


	///
	///	\brief Get the pointer to %PluginInfo.
	///
	///	Gets the pointer to the %PluginInfo that is identified by 
	///	the ID of the %Plugin and the user's IP address. If the required PluginInfo is not jet
	///	created, it returns a NULL pointer.
	///
	PluginInfo* getPluginInfo(int IDPlugin, IPv4Address ip);

	///
	/// \brief Start background maintenance schedule.
	///
	void start (void);

	///
	/// \brief Stop background maintenance schedule.
	///
	void stop (void);

	///
	/// \brief Write a row to persistence file.
	///
	/// \param i Row to write.
	///
	void writeRowToFile (int i);

	///
	/// \brief Read a row from persistence file.
	///
	void readTableFromFile (void);
	
	//
	// Inherited from PSTI.
	//
	virtual EthernetMACAddress *getMAC (IPv4Address ip);
	virtual void associateMACtoIP (EthernetMACAddress macUser, IPv4Address ipUser);
	virtual void registerDHCPService (StatusTableListener *p);
  //0 manual 1 DHCP
  virtual void setIPMode(IPv4Address ipUser, bool ipMode);
 // virtual void setPresenceCheck(IPv4Address ipUser,time_t nextCheck);

	//
	// Inherited from TimerListener.
	//
	virtual bool timerTick (void);

	//
	// Inherited fom RSTI
	//
	virtual void listUsers (ostream &o, statusType l);
	virtual bool getUserData (IPv4Address ip, char *name, char *email, time_t *expIP, time_t *expAuth, statusType *s, int bufsize);

	//
	// Inherited fom ASTI
	//
	virtual void setUserData (IPv4Address ip, const char *name, const char *email);


	//albino here
	virtual bool isPreAuthenticated(EthernetMACAddress &mac,IPv4Address ip);
	
};

inline IPv4Address StatusTable::getFirstIP() 
{
	return user[0].ip;
}






}



#endif



