///
/// \file statustable_interface.h
///	\author Danilo Severina
///
///	The file contains the definition of functions that work on
/// Status Table object and that are common to interfaces towards 
/// plugin sub-classes and authentication class.
///
///	$Id: statustable_interface.h,v 1.5 2005/12/13 17:18:53 cvs Exp $
///

/*##############################################################################
   
   Developed by NetMob Group, University of Trento, Italy

   University of Trento
   DIT - Department of Informatics and Telecommunications 
   Computer Networks and Mobility Research Program
   http://netmob.unitn.it/ 
   
   This file is part of the WilmaGate system, created as part
   of the WILMA Project, sponsored by the Autonomous
   Province of Trento and participated by ITC-irst, the University
   of Trento and Alpikom Spa. More info at

     http://www.wilmaproject.org/		

   To download the project:
      - in tarball format
        http://netmob.unitn.it/wilmagate.html
      - connect to CVS repository
        :pserver:cvs@rtm.unitn.it:/home/cvsroot
   In both previous cases, the download is password protected. To obtain
   the password you must send a mail to 

      wilmagate@dit.unitn.it

################################################################################

   The WilmaGate Licence of Use


   Copyright (c) 2003-2005
                - Universita` di Trento,
                  Dipartimento di Informatica e Telecomunicazioni (DIT)
                  Computer Networks and Mobility Research Program


   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (hereinafter
   the "WilmaGate") directly from the copyright owner, to use the WilmaGate
   without restriction of purpose or time, including without limitation the
   rights to use, copy, modify, and merge the WilmaGate, subject to the
   following conditions:

   The above copyright notice and this permission notice shall be included
   in all copies or substantial portions of the WilmaGate. 

   The WilmaGate shall not be sub-licenced either commercially or free of
   charge, as is or in part or as a part of any other package.

   The licensee agrees to properly credit WilmaGate and its copyright 
   owners in any technical, scientific or divulgative publication, 
   whatever publishing means are used, that is based entirely or in part 
   on WilmaGate use, as well as in any booklet, web-site or brochure that 
   advertise services based on WilmaGate use. 

   If the WilmaGate is modified, the copyright owner shall be notified.

   THE WilmaGate IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
   OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
   IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
   CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
   TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
   SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 

##############################################################################*/

#ifndef _STATUSTABLE_INTERFACE_
#define _STATUSTABLE_INTERFACE_


#include <iostream>
//#include "status_table.h"
#include "../Common/addresses.h"
#include "token.h"

using namespace std;
using namespace Common;


namespace Gatekeeper {

///
///	The class contains the definition of functions that work on
/// Status Table object and that are common to interfaces towards 
/// plugin sub-classes and authentication class.
///	The classes PluginStatusTableInterface and AuthStatusTableInterface
///	derive from this class: they don't implement the virtual functions that
///	are defined in this class, because these virtual functions are implemented
///	by StatusTable class.
///

class StatusTableInterface{

private:
		

public:

		///
		/// \brief Virtual destructor.
		///
		virtual ~StatusTableInterface (void) {}

	///
	/// The status of the IP address is not only \e free or \e used, but it can assume
	///	different states. In fact it can be:
	///		- FORBIDDEN: the IP address cannot be used by a user;
	///		- FREE: the IP address is free and can be associated to a new client 
	///		upon a DHCP request;
	///		- FIRST_SEEN: teh IP address has been offered to anew client during
	///		the discovery process;
	///		- TEMPORARY_LEASE: th eIp address is leased to hte client with a short
	///		expiration time;
	///		- AUTHORIZED: the client has been identified and the gateway services are
	///		granted. Subseqeunt DHCP lease will confirm the current IP address with a
	///		longer expiration time.
	///	
	enum statusType {	FORBIDDEN,
				FREE,
				FIRST_SEEN,
				TEMPORARY_LEASE,
				AUTHORIZED};

	///
	/// This enum permits to keep track of authentications achieved by users.
	/// This information is useful when we want to decide if a user must be deauthenticated or not:
	/// suppose in fact that a user has performed an HTTP authentication and, after this,
	/// it registers to one of the trusted (from UniFy point of view) SIP authentication servers. 
	/// Now, if the user logout from the popup window, we want that this does not affect
	/// SIP services, so user's status in the status-table must not change.
	/// Whereas, if a user performs only an HTTP authentication and, after that, it performs a
	/// SIP one to an untrusted SIP server (always from UniFy point of view), when the 
	/// user logout from the browser popup window we want to close also all still running 
	/// SIP user activities.
	/// At the moment, only two possible types of authentication are possible:
	///		- HTTP: the traditional authentication achieved through the browser;
	///		- SIP: the new type of authentication, achievable through a softphone or
	///	    through a SIP enabled phone.
	/// However, this design choice has been thinked in order to allow both authentications separately,
	/// whereas at the moment, only one kind of authentication can be performed ad a time
	// added by Flavio Pompermaier
	enum authTypes {	NONE=0,
				UNKNOWN=1,
				PERSISTENT=2,
				HTTP=4,
				SIP=8
				//next values must be multiples of 2: 16,32...65536
				//since the values are treated as flags. In this way for example,
				//a user could be authenticated via:
				// HTTP: 		0100;
				// SIP : 		1000;
				// HTTP + SIP : 1100;
		    };
	///
	///	The function sets the URL required by an user.
	///	\param ip The Ip address of the user
	///	\param urlRequired The URL that is required by the user
	/// \return \e True if the setting was successfull, \e false otherwise.
	///
	virtual bool setURL(IPv4Address ip, const char *urlRequired) = 0;

	///
	///	The function sets the status of the Ip address of the user who are
	///	identified by the MAC and IP element.
	///	This method is defined \e virtual in StatusTableInterface.
	///	\param ip The IP address of the user.
	///	\param status The new status of IP address specified.
	///	\param newToken True if and only if a new token must be generated in place of the existing one.
	///
	/// \return \e True if the setting was successfull, \e false otherwise.
	///
	virtual bool setStatus(IPv4Address ip, statusType status, bool newToken = false) = 0;

	///
	///	The function sets the duration of the lease time of the Ip address.
	///	The user is identified by its MAC and IP address, and the duration will be
	///	storage in the Status Table as the absolute end time.
	///	\param ip The IP address of the user.
	///	\param sec The duration of the lease.
	/// \return \e true if the procedure is correct, \e falses otherwise.
	///
	virtual bool setExpirationIP(IPv4Address ip, int sec) = 0;

	///
	///	The function sets the duration of the lease time of the Authentication.
	///	The user is identified by its MAC and IP address, and the duration will be
	///	storage in the Status Table as the absolute end time.
	///	\param ip The IP address of the user.
	///	\param sec The duration of the lease.
	///	\param dhcpRenews True if authorization must be ensured as long
	///	as DHCP lease is kept.
	/// \return \e true if the procedure is correct, \e false otherwise.
	///
	virtual bool setExpirationAuth(IPv4Address ip, int sec, bool dhcpRenews = false) = 0;

	///
	///	The function gets the status of the IP address of the user who are
	///	identified by the MAC and IP element.
	///	This method is defined \e virtual in StatusTableInterface.
	///	\param ip The IP address of the user.
	///	\return The status of IP address.
	///
	virtual StatusTableInterface::statusType getStatus(IPv4Address ip) = 0;

	///
	/// The function gets a useful IP address for the user that requires one
	///	The functions does the following action in this order:
	///		- it looks for if there is a free IP address that was used in the past by the same
	///	MAC address;
	///		- if there no IP address that satisfies the previous condition, it looks for
	///	an IP address that has never used;
	///		- if also this condition is not satisfied, it looks for a FREE IP address and
	///	it choses the IP address that has the oldest expiration time of the IP.
	///	
	///	During the checks, the function check also the status of the IP addresses:
	///	if the lease time of the authentication (expireAuth) is expired, the status of the IP
	///	address is set TEMPORARY_LEASE and if the lease time of the DHCP is expired (expireIP),
	///	the status of IP address is set FREE.
	///	If all the IP address are used by users, the function returns an IP address that
	///	will be considered invalid: "0.0.0.0".
	///	\param macUser The MAC address of the user.
	///	\return The IP address associated to the user.
	///
	virtual IPv4Address getFreeIP (EthernetMACAddress &macUser) = 0;

	///
	/// The function returns the IP address associated to the MAC address.
	///	\param macUser The MAC address of the user.
	///	\return The IP address of the user.
	///
	virtual IPv4Address getIP(EthernetMACAddress macUser) = 0;

	///
	///	The function returns the URL required by the user during the 
	///	\e HTTP \e Capture session.
	///	\param ip The IP address of the user.
	///	\return  A pointer to the char array of the URL required by the user.
	///
	virtual const char* getURL(IPv4Address ip) = 0;

	///
	///	The function returs the MAC address associated to an IP address, NULL if none.
	///	\param ip The IP address of the user.
	///	\return The MAC address of the user.
	///
	virtual EthernetMACAddress *getMAC(IPv4Address ip) = 0;

	///
	/// The function gets the Token associated to a specified user.
	///	\param ip The IP address of the user.
	///	\return The pointer to the Token associated to the user identified by the parameters.
	///
	virtual Token* getToken(IPv4Address ip) = 0;

	///
	///	The function describe the validity of the IP address.
	///	\param ip The IP address of the user.
	///	\return \e true if the expiration time of the IP is not reached, \e false otherwise
	///
	virtual bool validExpirationIP(IPv4Address ip) = 0;

	///
	///	The function describe the validity of the authorization.
	///	\param ip The IP address of the user.
	///	\return \e true if the expiration time of the authorization is not reached, \e false otherwise
	///
	virtual bool validExpirationAuth(IPv4Address ip) = 0;

	//Used from DHCP relay plugin to write MAC address of a client in the status table.
	///	\param macUser The MAC address of the user.
	///	\param ipUser The IP address of the user.
	virtual void associateMACtoIP(EthernetMACAddress macUser, IPv4Address ipUser) =0;

	//albino here
	virtual bool isPreAuthenticated(EthernetMACAddress &mac,IPv4Address ip)= 0;

	
	//#########  Functions added for SIP Plugin: ##########################
	// Added by Flavio Pompermaier

	///
	///	The function keep track of users's authentication achieved through HTTP,
	/// SIP or whatever other protocol/authentication servers.
	///	\param ip The IP address of the user.
	///	\param auth The type of the authentication to add
	/// \return \e true if the procedure is correct, \e falses otherwise.
	///	
	virtual bool addAuthType(IPv4Address ip, authTypes auth) = 0;
	
	///
	///	The function remove a specific authentication from those associated with 
	/// the specified user
	///	\param ip The IP address of the user.
	///	\param auth The type of the authentication to remove
	/// \return \e true if the procedure is correct, \e falses otherwise.
	///	
	virtual bool removeAuthType(IPv4Address ip, authTypes auth) = 0;
	
	///
	///	The function checks if a user is authenticated with some trusted server,
	/// either HTTP, either SIP or whatever
	///	\param ip The IP address of the user.
	/// \return \e true if the user is authenticated, \e false otherwise.
	///	
	virtual bool isAuthenticated(IPv4Address ip) = 0;
	
	///
	///	The function checks if a user is authenticated with a trusted (from Unify 
	/// point of view) HTTP server
	///	\param ip The IP address of the user.
	/// \return \e true if the user is HTTP authenticated, \e false otherwise.
	///	
	virtual bool isHTTPauthenticated(IPv4Address ip) = 0;
	
	///
	///	The function checks if a user is authenticated with a trusted (from Unify 
	/// point of view) SIP server
	///	\param ip The IP address of the user.
	/// \return \e true if the user is SIP authenticated, \e false otherwise.
	///	
	virtual bool isSIPauthenticated(IPv4Address ip) = 0;
	
	///
	/// This function checks in an atomic way if a user must be authenticated or
	/// not (because it is already authenticated). The method is
	/// actually a call in sequence of the functions: setStatus(), addAuthType() 
	/// plus some checking like isAuthenticated(). The function has become necessary
	/// since this functions can leave the system in inconsistent state if 
	/// called out of synchronization, such as when multiple authenticator 
	/// (like a SIP and an HTTP one) try to authenticate some IP address/user.
	///	\param ip the user who need to be authorized for that kind of traffic,
	/// if no other authorization arrives before the exclusive lock of the semaphore
	///	\param authType the type of authorization
	///	\param name the name of the user (for traceability reasons)
	///	\param email the email/sip account of the user (for traceability reasons)
	///	\return \e true if the user is SIP authenticated, \e false otherwise.
	///	
	virtual bool authorizeUserFor(IPv4Address ip,authTypes authType,const char *name, const char *email) = 0;

	//###########################################################
};

}

#endif

