///
/// \file WTP_table.h
///	\author Nicolo Pernigo
///
///	\brief Implementation and management for allowing the WTP (Access Point).
///
///	The file WTP_table.h contains the definition of the parameters 
///	that identify the status of the WTP. The used structure is an vector of element
/// and each element contains information about an WTP: the vector has
///	a dinamic.
///
///	The structure contains the following information:
///		- WTP IP address
///		- Session Identifier
///		- WTP Mac address
///		- Port
///		- Status of CAPWAP session (Idle, Discovery, Sulking, Configure, Run, Reset)
///		- WaitJoin Timer
///		- All MIB register, derived to reception of WiFi Response
///		- Location of WTP
///		- Name of WTP
///		- Number of radios
///		- Identifier of Radio in use
///		- Radio Type
///		- State of radio
///		- Cause
///		- Channel in use
///		- Trasmission Power
///
///	$Id: ap_table.h,v 1.0 2006/10/09 17:51:00 cvs Exp $
///	

/*##############################################################################
   
   Developed by NetMob Group, University of Trento, Italy

   University of Trento
   DIT - Department of Informatics and Telecommunications 
   Computer Networks and Mobility Research Program
   http://netmob.unitn.it/ 
   
   This file is part of the WilmaGate system, created as part
   of the WILMA Project, sponsored by the Autonomous
   Province of Trento and participated by ITC-irst, the University
   of Trento and Alpikom Spa. More info at

     http://www.wilmaproject.org/		

   To download the project:
      - in tarball format
        http://netmob.unitn.it/wilmagate.html
      - connect to CVS repository
        :pserver:cvs@rtm.unitn.it:/home/cvsroot
   In both previous cases, the download is password protected. To obtain
   the password you must send a mail to 

      wilmagate@dit.unitn.it

################################################################################

   The WilmaGate Licence of Use


   Copyright (c) 2003-2005
                - Universita` di Trento,
                  Dipartimento di Informatica e Telecomunicazioni (DIT)
                  Computer Networks and Mobility Research Program


   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (hereinafter
   the "WilmaGate") directly from the copyright owner, to use the WilmaGate
   without restriction of purpose or time, including without limitation the
   rights to use, copy, modify, and merge the WilmaGate, subject to the
   following conditions:

   The above copyright notice and this permission notice shall be included
   in all copies or substantial portions of the WilmaGate. 

   The WilmaGate shall not be sub-licenced either commercially or free of
   charge, as is or in part or as a part of any other package. 

   The licensee agrees to properly credit WilmaGate and its copyright 
   owners in any technical, scientific or divulgative publication, 
   whatever publishing means are used, that is based entirely or in part 
   on WilmaGate use, as well as in any booklet, web-site or brochure that 
   advertise services based on WilmaGate use. 

   If the WilmaGate is modified, the copyright owner shall be notified.

   THE WilmaGate IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
   OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
   IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
   CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
   TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
   SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 

##############################################################################*/

#ifndef _WTP_TABLE_
#define _WTP_TABLE_

#include <iostream>
#include "wtptable_interface.h"
#include "../Common/parameters.h"
#include "../Common/access.h"
#include "../Common/timers.h"
#include "plugin_info.h"
#include "capwap_plugin_interface.h"
#include "capwap_plugin_B.h"
#include <ctime>
#include <vector>

#include "../Common/utilities.h"
#include "../Common/parameters.h"
#include <string>
extern "C"{
	#include "netinet/in.h"
	#include "arpa/inet.h"
	#include "inttypes.h"
}
 
using namespace std;
using namespace Common;

namespace Gatekeeper {


//	The class has a derivated class (StatusTableInterface). Maybe some functions
//	should be implemented as protected methods, so only the first level derived class
//	can access to them. Other solution with "friends" class (but so you can access
//	also to private elements) or method.
//

/**
*	The constant defines the maximun number of WTP that the system support.
*/
const int MAX_WTP=64;

//State machine
//Idle, Discovery, Sulking, Join, Configure, Image Data, Run, Reset
/*
int STATE[8,8] = {
{1,1,0,1,0,0,0,0},
{0,1,1,1,0,0,0,0},
{1,0,1,0,0,0,0,0},
{1,1,0,1,1,0,0,0},
{0,0,0,0,1,1,1,0},
{0,0,0,0,0,1,0,1},
{0,0,0,0,0,0,1,1},
{1,0,0,0,0,0,0,1}};
*/
class PluginInfo;

///
///	\brief Table of the status of WTP.
///
class WTPTable : public WTPTableInterface, public TimerListener {

private:

	///
	///	\brief Struct that contains the information of single WTP.
	///	

	struct WTPElement{
		IPv4Address ip;			// IP address of the AP
		uint32_t sid;			// Session Identifier
		EthernetMACAddress mac;		// MAC address of the AP
		StatusType status;		// Status of CAPWAP session
		string location;		// Location of WTP
		string name;			// Name of WTP
		uint8_t rid;			//Radio Identifier
		uint8_t channel;		// Channel in use
		uint32_t txpower;		// Trasmission Power
		StatusConfiguration preconfigure;// PreConfigure 
		time_t fresh;			//Field for verify fresh of WTP (update every echo request)
	};

	///
	///	\brief Vector that contains information of WTPs.
	///	
		vector <WTPElement> wtps;

		string acname;
		string wtpessid;
		IPv4Address wtpnetmask;
		IPv4Address wtpgateway;
		int outudp;
		int inudp;

	///
	///	\brief Variable to control the access to the StatusTable
	///
	///	The class AccesRight allows to user to acces to the Status Table with shared acces
	///	exclusive access. The shared one allows to other classes to acces the same Status
	///	Table with a shared access, but not in a exclusive mode. Instead, the exclusive 
	///	access forbids any other access (shared or exclusive).
	///
	AccessRights ar[MAX_WTP];

	///
	///	\brief Number of registered Plugins.
	///
	///	The number of registered Plugins to the %Dispatcher is used to create the array of
	///	pointer to PluginInfo to storage the information about each used by each Plugins.
	///
	int nPlugin;

	///
	///	\brief Timer object for automatic periodic maintenance.
	///
	CoarseTimer timer;

	///
	///	\brief Persistence file, where table is stored between program executions.
	///
	fstream persistenceFile;
	
	bool ReadPMT(string pmtfile); //Read parameters from pmtfile
	bool configurationupdate; //Set True if new read of pmt file (with WTP configuration) is required
	string pmtfile; //Path of file with CAPWAP parameters and WTP configuration imfotmation
	CapwapPluginB *cpi; // reference to interface between WTPTable and CAPWAP Plugin B

public:

	///
	///	\brief Constructor. Inizialise array of userElement.
	///	
	///	The constructor inizialize the private array users: each element of the array
	///	is a pointer to a stuct userElement. The each userElement is inizialized 
	///	with the IP address and the status is set to FREE or FORBIDDEN for reserved 
	///	IP address.
	///
	///	\param fname Name of file storing persistent info.
	///
	WTPTable(string newpmtfile);

	///
	///	\brief Destructor.
	///
	~WTPTable();

	// Inherited
	bool AddJoinRequestInformation(IPv4Address ip, 
	EthernetMACAddress mac, 
	uint32_t newsid);
	
	bool AddConfigurationStatusRequestInformation(
	IPv4Address newip,
	string newwtpname,
	string newwtplocation,
	uint8_t newrid,
	uint8_t newchannel,
	uint32_t newtxpower,
	string newessid);
	
	bool AddPreConfigurationInformation(
	IPv4Address newip, 
	string newwtpname, 
	string newwtplocation, 
	uint8_t newchannel,
	uint32_t newtxpower);
	
	bool SetStatus(IPv4Address wtpip, StatusType newstatus);
	bool SetPreConfigure(IPv4Address wtpip, StatusConfiguration newvalue);

	IPv4Address GetIP(EthernetMACAddress WTPMAC);
	uint32_t GetSID(IPv4Address WTPIP);
	EthernetMACAddress GetMAC(IPv4Address WTPIP);
	WTPTableInterface::StatusType GetStatus(IPv4Address WTPIP);
	string GetLocation(IPv4Address WTPIP);
	string GetWTPName(IPv4Address WTPIP);
	uint8_t GetRadioID(IPv4Address WTPIP);
	uint8_t GetChannel(IPv4Address WTPIP);
	uint32_t GetTxPower(IPv4Address WTPIP);
	WTPTableInterface::StatusConfiguration GetPreConfigure(IPv4Address WTPIP);
	bool DelWTP(IPv4Address WTPIP);

	bool RefreshWTP(IPv4Address WTPIP);
	
	bool ConfigurationUpdate();
	
	bool Register_capwap_plugin_B(CapwapPluginB *newpointer);
	
	int Getoutudpport();
	int Getinudpport();
	
	string GetACName();
	string GetWTPESSID();
	IPv4Address GetWTPNetmask();
	IPv4Address GetWTPGateway();
	
	///
	///	\brief Get the max number of WTP.
	///
	inline int GetMaxNumberWTP(void){return (MAX_WTP);};

	///
	///	\brief Get the number of WTP acctualy associated.
	///
	int GetNumberWTP(void);

	///
	///	\brief Notify the addition of a %Plugin in the system.
	///
	///	The function is invoked by the Plugin when it is initializes. In this way the Status Table
	///	can know how many Plugins are in the system and can create an array (userElement::*info)
	///	with right dimensions.
	///	The %StatusTable doesn't know the type of the new %Plugin.
	///	\return The index of the colum of the array reserved for the added %Plugin.
	///
	inline int registerPlugin() {return nPlugin++;};

	///
	/// \brief Start background maintenance schedule.
	///
	void start (void);

	///
	/// \brief Stop background maintenance schedule.
	///
	void stop (void);

	//
	// Inherited from TimerListener.
	//
	virtual bool timerTick (void);
};

}

#endif



