/// \file clientlist.h
/// \author Mauro Brunato
///
/// \brief Definition of authorization list.
///
/// $Id: clientlist.h,v 1.2 2005/06/16 13:22:22 cvs Exp $

/*##############################################################################
   
   Developed by NetMob Group, University of Trento, Italy

   University of Trento
   DIT - Department of Informatics and Telecommunications 
   Computer Networks and Mobility Research Program
   http://netmob.unitn.it/ 
   
   This file is part of the WilmaGate system, created as part
   of the WILMA Project, sponsored by the Autonomous
   Province of Trento and participated by ITC-irst, the University
   of Trento and Alpikom Spa. More info at

     http://www.wilmaproject.org/		

   To download the project:
      - in tarball format
        http://netmob.unitn.it/wilmagate.html
      - connect to CVS repository
        :pserver:cvs@rtm.unitn.it:/home/cvsroot
   In both previous cases, the download is password protected. To obtain
   the password you must send a mail to 

      wilmagate@dit.unitn.it

################################################################################

   The WilmaGate Licence of Use


   Copyright (c) 2003-2005
                - Universita` di Trento,
                  Dipartimento di Informatica e Telecomunicazioni (DIT)
                  Computer Networks and Mobility Research Program


   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (hereinafter
   the "WilmaGate") directly from the copyright owner, to use the WilmaGate
   without restriction of purpose or time, including without limitation the
   rights to use, copy, modify, and merge the WilmaGate, subject to the
   following conditions:

   The above copyright notice and this permission notice shall be included
   in all copies or substantial portions of the WilmaGate. 

   The WilmaGate shall not be sub-licenced either commercially or free of
   charge, as is or in part or as a part of any other package. 

   The licensee agrees to properly credit WilmaGate and its copyright 
   owners in any technical, scientific or divulgative publication, 
   whatever publishing means are used, that is based entirely or in part 
   on WilmaGate use, as well as in any booklet, web-site or brochure that 
   advertise services based on WilmaGate use. 

   If the WilmaGate is modified, the copyright owner shall be notified.

   THE WilmaGate IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
   OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
   IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
   CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
   TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
   SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 

##############################################################################*/

#ifndef _H_LISTS_
#define _H_LISTS_

#include "addresses.h"
#include "access.h"

using namespace Common;

// commented in netcard.h
namespace Gateway {

	///
	/// \brief Interface for the client list towards the gateway.
	///
	/// Pure virtual class containing the functions that the gateway is authorized to use
	/// when checking for the authorization status of a client, given an IP and a MAC address.
	///
	class ClientListInterface {
	public:
		///
		/// \brief Destructor.
		///
		/// While it does nothing, it is being declared as virtual in order
		/// to shut up the compiler, who would complain for its absence.
		///
		virtual ~ClientListInterface (void);
		///
		/// \brief Get the authorization flag of a given IP address.
		///
		/// \param ip IP address of the client.
		///
		/// \return true if and only if client is authorized.
		///
		virtual bool isAuthorized (IPv4Address ip) = 0;
		///
		/// \brief Get the authorization flag of a given IP/MAC address pair.
		///
		/// \param ip IP address of the client.
		/// \param mac MAC address of the client.
		///
		/// \return true if and only if client is authorized.
		///
		virtual bool isAuthorized (IPv4Address ip, const EthernetMACAddress &mac) = 0;
		///
		/// \brief Get the MAC address corresponding to the IP address.
		///
		/// \param ip IP address of the client.
		///
		/// \return Pointer to the MAC address, NULL if not authorized.
		///
		virtual const EthernetMACAddress *getMAC (IPv4Address ip) const = 0;
		///
		/// \brief Check whether the IP address is in the admitted range.
		///
		/// \param ip The IP address to check.
		///
		virtual bool inRange (IPv4Address ip) const = 0;
		///
		/// \brief Reset traffic count of a client.
		///
		virtual void resetTraffic (IPv4Address ip) = 0;
		///
		/// \brief Add traffic to a client's row.
		///
		/// \param ip IP address of the client.
		/// \param in Quantity to be added to incoming traffic.
		/// \param out Quantity to be added to outgoing traffic.
		///
		virtual void addTraffic (IPv4Address ip, uint32_t in, uint32_t out) = 0;
		///
		/// \brief Get a client's traffic.
		///
		/// \param ip IP address of the client.
		/// \param in Incoming traffic.
		/// \param out Outgoing traffic.
		///
		virtual void getTraffic (IPv4Address ip, uint64_t &in, uint64_t &out) = 0;
	};

	///
	/// \brief Client authorization table.
	///
	/// Identifies client by IP/MAC address pairs, and stores just an authorization status value
	/// for each possible IP address.
	///
	class ClientList: public ClientListInterface {
	private:
		///
		/// \brief Authorization status of an address.
		///
		enum AuthorizationType{
			FORBIDDEN,	///< \brief The corresponding IP address should never be used.
			NOT_ASSIGNED,	///< \brief Address is OK, but not authorized yet.
			ASSIGNED,	///< \brief A user has been seen and the Ethernet MAC should be valid.
			AUTHORIZED	///< \brief Address is authorized.
		};
		///
		/// \brief Authorization table.
		///
		struct ClientRow {
			IPv4Address ip;		///< \brief IP address of the array row.
			EthernetMACAddress mac;	///< \brief Last MAC address the corresponding IP address was assigned to.
			AuthorizationType auth;	///< \brief Authorization status of the corresponding IP address.
			uint64_t intraffic;	///< \brief Incoming traffic in bytes.
			uint64_t outtraffic;	///< \brief Outgoing traffic in bytes.
		} table[256];
		///
		/// \brief First admitted IP address in host byte order (for fast range checking).
		///
		uint32_t fromIP;
		///
		/// \brief Last admitted IP address in host byte order (for fast range checking).
		///
		uint32_t toIP;
		///
		/// \brief LAN broadcast IP address in host byte order (for fast range checking).
		///
		uint32_t broadIP;
		///
		/// \brief Access synchronization object.
		///
		/// The client list is dynamically modified during normal operation;
		/// read and write operations occur concurrently, so it is necessary
		/// to ensure exclusive access to write operations.
		///
		mutable AccessRights access;
	public:
		///
		/// \brief Constructor of the authorization table.
		///
		/// \param from First IP address.
		/// \param to Last IP address.
		///
		/// \pre The two IP addresses should belong to the same C-class subnet.
		/// \post All rows corresponding to addresses in the from-to range (inclusive)
		/// are set to NOT_AUTHORIZED, all others are set to FORBIDDEN.
		///
		ClientList (IPv4Address from, IPv4Address to);
		///
		/// \brief Set or reset the authorization flag of a given IP/MAC address pair.
		///
		/// \param ip IP address of the client to be (un)authorized.
		/// \param mac MAC address of the client.
		/// \param set true if client is to be authorized, false otherwise.
		///
		/// \return true if everything works, false if MAC does not correspond to IP.
		///
		bool setAuthorization (IPv4Address ip, const EthernetMACAddress &mac, bool set);
		//
		// Inherited from ClientInterface.
		//
		~ClientList (void);
		bool isAuthorized (IPv4Address ip, const EthernetMACAddress &mac);
		bool isAuthorized (IPv4Address ip);
		const EthernetMACAddress *getMAC (IPv4Address ip) const;
		bool inRange (IPv4Address ip) const;
		void resetTraffic (IPv4Address ip);
		void addTraffic (IPv4Address ip, uint32_t in, uint32_t out);
		void getTraffic (IPv4Address ip, uint64_t &in, uint64_t &out);
	};

} // namespace Gateway

#endif
