/// \file command.cpp
///
/// \brief Network commands management related to gateway configuration
/// (user list in particular).
///
/// $Id: command.cpp,v 1.2 2005/06/16 13:22:22 cvs Exp $

/*##############################################################################
   
   Developed by NetMob Group, University of Trento, Italy

   University of Trento
   DIT - Department of Informatics and Telecommunications 
   Computer Networks and Mobility Research Program
   http://netmob.unitn.it/ 
   
   This file is part of the WilmaGate system, created as part
   of the WILMA Project, sponsored by the Autonomous
   Province of Trento and participated by ITC-irst, the University
   of Trento and Alpikom Spa. More info at

     http://www.wilmaproject.org/		

   To download the project:
      - in tarball format
        http://netmob.unitn.it/wilmagate.html
      - connect to CVS repository
        :pserver:cvs@rtm.unitn.it:/home/cvsroot
   In both previous cases, the download is password protected. To obtain
   the password you must send a mail to 

      wilmagate@dit.unitn.it

################################################################################

   The WilmaGate Licence of Use


   Copyright (c) 2003-2005
                - Universita` di Trento,
                  Dipartimento di Informatica e Telecomunicazioni (DIT)
                  Computer Networks and Mobility Research Program


   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (hereinafter
   the "WilmaGate") directly from the copyright owner, to use the WilmaGate
   without restriction of purpose or time, including without limitation the
   rights to use, copy, modify, and merge the WilmaGate, subject to the
   following conditions:

   The above copyright notice and this permission notice shall be included
   in all copies or substantial portions of the WilmaGate. 

   The WilmaGate shall not be sub-licenced either commercially or free of
   charge, as is or in part or as a part of any other package. 

   The licensee agrees to properly credit WilmaGate and its copyright 
   owners in any technical, scientific or divulgative publication, 
   whatever publishing means are used, that is based entirely or in part 
   on WilmaGate use, as well as in any booklet, web-site or brochure that 
   advertise services based on WilmaGate use. 

   If the WilmaGate is modified, the copyright owner shall be notified.

   THE WilmaGate IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
   OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
   IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
   CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
   TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
   SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 

##############################################################################*/

#include "utilities.h"
#include <sstream>

#include "command.h"

using namespace std;
using namespace Common;

static bool tokenize (char *message, int size, bool &enable, IPv4Address &ip, EthernetMACAddress &mac);

namespace Gateway {

	Command::Command (ClientList &l, IPv4Address lip, uint16_t lport, IPv4Address ip, uint16_t port):
		list (l),
		reportIP (ip),
		reportPort (port)
	{
		for ( int i = 0; i < 256; socketList[i++] = NULL);
		listener.bind (lip, lport);
		listener.registerConnectionListener (*this, 0);
		timer.registerListener (this);
		timer.start (120);
	}

	Command::~Command (void)
	{
		listener.stopListening ();
	}

	void Command::start (void)
	{
		listener.startListening ();
	}

	PacketListener::SuggestedAction Command::failed (int socketID)
	{
		socketList[socketID-1] = NULL;
		return socketID ? DELETE : STOP;
	}

	PacketListener::SuggestedAction Command::receivedPacket (uint8_t *message, int size, int socketID, IPv4Address senderIP,
							uint16_t senderPort)
	{
		if ( socketID ) {
			IPv4Address ip;
			EthernetMACAddress mac;
			bool enable;
			message[size] = '\0';
			if ( tokenize (reinterpret_cast<char*>(message), size, enable, ip, mac) ) {
				ostringstream oss;
				if ( list.setAuthorization (ip, mac, enable) ) {
					logger() << "Auth for " << ip << " (" << mac << ") set to " << enable << endl;
					oss << "OK " << ip << '\t' << mac << '\t' << enable << '\t';
					if ( enable )
						oss << "0\t0";
					else {
						uint64_t in, out;
						list.getTraffic (ip, in, out);
						list.resetTraffic (ip);
						oss << in << '\t' << out;
					}
				}
				else 
					oss << "NO - Error in authorizing IP " << ip;
				oss << endl;
				socketList[socketID-1]->send (oss.str().c_str(), oss.str().size());
			}
			else {
				socketList[socketID-1] = NULL;
				return DELETE;
			}
		}
		return CONTINUE;
	}

	PacketListener::SuggestedAction Command::incomingConnection (TCPSocket *s, int socketID)
	{
		int i;
		for ( i = 0; i < 256 && socketList[i]; i++ );
		if ( i == 256 )
			return REFUSE;
		socketList[i] = s;
		s->registerListener (*this, i+1);
		s->startReceiving ();
		return CONTINUE;
	}

	void Command::connectionOpen (TCPSocket *sk)
	{
		istringstream iss;
		string s = "list\n";
		sk->send (s.c_str(), s.length());
		while ( sk->readString (s) && s.length() > 0 ) {
			iss.str (s);
			IPv4Address ip;
			EthernetMACAddress mac;
			iss >> ip >> mac;
			list.setAuthorization (ip, mac, true);
		}
		s = "exit\n";
		sk->send (s.c_str(), s.length());
	}

	bool Command::getAuthorizations (void)
	{
		logger() << "Synching with Gatekeeper's list\n";
		TCPSocket *s = new TCPSocket ();
		if ( ! s->setDestination (reportIP, reportPort) )
			return false;
		s->registerStreamListener (*this);
		s->startSyncReceiving ();
		s->wait ();
		return true;
	}

	bool Command::timerTick (void)
	{
		getAuthorizations ();
		return true;
	}

} // namespace Gateway

static bool tokenize (char *message, int size, bool &enable, IPv4Address &ip, EthernetMACAddress &mac)
{
	// Size is used to prevent buffer overflows.
	char *buffer = new char [size];
	istringstream s (message);
	s >> buffer;
	if ( !s ) {
		delete buffer;
		return false;
	}
	enable = ! strcmp (buffer, "enable");
	delete buffer;
	s >> ip;
	if ( !s )
		return false;
	if ( enable ) {
		s >> mac;
		if ( !s )
			return false;
	}
	return true;
}

