/// \file layer2capture.h
/// \author Mauro Brunato
///
/// \brief Packet capture classes.
///
/// The only explicit references to libpcap functions and data types are
/// collected within the layer2.cpp file in order to ensure compatibility
/// in Cygwin systems (no mixture with libc functions allowed).
///
/// $Id: layer2capture.h,v 1.3 2005/12/13 17:18:53 cvs Exp $

/*##############################################################################
   
   Developed by NetMob Group, University of Trento, Italy

   University of Trento
   DIT - Department of Informatics and Telecommunications 
   Computer Networks and Mobility Research Program
   http://netmob.unitn.it/ 
   
   This file is part of the WilmaGate system, created as part
   of the WILMA Project, sponsored by the Autonomous
   Province of Trento and participated by ITC-irst, the University
   of Trento and Alpikom Spa. More info at

     http://www.wilmaproject.org/		

   To download the project:
      - in tarball format
        http://netmob.unitn.it/wilmagate.html
      - connect to CVS repository
        :pserver:cvs@rtm.unitn.it:/home/cvsroot
   In both previous cases, the download is password protected. To obtain
   the password you must send a mail to 

      wilmagate@dit.unitn.it

################################################################################

   The WilmaGate Licence of Use


   Copyright (c) 2003-2005
                - Universita` di Trento,
                  Dipartimento di Informatica e Telecomunicazioni (DIT)
                  Computer Networks and Mobility Research Program


   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (hereinafter
   the "WilmaGate") directly from the copyright owner, to use the WilmaGate
   without restriction of purpose or time, including without limitation the
   rights to use, copy, modify, and merge the WilmaGate, subject to the
   following conditions:

   The above copyright notice and this permission notice shall be included
   in all copies or substantial portions of the WilmaGate. 

   The WilmaGate shall not be sub-licenced either commercially or free of
   charge, as is or in part or as a part of any other package. 

   The licensee agrees to properly credit WilmaGate and its copyright 
   owners in any technical, scientific or divulgative publication, 
   whatever publishing means are used, that is based entirely or in part 
   on WilmaGate use, as well as in any booklet, web-site or brochure that 
   advertise services based on WilmaGate use. 

   If the WilmaGate is modified, the copyright owner shall be notified.

   THE WilmaGate IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
   OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
   IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
   CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
   TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
   SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 

##############################################################################*/

#ifndef _H_LAYER2CAPTURE_
#define _H_LAYER2CAPTURE_

#include <inttypes.h>
#include "exceptions.h"

//
// The structure is implemented in Common/packets.h.
// The file cannot be included because it clashes with the
// WIN32 version of libpcap, which is used by the corresponding
// cpp file.
//
namespace Common {
	class EthernetFrame;
}

using namespace Common;

// commented in netcard.h
namespace Gateway {

	////////////////////////////////////////////////////
	///
	/// \brief Exception raised for critical events in the Layer2Capture class hierarchy
	///
	class CaptureException:
		public WilmaGateException {
	public:
		///
		/// \brief Constructor.
		///
		/// \param c Subcode of the exception.
		///
		inline CaptureException (int c);
	};

	inline CaptureException::CaptureException (int c):
		WilmaGateException ("Capture interface", c)
	{
	}

	////////////////////////////////////////////////////////////
	///
	/// \brief Container for a Layer2 class callback function.
	///
	/// Pure virtual interface. All classes that need access to captured packets
	/// must implement this interface.
	///
	class Layer2Listener {
	public:
		///
		/// \brief Virtual destructor.
		///
		virtual ~Layer2Listener (void);
		///
		/// \brief Callback function, invoked when a packet is captured.
		///
		/// Pure virtual function, called for every captured frame.
		///
		/// \param eth Pointer to a read-only Ethernet frame.
		/// \param l Length of the frame on the wire.
		///
		virtual void capturedFrame (EthernetFrame *eth, int l) = 0;
	};

	///////////////////////////////////////////////////////////
	///
	/// \brief Layer 2 intercept functions.
	///
	class Layer2Capture {
	public:
		///
		/// \brief Error states after object construction.
		///
		enum StatusType {
			GOOD = 0,	///< \brief No error encountered.
			LOOKUP = 1,	///< \brief Lookup function did not work.
			NODEVICE = 2,	///< \brief Device was not returned in list.
			BADOPEN = 3	///< \brief Could not open the device.
		};
		///
		/// \brief Infinite number of captures.
		///
		static const int INFINITE_PACKETS = -1;
	private:
		///
		/// \brief Pointer to the capture interface.
		///
		/// It is masked as a void pointer for compatibility, and will
		/// be cast to pcap_t* whenever it is necessary in layer2capture.cpp.
		///
		void *pt;
	public:
		///
		/// \brief Constructor: locate and open an interface given its name.
		///
		/// This is simply a wrapper function that executes all necessary calls to the
		/// pcap library.
		///
		/// \param ifname The name of the interface.
		///
		Layer2Capture (const char *ifname) throw (CaptureException);
		///
		/// \brief Destructor: close the open interface
		///
		~Layer2Capture ();
		///
		/// \brief Packet interception loop.
		///
		/// All intercepted packets are routed to the Layer2Listener::capturedPacket
		/// method of the passed object.
		///
		/// \param l The listening object.
		/// \param n Maximum number of packets to capture (or Layer2Capture::INFINITE_PACKETS)
		///
		void intercept (Layer2Listener *l, int n);
		///
		/// \brief Intercept one packet in nonblocking mode.
		///
		/// The intercepted packet is routed to the Layer2Listener::capturedPacket
		/// method of the passed object.
		///
		/// \param l The listening object.
		///
		void getNext (Layer2Listener *l);
		///
		/// \brief Print names and descriptions of all interfaces that can be opened.
		///
		static void listInterfaces (void) throw (CaptureException);
	};

} // namespace Gateway

#endif
