/// \file layer2send.h
/// \author Mauro Brunato
///
/// \brief Ethernet frame sending classes.
///
/// The only explicit references to libpcap functions and data types are
/// collected within the layer2.cpp file in order to ensure compatibility
/// in Cygwin systems (no mixture with libc functions allowed).
///
/// $Id: layer2send.h,v 1.2 2005/06/16 13:22:22 cvs Exp $

/*##############################################################################
   
   Developed by NetMob Group, University of Trento, Italy

   University of Trento
   DIT - Department of Informatics and Telecommunications 
   Computer Networks and Mobility Research Program
   http://netmob.unitn.it/ 
   
   This file is part of the WilmaGate system, created as part
   of the WILMA Project, sponsored by the Autonomous
   Province of Trento and participated by ITC-irst, the University
   of Trento and Alpikom Spa. More info at

     http://www.wilmaproject.org/		

   To download the project:
      - in tarball format
        http://netmob.unitn.it/wilmagate.html
      - connect to CVS repository
        :pserver:cvs@rtm.unitn.it:/home/cvsroot
   In both previous cases, the download is password protected. To obtain
   the password you must send a mail to 

      wilmagate@dit.unitn.it

################################################################################

   The WilmaGate Licence of Use


   Copyright (c) 2003-2005
                - Universita` di Trento,
                  Dipartimento di Informatica e Telecomunicazioni (DIT)
                  Computer Networks and Mobility Research Program


   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (hereinafter
   the "WilmaGate") directly from the copyright owner, to use the WilmaGate
   without restriction of purpose or time, including without limitation the
   rights to use, copy, modify, and merge the WilmaGate, subject to the
   following conditions:

   The above copyright notice and this permission notice shall be included
   in all copies or substantial portions of the WilmaGate. 

   The WilmaGate shall not be sub-licenced either commercially or free of
   charge, as is or in part or as a part of any other package. 

   The licensee agrees to properly credit WilmaGate and its copyright 
   owners in any technical, scientific or divulgative publication, 
   whatever publishing means are used, that is based entirely or in part 
   on WilmaGate use, as well as in any booklet, web-site or brochure that 
   advertise services based on WilmaGate use. 

   If the WilmaGate is modified, the copyright owner shall be notified.

   THE WilmaGate IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
   OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
   IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
   CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
   TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
   SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 

##############################################################################*/

#ifndef _H_LAYER2SEND_
#define _H_LAYER2SEND_

#ifndef WINDOWSENV
#include <netpacket/packet.h>
#endif

#include "exceptions.h"

//
// The structure is implemented in Common/packets.h.
// The file cannot be included because it clashes with the
// WIN32/Cygwin version of libpcap, which is used by the corresponding
// cpp file.
//
namespace Common {
	class EthernetFrame;
}

using namespace Common;

// commented in netcard.h
namespace Gateway {

	////////////////////////////////////////////////////
	///
	/// \brief Exception raised for critical events in the Layer2Send class.
	///
	class SendException:
		public WilmaGateException {
	public:
		///
		/// \brief Constructor.
		///
		/// \param c Subcode of the exception.
		///
		inline SendException (int c);
	};

	inline SendException::SendException (int c):
		WilmaGateException ("Send interface", 1)
	{
	}

	//////////////////////////////////////////////////////////////////////////////////
	///
	/// \brief Raw ethernet packet sending.
	///
	/// Wrapper class for sending raw Ethernet packets. Its implementation
	/// significantly differs between WIN32/Cygwin and Linux systems, because
	/// GNU/Linux provides raw Ethernet sockets, while under Cygwin we need to
	/// use an appropriate winpcap extension, which is not available in the
	/// ordinary GNU libpcap.
	///
	class Layer2Send {
	public:
		///
		/// \brief Error states after object construction.
		///
		enum StatusType {
			GOOD = 0,	///< \brief No error encountered.
			LOOKUP = 1,	///< \brief Lookup function did not work.
			NODEVICE = 2,	///< \brief Device was not returned in list.
			BADOPEN = 3	///< \brief Could not open the device.
		};
	private:
	#ifdef WINDOWSENV
		///
		/// \brief [WIN32/Cygwin version] Pointer to the capture interface.
		///
		/// It is masked as a void pointer for compatibility, and will
		/// be cast to pcap_t* whenever it is necessary in layer2send.cpp.
		///
		void *pt;
	#else
		///
		/// \brief [GNU/Linux version] Raw Ethernet socket.
		///
		int skt;
		///
		/// \brief [GNU/Linux version] Address structure for the Ethernet socket.
		sockaddr_ll addr;
	#endif
	public:
		///
		/// \brief Constructor.
		///
		/// \param ifname The Interface system name.
		/// \param index The interface index (only used in the GNU/Linux version, 0 in WIN32/Cygwin).
		///
		Layer2Send (const char *ifname, int index) throw (SendException);
		///
		/// \brief Destructor.
		///
		~Layer2Send (void);
		///
		/// \brief Send an Ethernet frame.
		///
		/// \param f The Ethernet frame, completed with the header.
		/// \param l Total length of the frame.
		///
		void send (EthernetFrame *f, int l) throw (SendException);
	};

} // namespace Gateway

#endif
