/// \file netcard.cpp
/// \author Mauro Brunato
///
/// \brief Implementation of NetworkCard class.
///
/// $Id: netcard.cpp,v 1.4 2005/12/22 12:17:26 cvs Exp $

/*##############################################################################
   
   Developed by NetMob Group, University of Trento, Italy

   University of Trento
   DIT - Department of Informatics and Telecommunications 
   Computer Networks and Mobility Research Program
   http://netmob.unitn.it/ 
   
   This file is part of the WilmaGate system, created as part
   of the WILMA Project, sponsored by the Autonomous
   Province of Trento and participated by ITC-irst, the University
   of Trento and Alpikom Spa. More info at

     http://www.wilmaproject.org/		

   To download the project:
      - in tarball format
        http://netmob.unitn.it/wilmagate.html
      - connect to CVS repository
        :pserver:cvs@rtm.unitn.it:/home/cvsroot
   In both previous cases, the download is password protected. To obtain
   the password you must send a mail to 

      wilmagate@dit.unitn.it

################################################################################

   The WilmaGate Licence of Use


   Copyright (c) 2003-2005
                - Universita` di Trento,
                  Dipartimento di Informatica e Telecomunicazioni (DIT)
                  Computer Networks and Mobility Research Program


   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (hereinafter
   the "WilmaGate") directly from the copyright owner, to use the WilmaGate
   without restriction of purpose or time, including without limitation the
   rights to use, copy, modify, and merge the WilmaGate, subject to the
   following conditions:

   The above copyright notice and this permission notice shall be included
   in all copies or substantial portions of the WilmaGate. 

   The WilmaGate shall not be sub-licenced either commercially or free of
   charge, as is or in part or as a part of any other package. 

   The licensee agrees to properly credit WilmaGate and its copyright 
   owners in any technical, scientific or divulgative publication, 
   whatever publishing means are used, that is based entirely or in part 
   on WilmaGate use, as well as in any booklet, web-site or brochure that 
   advertise services based on WilmaGate use. 

   If the WilmaGate is modified, the copyright owner shall be notified.

   THE WilmaGate IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
   OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
   IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
   CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
   TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
   SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 

##############################################################################*/

#include "utilities.h"
#include <pthread.h>
#include <unistd.h>

#include "netcard.h"

using namespace Common;

namespace Gateway {

	NetworkCard::NetworkCard (string &ifn, int ifi,
				RouterInterface &t, Rule::SenseType s,
				IPv4Address ip, EthernetMACAddress &mac,
				IPv4Address net, IPv4Address mask,
				IPv4Address arpnet, IPv4Address arpmask) :
		tap (strncmp(ifn.c_str(),"tap",3) ? NULL : ifn.c_str()),
		cap (tap.ok() ? NULL : ifn.c_str()),
		snd (tap.ok() ? NULL : ifn.c_str(), ifi),
		router (t),
		sense (s),
		myIP (ip),
		myMAC (mac),
		netIP (net),
		netmask (mask),
		arpIP (arpnet),
		arpMask (arpmask)
	{
		router.registerListener (this);
		if ( tap.ok() ) {
			tap.registerListener (this);
		}
	}

	void NetworkCard::start (void)
	{
		if ( tap.ok() ) {
			tap.startReceiving ();
			status = PACKET_HANDLING;
		}
		else
			pthread_create (&thread, NULL, threadFunction, this);
	}

	void NetworkCard::stop (void)
	{
		pthread_cancel (thread);
	}

	void NetworkCard::capturedFrame (EthernetFrame *eth, int l)
	{
		switch ( status ) {
		case GATEWAY_SEARCH:
			if ( eth->isProtocol (EthernetFrame::ARP_PROTOCOL)
					&& eth->ARPPayload().opcode == htons (ARPPacket::ANSWER)
					&& eth->ARPPayload().senderIP == gwIP ) {
				gwMAC = eth->ARPPayload().senderMAC;
				gwFound = true;
			}
			break;
		case PACKET_HANDLING:
			if ( eth->sender == myMAC )
				return;
//logger()<<"From:"<<eth->sender<<" To:"<<eth->receiver<<" Protocol:"<<hex<<ntohs(eth->protocol)<<'\n';
			if ( eth->isProtocol (EthernetFrame::ARP_PROTOCOL)
					&& eth->ARPPayload().opcode == htons (ARPPacket::QUERY)
					&& (eth->ARPPayload().receiverIP & arpMask) == arpIP ) {
				EthernetFrame *e = new (sizeof (ARPPacket)) EthernetFrame;
				e->sender = myMAC;
				e->receiver = eth->sender;
				e->protocol = eth->protocol;
				e->ARPPayload().forgeResponse (eth->ARPPayload(), myMAC);
//logger()<<"ARP\n";
				if ( tap.ok() )
					tap.sendFrame (e, 14 + sizeof(ARPPacket));
				else
					snd.send (e, 14 + sizeof(ARPPacket));
			}
			else if ( eth->isProtocol (EthernetFrame::IP_PROTOCOL) ) {
				if ( eth->IPPayload().destination == myIP )
					return;
				router.sendFrame (eth, l, sense);
			}
			break;
		}
	}

	///
	/// \brief Thread function for the asynchronous packet capture mechanism.
	///
	void *threadFunction (void *p)
	{
		NetworkCard *& that = reinterpret_cast<NetworkCard*&>(p);
		that->status = NetworkCard::PACKET_HANDLING;
		that->cap.intercept (that, Layer2Capture::INFINITE_PACKETS);
		return NULL;
	}

	void NetworkCard::locateGateway (IPv4Address ip)
		throw (GatewayNotFoundException)
	{
		if ( tap.ok() ) {
			gwMAC = myMAC;
			myMAC = "00:01:01:01:01:01";
		}
		else {
			gwIP = ip;
			EthernetFrame *e = new (sizeof(ARPPacket)) EthernetFrame;
			e->sender = myMAC;
			e->receiver = "ff:ff:ff:ff:ff:ff";
			e->protocol = htons(EthernetFrame::ARP_PROTOCOL);
			e->ARPPayload().forgeRequest (myIP, ip, myMAC);
			status = GATEWAY_SEARCH;
			gwFound = false;
			time_t stt = time (NULL);
			do {
				if ( tap.ok() ) {
					tap.sendFrame (e, 14+sizeof(ARPPacket));
					tap.getNext (this);
				}
				else {
					snd.send (e, 14+sizeof(ARPPacket));
					cap.getNext (this);
				}
				if ( !gwFound )
					usleep (500000);
			} while ( !gwFound && time(NULL)-stt < 10 );
			delete e;
			if ( !gwFound )
				throw GatewayNotFoundException();
		}
	}

	bool NetworkCard::receiveFrame (EthernetFrame *eth, int l)
	{
		if ( (eth->IPPayload().destination & netmask) != netIP
				&& (sense == Rule::OUTER
					|| eth->IPPayload().protocol != IPPacket::UDP_PROTOCOL
					|| eth->IPPayload().UDPPayload().sourcePort != htons (0x0043)) )
			return false;
		if ( sense == Rule::OUTER )
			eth->receiver = gwMAC;
		eth->sender = myMAC;
		if ( tap.ok() )
			tap.sendFrame (eth, l);
		else
			snd.send (eth, l);
		return true;
	}

	Rule::SenseType NetworkCard::getSense (void) const
	{
		return sense;
	}

} // namespace Gateway
