/// \file netcard.h
/// \author Mauro Brunato
///
/// \brief Layer 2 networking interface abstractions
///
/// $Id: netcard.h,v 1.3 2005/12/13 17:18:53 cvs Exp $

/*##############################################################################
   
   Developed by NetMob Group, University of Trento, Italy

   University of Trento
   DIT - Department of Informatics and Telecommunications 
   Computer Networks and Mobility Research Program
   http://netmob.unitn.it/ 
   
   This file is part of the WilmaGate system, created as part
   of the WILMA Project, sponsored by the Autonomous
   Province of Trento and participated by ITC-irst, the University
   of Trento and Alpikom Spa. More info at

     http://www.wilmaproject.org/		

   To download the project:
      - in tarball format
        http://netmob.unitn.it/wilmagate.html
      - connect to CVS repository
        :pserver:cvs@rtm.unitn.it:/home/cvsroot
   In both previous cases, the download is password protected. To obtain
   the password you must send a mail to 

      wilmagate@dit.unitn.it

################################################################################

   The WilmaGate Licence of Use


   Copyright (c) 2003-2005
                - Universita` di Trento,
                  Dipartimento di Informatica e Telecomunicazioni (DIT)
                  Computer Networks and Mobility Research Program


   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (hereinafter
   the "WilmaGate") directly from the copyright owner, to use the WilmaGate
   without restriction of purpose or time, including without limitation the
   rights to use, copy, modify, and merge the WilmaGate, subject to the
   following conditions:

   The above copyright notice and this permission notice shall be included
   in all copies or substantial portions of the WilmaGate. 

   The WilmaGate shall not be sub-licenced either commercially or free of
   charge, as is or in part or as a part of any other package. 

   The licensee agrees to properly credit WilmaGate and its copyright 
   owners in any technical, scientific or divulgative publication, 
   whatever publishing means are used, that is based entirely or in part 
   on WilmaGate use, as well as in any booklet, web-site or brochure that 
   advertise services based on WilmaGate use. 

   If the WilmaGate is modified, the copyright owner shall be notified.

   THE WilmaGate IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
   OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
   IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
   CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
   TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
   SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 

##############################################################################*/

#ifndef _H_NET_CARD_
#define _H_NET_CARD_

#include <inttypes.h>

#include "router.h"
#include "layer2capture.h"
#include "layer2send.h"
#include "tapinterface.h"
#include "exceptions.h"
#include "addresses.h"

using namespace Common;

///
/// \brief Entities used for the gateway program.
///
namespace Gateway {

	////////////////////////////////////////////////////
	///
	/// \brief Exception raised if external gateway is not found
	///
	class GatewayNotFoundException:
		public WilmaGateException {
	public:
		///
		/// \brief Constructor.
		///
		inline GatewayNotFoundException (void);
	};

	inline GatewayNotFoundException::GatewayNotFoundException (void):
		WilmaGateException ("External gateway not found", 1)
	{
	}

	//////////////////////////////////////////////////////////////
	///
	/// \brief Handler class for managing a layer 2 network interface.
	///
	/// This class is the abstraction of a layer-2 networking interface.
	/// It extends the pure virtual interface Layer2Listener,
	/// which defines a callback for layer 2 captured packets,
	/// and the pure virtual interface RouterListener,
	/// defining a callback for outgoing packets.
	///
	class NetworkCard:
			public Layer2Listener,
			public RouterListener
	{
		friend void *threadFunction (void *p);
	private:
		///
		/// \brief Status of the class with respect to the capture function.
		///
		enum StatusType {
			GATEWAY_SEARCH,	///< \brief Looking for a gateway (waiting for an ARP response only).
			PACKET_HANDLING	///< \brief Regularly handling packets (normal operation).
		} status;
		///
		///
		///
		TapInterface tap;
		///
		/// \brief Layer 2 capture object.
		///
		Layer2Capture cap;
		///
		/// \brief Layer 2 send object.
		///
		Layer2Send snd;
		///
		/// \brief Object towards which gatekeeper packets must be directed.
		///
		RouterInterface &router;
		///
		/// \brief Direction of the significant packets.
		///
		const Rule::SenseType sense;
		///
		/// \brief Thread handle.
		///
		/// Most of the object lifetime is spent in asynchronous execution within a packet capture loop.
		///
		pthread_t thread;
		///
		/// \brief IP of the interface.
		///
		const IPv4Address myIP;
		///
		/// \brief MAC address of the Layer2 interface.
		///
		EthernetMACAddress myMAC;
		///
		/// \brief IP of the managed network.
		///
		const IPv4Address netIP;
		///
		/// \brief Mask of the managed network.
		///
		const IPv4Address netmask;
		///
		/// \brief IP of the ARP answer domain.
		///
		/// The object will directly respond with ARP packets
		/// to all queries for addresses of this subnetwork.
		///
		const IPv4Address arpIP;
		///
		/// \brief Mask of the ARP answer domain.
		///
		const IPv4Address arpMask;
		///
		/// \brief IP address of the gateway (only useful if outbound interface)
		///
		IPv4Address gwIP;
		///
		/// \brief MAC address of the gateway (only useful if outbound interface)
		///
		EthernetMACAddress gwMAC;
		///
		/// \brief True if gateway has sent its ARP response and NetworkCard::gwMAC
		/// contains a valid MAC address.
		///
		bool gwFound;
	public:
		///
		/// \brief Constructor: set parameters.
		///
		/// \param ifn Reference to the interface string
		/// \param ifi Index of the interface.
		/// \param t Interface object to the gateway router object.
		/// \param s Sense of the interface (see NetworkCard::sense for exact meaning).
		/// \param ip IP address of the interface.
		/// \param mac MAC address of the interface.
		/// \param net Subnet managed by the network card.
		/// \param mask Subnet mask managed by the network card.
		/// \param arpnet ARP-responding subnet (see NetworkCard::arpIP for details)
		/// \param arpmask Corresponding netmask.
		///
		NetworkCard (string &ifn, int ifi, RouterInterface &t, Rule::SenseType s,
				IPv4Address ip, EthernetMACAddress &mac, IPv4Address net, IPv4Address mask,
				IPv4Address arpnet, IPv4Address arpmask);
		///
		/// \brief Given an IP address, locate the gateway's ethernet address.
		///
		/// Locates the MAC address of the gateway and sets it into the corresponding Layer2Send interface.
		///
		/// \param ip [in] IP address of the gateway.
		///
		/// \remark This function uses the packet capture function; however, it is executed in a synchronous fashion,
		/// and blocks the program for a short while (or until a timeout expires). Valid only
		/// on outbound interfaces.
		///
		void locateGateway (IPv4Address ip) throw (GatewayNotFoundException);
		///
		/// \brief Start the asynchronous packet capture thread.
		///
		void start (void);
		///
		/// \brief Stop the thread executing the asynchronous packet capture.
		///
		void stop (void);
		///
		/// \brief Return the gateway MAC address
		///
		inline const EthernetMACAddress &gatewayMAC (void) const;
		///
		/// \brief Wait for the running thread to yield.
		///
		inline void wait (void);
		//
		// Inherited from Layer2Listener
		//
		virtual void capturedFrame (EthernetFrame *eth, int l);
		//
		// Inherited from RouterListener
		//
		virtual Rule::SenseType getSense (void) const;
		virtual bool receiveFrame (EthernetFrame *eth, int l);
	};

	inline const EthernetMACAddress &NetworkCard::gatewayMAC (void) const
	{
		return gwMAC;
	}

	inline void NetworkCard::wait (void)
	{
		pthread_join (thread, NULL);
	}

} // namespace Gateway

#endif
