/// \file router.h
/// \author Mauro Brunato
///
/// \brief Classes related to routing and the gatekeeper tunnel
///
/// $Id: router.h,v 1.4 2005/12/13 17:18:53 cvs Exp $

/*##############################################################################
   
   Developed by NetMob Group, University of Trento, Italy

   University of Trento
   DIT - Department of Informatics and Telecommunications 
   Computer Networks and Mobility Research Program
   http://netmob.unitn.it/ 
   
   This file is part of the WilmaGate system, created as part
   of the WILMA Project, sponsored by the Autonomous
   Province of Trento and participated by ITC-irst, the University
   of Trento and Alpikom Spa. More info at

     http://www.wilmaproject.org/		

   To download the project:
      - in tarball format
        http://netmob.unitn.it/wilmagate.html
      - connect to CVS repository
        :pserver:cvs@rtm.unitn.it:/home/cvsroot
   In both previous cases, the download is password protected. To obtain
   the password you must send a mail to 

      wilmagate@dit.unitn.it

################################################################################

   The WilmaGate Licence of Use


   Copyright (c) 2003-2005
                - Universita` di Trento,
                  Dipartimento di Informatica e Telecomunicazioni (DIT)
                  Computer Networks and Mobility Research Program


   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (hereinafter
   the "WilmaGate") directly from the copyright owner, to use the WilmaGate
   without restriction of purpose or time, including without limitation the
   rights to use, copy, modify, and merge the WilmaGate, subject to the
   following conditions:

   The above copyright notice and this permission notice shall be included
   in all copies or substantial portions of the WilmaGate. 

   The WilmaGate shall not be sub-licenced either commercially or free of
   charge, as is or in part or as a part of any other package. 

   The licensee agrees to properly credit WilmaGate and its copyright 
   owners in any technical, scientific or divulgative publication, 
   whatever publishing means are used, that is based entirely or in part 
   on WilmaGate use, as well as in any booklet, web-site or brochure that 
   advertise services based on WilmaGate use. 

   If the WilmaGate is modified, the copyright owner shall be notified.

   THE WilmaGate IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
   OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
   IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
   CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
   TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
   SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 

##############################################################################*/

#ifndef _ROUTER_H_
#define _ROUTER_H_

#include "packets.h"
#include "socket.h"
#include "clientlist.h"
#include "rules.h"

using namespace Common;

// Commented in netcard.h
namespace Gateway {

	////////////////////////////////////////////////////////////////////
	///
	/// \brief Receive-only interface to the router.
	///
	/// Used by the Interceptor class in order to set a receive callback.
	/// Many interceptors may be enlisted within the router to use this callback.
	///
	class RouterListener {
	public:
		///
		/// \brief Virtual destructor.
		///
		virtual ~RouterListener (void) {}
		///
		/// \brief Return the sense of the listener interface.
		///
		virtual Rule::SenseType getSense (void) const = 0;
		///
		/// \brief An encapsulated Ethernet frame has been received
		/// and must be forwarded.
		///
		/// \param eth A pointer to the received frame.
		/// \param l Length of the frame.
		///
		/// \return True iff the interceptor has sent the packet.
		///
		virtual bool receiveFrame (EthernetFrame *eth, int l) = 0;
	};

	////////////////////////////////////////////////////////////////////
	///
	/// \brief Send-only interface to the router.
	///
	/// Used by the Interceptor class in order to send packets and register
	/// for listening to a router interface without the burden of other functionalities.
	///
	class RouterInterface {
	public:
		///
		/// \brief Virtual destructor.
		///
		virtual ~RouterInterface (void) {}
		///
		/// \brief Send an Ethernet frame to the router.
		///
		/// \param eth A pointer to the frame to be sent.
		/// \param l Length of the frame.
		/// \param sense Sense of the interface issuing the packet.
		///
		virtual void sendFrame (EthernetFrame *eth, int l, Rule::SenseType sense) = 0;
		///
		/// \brief Register to the router.
		///
		/// \param t The router listening class.
		///
		virtual void registerListener (RouterListener *t) = 0;
	};

	////////////////////////////////////////////////////////////////////
	///
	/// \brief Router abstraction.
	///
	/// This class represents the central routing mechanism
	///
	class Router:
		public RouterInterface,
		public PacketListener {
	private:
		///
		/// \brief Number of registered listening objects.
		///
		int numListeners;
		///
		/// \brief Array of pointers to registered listeners.
		///
		RouterListener *listeners[2];
		///
		/// \brief Authorization list.
		///
		/// The member is not const because authorizations can be changed by the router.
		///
		ClientListInterface &clients;
		///
		/// \brief Rule table.
		///
		const RuleTable &rules;
		///
		/// \brief UDP socket for gatekeeper communications.
		///
		UDPSocket gatekeeper;
		///
		/// \brief UDP socket for directing dropped frames.
		///
		UDPSocket sink;
		///
		/// \brief True iff the sink for dropped frames is enabled.
		///
		const bool sinkEnabled;
	public:
		///
		/// \brief Constructor.
		///
		/// \param c Reference to the client authorization list.
		/// \param r Reference to rule table.
		/// \param gwIP IP address of the gateway (local IP).
		/// \param gwPort UDP port of the gateway.
		/// \param gkIP IP address of the gatekeeper.
		/// \param gkPort UDP port of the gatekeeper.
		/// \param sinkIP IP address of the dropped packet sink.
		/// \param sinkPort UDP port of the dropped packet sink.
		///
		Router (ClientListInterface &c, RuleTable &r, IPv4Address gwIP, uint16_t gwPort,
				IPv4Address gkIP, uint16_t gkPort,
				IPv4Address sinkIP, uint16_t sinkPort);
		///
		/// \brief Virtual destructor.
		///
		virtual ~Router (void);
		///
		/// \brief Start the UDP socket receiver.
		///
		inline void start (void);
		//
		// Inherited from PacketListener.
		//
		virtual SuggestedAction failed (int socketID);
		virtual SuggestedAction receivedPacket (uint8_t *message, int size, int socketID, IPv4Address senderIP,
							uint16_t senderPort);
		//
		// Inherited from RouterInterface.
		//
		virtual void sendFrame (EthernetFrame *eth, int l, Rule::SenseType sense);
		virtual void registerListener (RouterListener *t);
	};

	inline void Router::start (void)
	{
		gatekeeper.startReceiving ();
	}

} // namespace Gateway

#endif
