/// \file rules.h
/// \author Mauro Brunato
///
/// \brief Definition of firewall rules and rule table.
///
/// $Id: rules.h,v 1.3 2005/12/13 17:18:53 cvs Exp $

/*##############################################################################
   
   Developed by NetMob Group, University of Trento, Italy

   University of Trento
   DIT - Department of Informatics and Telecommunications 
   Computer Networks and Mobility Research Program
   http://netmob.unitn.it/ 
   
   This file is part of the WilmaGate system, created as part
   of the WILMA Project, sponsored by the Autonomous
   Province of Trento and participated by ITC-irst, the University
   of Trento and Alpikom Spa. More info at

     http://www.wilmaproject.org/		

   To download the project:
      - in tarball format
        http://netmob.unitn.it/wilmagate.html
      - connect to CVS repository
        :pserver:cvs@rtm.unitn.it:/home/cvsroot
   In both previous cases, the download is password protected. To obtain
   the password you must send a mail to 

      wilmagate@dit.unitn.it

################################################################################

   The WilmaGate Licence of Use


   Copyright (c) 2003-2005
                - Universita` di Trento,
                  Dipartimento di Informatica e Telecomunicazioni (DIT)
                  Computer Networks and Mobility Research Program


   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (hereinafter
   the "WilmaGate") directly from the copyright owner, to use the WilmaGate
   without restriction of purpose or time, including without limitation the
   rights to use, copy, modify, and merge the WilmaGate, subject to the
   following conditions:

   The above copyright notice and this permission notice shall be included
   in all copies or substantial portions of the WilmaGate. 

   The WilmaGate shall not be sub-licenced either commercially or free of
   charge, as is or in part or as a part of any other package. 

   The licensee agrees to properly credit WilmaGate and its copyright 
   owners in any technical, scientific or divulgative publication, 
   whatever publishing means are used, that is based entirely or in part 
   on WilmaGate use, as well as in any booklet, web-site or brochure that 
   advertise services based on WilmaGate use. 

   If the WilmaGate is modified, the copyright owner shall be notified.

   THE WilmaGate IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
   OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
   IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
   CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
   TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
   SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 

##############################################################################*/

#ifndef _H_RULES_
#define _H_RULES_

#include "packets.h"

using namespace Common;

// Commented in netcard.h
namespace Gateway {

	////////////////////////////////////////////////////////////
	///
	/// \brief Firewall rule.
	///
	/// Defines a generic rule and applies it to an unauthorized packet in order
	/// to decide whether it can pass, must be dropped or must be
	/// forwarded to the GateKeeper for further analysis.
	///
	class Rule {
	public:
		///
		/// \brief Type of action to recommend to the Gateway.
		///
		enum ActionType {
			PASS,		///< \brief Let the packet pass unaltered.
			SUBMIT,		///< \brief Reroute the packet to the Gatekeeper.
			DROP,		///< \brief Drop the packet without further action.
			DONTCARE	///< \brief The packet does not match the rule (only used as return value).
		};
		///
		/// \brief Position of the network interface.
		///
		enum SenseType {
			INNER = 0,	///< \brief The interface is in the inside part (wireless).
			OUTER = 1,	///< \brief The interface is in the outside part (world).
			ANY_SENSE = 2	///< \brief Sense doesn't matter.
		};
	private:
		///
		/// \brief Type of protocol contained in the Ethernet frame.
		///
		/// \remark Protocol is stored in network byte order for faster comparison.
		///
		EthernetFrame::ProtocolType etherProto;
		///
		/// \brief Source IP address (0xFFFFFFFFUL if any).
		///
		IPv4Address sourceIP;
		///
		/// \brief Destination IP address (0xFFFFFFFFUL if any).
		///
		IPv4Address destinationIP;
		///
		/// \brief Type of protocol contained in the IP packet (IPPacket::ANY_PROTOCOL if any).
		///
		IPPacket::ProtocolType transportProto;
		///
		/// \brief Source port in the transport layer (0xFFFFUL if any).
		///
		/// \remark Port is stored in network byte order for faster comparison.
		///
		uint16_t sourcePort;
		///
		/// \brief Destination port in the transport layer (0xFFFFUL if any).
		///
		/// \remark Port is stored in network byte order for faster comparison.
		///
		uint16_t destinationPort;
		///
		/// \brief Sense of the packet (Rule::ANY_SENSE if any).
		///
		SenseType sense;
		///
		/// \brief Action to be taken for a non-authorized packet matching the rule.
		///
		ActionType noauthAction;
		///
		/// \brief Action to be taken for an authorized packet matching the rule.
		///
		ActionType authAction;
	public:
		///
		/// \brief Set the current rule.
		///
		/// \param e The protocol type as detailed in the Ethernet frame (usually IP).
		/// \param sip The source IP address (0xFFFFFFFFUL if any).
		/// \param dip The destination IP address (0xFFFFFFFFUL if any).
		/// \param i The transport protocol, in case the frame is IP (IPPacket::ANY_PROTOCOL if any).
		/// \param sp The transport layer source port, if transport is UDP or TCP (0xFFFFUL if any).
		/// \param dp The transport layer source port, if transport is UDP or TCP (0xFFFFUL if any).
		/// \param s Sense of the interface that received the packet (Rule::ANY_SENSE if any).
		/// \param noa The action to be taken in case of a matching frame from an unauthorized client.
		/// \param aa The action to be taken in case of a matching frame from an authorized client.
		///
		void set (
			EthernetFrame::ProtocolType e,
			IPv4Address sip,
			IPv4Address dip,
			IPPacket::ProtocolType i,
			uint16_t sp,
			uint16_t dp,
			SenseType s,
			ActionType noa,
			ActionType aa);
		///
		/// \brief Check if the packet matches the rule.
		///
		/// \param e The ethernet frame to be matched.
		/// \param s The direction the packet is traveling.
		///
		/// \return True if and only if the packet matches.
		///
		bool getMatch (EthernetFrame &e, SenseType s) const;
		///
		/// Return the action according to the authorization of the client.
		///
		/// \param auth Authorization of the client.
		///
		inline ActionType getAction (bool auth) const;
	};

	inline Rule::ActionType Rule::getAction (bool auth) const
	{
		return auth ? authAction : noauthAction;
	}

	////////////////////////////////////////////////////////////////////////////
	///
	/// \brief Read-only interface to the rule table.
	///
	/// The gateway packet handler doesn't need to add rules, just to ask the
	/// table what action to take for a packet. So we provide it with a limited interface.
	///
	class RuleTableInterface {
	public:
		///
		/// \brief Virtual destructor.
		///
		virtual ~RuleTableInterface (void) {}
		///
		/// \brief Return what action should be taken on a packet.
		///
		/// Scans the rule table until it finds a matching rule or the end of the table.
		///
		/// \param e The Ethernet frame.
		/// \param s The sense of travel.
		/// \param auth Authorization of the client.
		///
		/// \return The action (default action if no match is found).
		///
		virtual Rule::ActionType getAction (EthernetFrame &e, Rule::SenseType s, bool auth) const = 0;
	};

	////////////////////////////////////////////////////////////////////////////
	///
	/// \brief Firewall rule table
	///
	/// Basically, an array of rules, with a default behavior for packets that
	/// do not match any rule.
	///
	/// The rule table only concerns packets from unauthorized clients.
	/// Traffic from authorized clients should pass the inspection of a firewall,
	/// which this program is not intended to replace (at the moment).
	///
	class RuleTable: public RuleTableInterface {
	private:
		///
		/// \brief Maximum number of rules.
		///
		/// The rule table is statically allocated, just as everything else in this program.
		///
		static const int MAXRULES = 256;
		///
		/// \brief Default behavior for unauthorized packets if no rule matches the frame.
		///
		const Rule::ActionType noauthDefaultAction;
		///
		/// \brief Default behavior for unauthorized packets if no rule matches the frame.
		///
		const Rule::ActionType authDefaultAction;
		///
		/// \brief Rule table.
		///
		Rule table [MAXRULES];
		///
		/// \brief Number of rules currently stored in the table.
		///
		int nrules;
	public:
		///
		/// \brief Void constructor.
		///
		/// \param noauth Default action for unauthorized packets.
		/// \param auth Default action for authorized packets.
		///
		inline RuleTable (Rule::ActionType noauth, Rule::ActionType auth);
		///
		/// \brief Add a rule to the table.
		///
		/// This is just an interface to the Rule::set function, and parameters are the same.
		///
		inline void newRule (
			EthernetFrame::ProtocolType e,
			IPv4Address sip,
			IPv4Address dip,
			IPPacket::ProtocolType i,
			uint16_t sp,
			uint16_t dp,
			Rule::SenseType s,
			Rule::ActionType noa,
			Rule::ActionType aa);
		//
		// Inherited from RuleTableInterface.
		//
		Rule::ActionType getAction (EthernetFrame &e, Rule::SenseType s, bool auth) const;
	};

	inline RuleTable::RuleTable (Rule::ActionType noauth, Rule::ActionType auth) :
		noauthDefaultAction (noauth),
		authDefaultAction (auth),
		nrules (0)
	{
	}

	inline void RuleTable::newRule (
			EthernetFrame::ProtocolType e,
			IPv4Address sip,
			IPv4Address dip,
			IPPacket::ProtocolType i,
			uint16_t sp,
			uint16_t dp,
			Rule::SenseType s,
			Rule::ActionType noa,
			Rule::ActionType aa)
	{
		table[nrules++].set (e, sip, dip, i, sp, dp, s, noa, aa);
	}

} // namespace Gateway

#endif
