/// \file tapinterface.h
/// \author Mauro Brunato
///
/// \brief Packet capture classes.
///
/// The only explicit references to libpcap functions and data types are
/// collected within the layer2.cpp file in order to ensure compatibility
/// in Cygwin systems (no mixture with libc functions allowed).
///
/// $Id: tapinterface.h,v 1.1 2005/12/13 17:18:53 cvs Exp $

/*##############################################################################
   
   Developed by NetMob Group, University of Trento, Italy

   University of Trento
   DIT - Department of Informatics and Telecommunications 
   Computer Networks and Mobility Research Program
   http://netmob.unitn.it/ 
   
   This file is part of the WilmaGate system, created as part
   of the WILMA Project, sponsored by the Autonomous
   Province of Trento and participated by ITC-irst, the University
   of Trento and Alpikom Spa. More info at

     http://www.wilmaproject.org/		

   To download the project:
      - in tarball format
        http://netmob.unitn.it/wilmagate.html
      - connect to CVS repository
        :pserver:cvs@rtm.unitn.it:/home/cvsroot
   In both previous cases, the download is password protected. To obtain
   the password you must send a mail to 

      wilmagate@dit.unitn.it

################################################################################

   The WilmaGate Licence of Use


   Copyright (c) 2003-2005
                - Universita` di Trento,
                  Dipartimento di Informatica e Telecomunicazioni (DIT)
                  Computer Networks and Mobility Research Program


   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (hereinafter
   the "WilmaGate") directly from the copyright owner, to use the WilmaGate
   without restriction of purpose or time, including without limitation the
   rights to use, copy, modify, and merge the WilmaGate, subject to the
   following conditions:

   The above copyright notice and this permission notice shall be included
   in all copies or substantial portions of the WilmaGate. 

   The WilmaGate shall not be sub-licenced either commercially or free of
   charge, as is or in part or as a part of any other package. 

   The licensee agrees to properly credit WilmaGate and its copyright 
   owners in any technical, scientific or divulgative publication, 
   whatever publishing means are used, that is based entirely or in part 
   on WilmaGate use, as well as in any booklet, web-site or brochure that 
   advertise services based on WilmaGate use. 

   If the WilmaGate is modified, the copyright owner shall be notified.

   THE WilmaGate IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
   OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
   IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
   CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
   TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
   SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 

##############################################################################*/

#ifndef _H_TAPIF_
#define _H_TAPIF_

#include <inttypes.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/ioctl.h>
#include <netinet/in.h>
#include <linux/if.h>
#include <linux/if_tun.h>

#include "layer2capture.h"
#include "packets.h"

using namespace Common;

// commented in netcard.h
namespace Gateway {

	//////////////////////////////////////////////////////////////////////////
	///
	/// \brief Base class for tun/tap captures
	///
	class TapInterface {

	private:
		///
		/// \brief Name of the device (tap0 or similar)
		///
		string deviceName;
		///
		/// \brief File descriptor (handle)
		//
		int descriptor;
		///
		/// \brief Receiving thread handle.
		///
		pthread_t thread;
		///
		/// \brief Pointer to the registered listener object.
		///
		Layer2Listener *listener;
		//
		// See socket.cpp for details.
		//
		friend void *TapRecvThread (void *tap);
	public:
		///
		/// \brief Constructor
		///
		/// \param ifname Name of the tap interface.
		///
		TapInterface (const char *) throw (CaptureException);
		///
		/// \brief Destructor
		///
		~TapInterface (void) throw (CaptureException);
		///
		/// \brief Name of the TAP device.
		///
		const string &getDeviceName (void) const { return deviceName; }
		///
		/// \brief Send a frame through the interface
		///
		/// \param e Pointer to the Ethernet frame
		/// \param l Frame size	
		///
		void sendFrame (const EthernetFrame *e, int l) throw (CaptureException);
		///
		/// \brief Synchronously receive a frame
		///
		/// \param e Pointer to buffer to be filled by an Ethernet frame
		/// \param l Size of the buffer
		///
		/// \return Size of the received frame
		///
		int recvFrame (EthernetFrame *e, int l);
		///
		/// \brief Listener registration.
		///
		/// Called by the listener object to register itself to the interface.
		/// From this moment on, if the receiving thread is started,
		/// all received packets will cause a call to the Gateway::Layer2Listener::ReceivedPacket method
		/// of the registered object.
		///
		/// Since a listener can register itself to many interfaces, it will also
		/// pass an arbitrary ID value which shall be used in all subsequent callback communications
		/// by the interface.
		///
		/// \param l pointer to the listener object.
		///
		void registerListener (Layer2Listener *l);
		///
		/// \brief Wait for completion of the receiving thread.
		///
		inline void wait (void);
		///
		/// \brief Start the receiving thread.
		///
		/// All received packets shall be sent to the registered listener via its
		/// Gateway::Layer2Listener::receivedPacket method.
		///
		void startReceiving (void) throw (CaptureException);
		///
		/// \brief Stop the receiving thread.
		///
		void stopReceiving (void) throw (CaptureException);
		///
		/// \brief Read one packet in nonblocking mode.
		///
		/// The read packet is routed to the Layer2Listener::capturedPacket
		/// method of the passed object.
		///
		/// \param l The listening object.
		///
		void getNext (Layer2Listener *l);
		///
		/// \brief Report whether interface is open.
		///
		/// \return true if interface is correctly open.
		///
		inline bool ok (void) const;
	};

	inline void TapInterface::wait (void)
	{
		pthread_join (thread, NULL);
	}

	inline bool TapInterface::ok (void) const
	{
		return descriptor >= 0;
	}

} // namespace Gateway

#endif
