/*
 * \file unify_mod.c
 * 
 * \author Edoardo Bello e Flavio Pompermaier
 * 
 * \brief Implementation of Unify authentication module of SER,
 * 		  a free SIP server
 * 
 * $Id: unify_mod.c,v 1.2 2006/11/13 9:00:00 cvs Exp $ 
 *
 * ######################################################################
 *  
 *  Developed by NetMob Group, University of Trento, Italy
 *
 * University of Trento
 * DIT - Department of Informatics and Telecommunications 
 * Computer Networks and Mobility Research Program
 * http://netmob.unitn.it/ 
 * 
 * This file is part of the WilmaGate system, created as part
 * of the WILMA Project, sponsored by the Autonomous
 * Province of Trento and participated by ITC-irst, the University
 * of Trento and Alpikom Spa. More info at
 *
 *   http://www.wilmaproject.org/		
 *
 * To download the project:
 *    - in tarball format
 *      http://netmob.unitn.it/wilmagate.html
 *    - connect to CVS repository
 *      :pserver:cvs@rtm.unitn.it:/home/cvsroot
 * In both previous cases, the download is password protected. To obtain
 * the password you must send a mail to 
 *
 *    wilmagate@dit.unitn.it
 *
 * ######################################################################
 *
 * The WilmaGate Licence of Use
 *
 *
 * Copyright (c) 2003-2005
 *              - Universita` di Trento,
 *                Dipartimento di Informatica e Telecomunicazioni (DIT)
 *                Computer Networks and Mobility Research Program
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (hereinafter
 * the "WilmaGate") directly from the copyright owner, to use the WilmaGate
 * without restriction of purpose or time, including without limitation the
 * rights to use, copy, modify, and merge the WilmaGate, subject to the
 * following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the WilmaGate. 
 *
 * The WilmaGate shall not be sub-licenced either commercially or free of
 * charge, as is or in part or as a part of any other package. 
 * 
 * The licensee agrees to properly credit WilmaGate and its copyright 
 * owners in any technical, scientific or divulgative publication, 
 * whatever publishing means are used, that is based entirely or in part 
 * on WilmaGate use, as well as in any booklet, web-site or brochure that 
 * advertise services based on WilmaGate use. 
 *
 * If the WilmaGate is modified, the copyright owner shall be notified.
 *
 * THE WilmaGate IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
 * CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 
 *
 * ######################################################################
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>

#include "../../sr_module.h"
#include "../../dprint.h"
#include "../../error.h"
#include "../../ut.h"
#include "../../mem/mem.h"
#include "../../parser/hf.h"
#include "../../parser/digest/digest.h"
#include "../../ip_addr.h"

#include "unify_macro.h"
#include "net_funcs.h"
#include "shm_funcs.h"
#include "sem_funcs.h" 

MODULE_VERSION

/* Define host parameters with default values */
static char *unify_auth_host = "localhost", 
	    *unify_report_host = "localhost";
/* Define port parameters  with default values */
static int unify_auth_port = 54273, 
	   unify_report_port = 54272; 
/* Shared memory  pointer */
unify_shm_t *unify_shm;

/* Define SER module management function */
static int mod_init(void);
static void destroy(void);

/* Define process_unify_auth as core function of the module  */
static int process_unify_auth(struct sip_msg* msg,char* str,char* str2);
/* Define utility function */
static int send_auth(int scktfd, char* s_addr, str *user, str *realm, char *token);
static int revoke_auth(int scktfd, char* s_addr, char *token);

int expires_comp(struct sip_msg* msg);

static cmd_export_t cmds[] = {
	{
	 "unify_authorize",
	 process_unify_auth,
	 0,
	 0,
	 REQUEST_ROUTE
	},
	{0, 0, 0, 0, 0}
};

static param_export_t params[] = {
	{"unify_auth_host", 	STR_PARAM, &unify_auth_host},
	{"unify_report_host",	STR_PARAM, &unify_report_host},
	{"unify_auth_port",		INT_PARAM, &unify_auth_port},
	{"unify_report_port",	INT_PARAM, &unify_report_port},
	{0, 0, 0}
};

struct module_exports exports = {
	"unify",
	cmds,       /* exported functions */
	params,     /* exported params */
	mod_init,   /* initialization module */
	0,			/* response function */
	destroy,    /* destroy function */
	0,			/* oncancel function */
	0			/* per-child init function */
};


/* Module initialization stuff: acquire necessary resources and check parameter value correctness */
static int mod_init(void)
{
	LOG(L_NOTICE, "Unify module - initializing\n");
	
	unify_shm = (unify_shm_t *)shmcrt(SHM_NAME, sizeof(unify_shm_t), 0666);
	
	if(!unify_shm && errno == EEXIST){
		LOG(L_WARN, "WARNING unify mod_init():"
			" cannot create shared memory segment because already exist!\n");
		unify_shm = (unify_shm_t *)getshm(SHM_NAME, 0666);
		semrmv(unify_shm->semid);
		if(shmrmv(SHM_NAME) < 0){
			LOG(L_ERR, "ERROR unify mod_init(): cannot remove shared memory segment '%s' (reason: '%s')\n",
				SHM_NAME, strerror(errno));
			return E_UNSPEC;
		}
		unify_shm = (unify_shm_t *)shmcrt(SHM_NAME, sizeof(unify_shm_t), 0666);
	}
	
	if(!unify_shm){
		LOG(L_ERR, "ERROR unify mod_init():"
			" cannot create shared memory segment (reason(%d) = %s)\n", errno, strerror(errno));
		return E_UNSPEC;
	}
		
	unify_shm->semid = semcrt(MAX_SEM_NUM);
	if(unify_shm->semid < 0){
		LOG(L_ERR, "ERROR unify mod_init(): cannot create semaphore set (reason = %s)\n", strerror(errno));
		return E_UNSPEC;
	}
	
	if(!gethostbyname(unify_auth_host)){
		LOG(L_ERR, "ERROR unify mod_init(): DNS failure in lookup of auth_host <%s>\n",
			unify_auth_host);
		return E_UNSPEC;
    }
    
    if(unify_auth_port < 0 || unify_auth_port > 65535){
		LOG(L_ERR, "ERROR unify mod_init(): auth_port <%d> [min: 0, max: 65535]\n",
			unify_auth_port);
		return E_UNSPEC;
	}
    
    if(!gethostbyname(unify_report_host)){
		LOG(L_ERR, "ERROR unify mod_init(): DNS failure in lookup of report_host <%s>\n",
			unify_report_host);
		return E_UNSPEC;
    }
	
	if(unify_report_port < 0 || unify_report_port > 65535){
		LOG(L_ERR, "ERROR unify mod_init(): auth_port <%d> [min: 0, max: 65535]\n",
			unify_report_port);
		return E_UNSPEC;
	}
	
	return 0;
}

/* Module exiting stuff: release resources */
static void destroy(void){
	LOG(L_NOTICE, "Unify module - destroy\n");
	if(!unify_shm)
		return;
	if(unify_shm->semid > 0 && semrmv(unify_shm->semid) < 0){
		LOG(L_ERR, "ERROR unify destroy(): cannot remove semaphore set %d (reason: '%s')\n",
			unify_shm->semid, strerror(errno));
	}
	if(shmrmv(SHM_NAME) < 0){
		LOG(L_ERR, "ERROR unify destroy(): cannot remove shared memory segment '%s' (reason: '%s')\n",
			SHM_NAME, strerror(errno));
	}
}

/*
 * This function is the core of the module. It takes as input the sip REGISTER message and
 * other two default parameters, extract useful information from it and connect with Authenticator and Provider in order
 * to inform Unify that a User has correctly authenticate, registered/deregister and so he/she can make SIP calls within 
 * Unify
 * */
static int process_unify_auth(struct sip_msg* msg, char* str1, char* str2){

	auth_body_t *cred;		/* header field containing parsed authorized digest credentials */
	unify_user_t *unify_user;	/* entry of the shared memory */
	struct hdr_field *h;            /* header field variable */
	struct ip_addr *ip;		/* client IPv4 or IPv6 address */
	int i,
		exp_val,		/* expire time value */
		scktfd,         	/* socket file descriptor */
		ll,			/* line lenght */
		round,			/* counter variable */
		exit_val;		/* exit value */
	struct in_addr in;		/* IPv4 address */
	char *s_addr,			/* source address as string */
		 *p,
		 buf[BUFFER_LENGTH];	/* buffer for reading/writing */
		 
	exit_val = 0;			/* Default exit value: 0 - REGISTER message correctly processed */
	
	/* Get a pointer to Authorization Header field and assign them to the variable h */
	get_authorized_cred(msg->authorization, &h);
	if(!h) /* if Authorization Header field not present search for Proxy-Authorization */
		get_authorized_cred(msg->proxy_auth, &h);
	if(!h) /* if also this Header field not present then no authorization credentials to process, so exit */
		return 0;
	/* if authorization credentials present assign them to variable cred */
	cred = (auth_body_t *)(h->parsed);
	
	/* Verify if the crediantials are present or not */
	if(!cred || !cred->digest.username.user.len)
		return 0;
	/* Get the IP client from the message */
	ip = &msg->rcv.src_ip;
				
	/* Log username and realm of the authorization credentials */
	LOG(L_DBG, "DEBUG unify process_unify_auth(): %s\n", cred->digest.username.user.s);
	LOG(L_DBG, "DEBUG unify process_unify_auth(): %s\n", cred->digest.realm.s);

	/* Get the pointer to the shared memory segment for the specific client IP. Notice that here we suppose
	 * that:
	 *	- all networks are of class C (255 users max)
	 *	- user 172.31.194.5 has semaphore 5 to access at shared memory
	 * */
	unify_user = &unify_shm->users[ip->u.addr[3]];
	/* Get the expires time from the message */	
	exp_val = expires_comp(msg);
	if (exp_val < 0){
		if(exp_val == -1) /* fetch bindings */
			return 1;
		return exp_val; /* error */
	}
	
	/* Log information about Registration (exp_val>0) or Deregistration (exp_val=0) */
	LOG(L_NOTICE, "NOTICE unify process_unify_auth(): %s of %d.%d.%d.%d\n",
		(!exp_val ? "De-registration" : "Registration"),
		ip->u.addr[0], ip->u.addr[1], ip->u.addr[2], ip->u.addr[3]);
	/*
	 * Convert client IP from the binary data to the number-and-dots notation because
	 * Authenticator and Reporter used a ASCII-based  protocol
	 */	
	in.s_addr = *(uint32_t*)(ip->u.addr);
	s_addr = (char *)inet_ntoa(in);
	/* Mutex lock: Enter in the critical session  */
	mtxlock(unify_shm->semid, (ip->u.addr[3]%MAX_SEM_NUM));
	/* Registration: (expires > 0); Request token only if it is obsolete */
	if(exp_val){
		/* now - last > renew + renew/2	 : obsolete token (considering delay-propagation error) */
		if(time(0) - unify_user->last > unify_user->renew + (unify_user->renew/2)){
			
			LOG(L_DBG, "DEBUG unify process_unify_auth(): try to get providers list!\n");
			
			/* Create a TCP connection with the reporter  */
			scktfd = init_tcp_conn(unify_report_host, unify_report_port);
			if(scktfd < 0)
				goto exit;
			/* Reporter request ex: providers 172.31.194.153\n  */
			/* Multiple writen for semplicity rason...of course could be collapse in one */
			if(writen(scktfd, "providers ", 10)){
				exit_val |= CLOSE_SOCK;
				goto exit;
			}
			if(writen(scktfd, s_addr, strlen(s_addr))){
				exit_val |= CLOSE_SOCK;
				goto exit;
			}
			if(writen(scktfd, "\n", 1)){
				exit_val |= CLOSE_SOCK;
				goto exit;
			}
			/* Read response section */
			round = 0; /* count how many lines (while cicles) are present */
			while(1){
				readline(scktfd, buf, BUFFER_LENGTH, &ll);
				if(ll <= 0){/* Input/output error */
					exit_val |= CLOSE_SOCK;
					goto exit;
				}else if(ll == 1)
					break; /* server finished */
				else{/* Log all lines of the response coming from the Reporter */
					LOG(L_DBG, "DEBUG unify process_unify_auth(): REP line_%d[%d] %s\n", round, ll, buf);
					if(!(round++)){/* extract information only form the first line (token) */						
						/* Copy token value into user token info */
						for (i = 0; i < ll; ++i) {
							unify_user->token[i] = buf[i];
						}
					}
				}
			}
			/* Close connection with Reporter */
			if(writen(scktfd, "exit\n", 5)){
				exit_val |= CLOSE_SOCK;
				goto exit;
			}
			close_tcp_conn(scktfd);
		}
	}
	LOG(L_NOTICE, "NOTICE unify process_unify_auth(): ip %s token %s\n", s_addr, unify_user->token);
	/* De-Registration and De-Authentication on Unify (expires time = 0) */
	if(!exp_val){
		/* if (de)registration arrives too late */
		if(time(0) - unify_user->last > unify_user->renew){
			LOG(L_NOTICE, "NOTICE unify process_unify_auth(): de-registration of %s isn't necessary!\n", s_addr);
			exit_val |= EXIT_OK;
			goto exit;
		}
		/* init tcp connection to the Authenticator */
		scktfd = init_tcp_conn(unify_auth_host, unify_auth_port);
		if(scktfd < 0)
			goto exit;		
		/* De-authenticate user on Unify using socket */
		if(!revoke_auth(scktfd, s_addr, unify_user->token)){
			LOG(L_ERR, "ERROR unify process_unify_auth(): revoke authentication to %s, token %s\n",
				s_addr, unify_user->token);
			exit_val |= CLOSE_SOCK;
			goto exit;
		}
	/* Authentication on Unify */
	}else{
		/* Renewal of registration too fast */
		if(time(0) - unify_user->last < unify_user->renew/2){
			LOG(L_NOTICE, "NOTICE unify process_unify_auth(): renew registration of %s isn't necessary!\n", s_addr);
			exit_val |= EXIT_OK;
			goto exit;
		}
		/* init tcp connection to the Authenticator */
		scktfd = init_tcp_conn(unify_auth_host, unify_auth_port);
		if(scktfd < 0)
			goto exit;
		/* Authenticate on Unify using socket (similar to Telnet) */
		if(!send_auth(scktfd, s_addr, &cred->digest.username.user, &cred->digest.realm, unify_user->token)){
			LOG(L_ERR, "ERROR unify process_unify_auth(): send authentication for %s, token %s)\n",
				s_addr, unify_user->token);
			exit_val |= CLOSE_SOCK;
			goto exit;
		}
	}
	
	round = 0;
	while(1){
		readline(scktfd, buf, BUFFER_LENGTH, &ll);
		if(ll <= 0){ /* Input/output error */
			exit_val |= CLOSE_SOCK;
			goto exit;
		}
		else if(ll == 1)
			break; /* server finished */
		else{
			LOG(L_DBG, "DEBUG unify process_unify_auth(): AUTH line(%d) %s\n", round, buf);
			if(!(round++)){
				if(!strncmp(buf, "ok", 2)){
					exit_val |= EXIT_OK;
					if(exp_val == 0){
						memset((void *)unify_user, 0, sizeof(unify_user_t));
						LOG(L_DBG, "DEBUG unify process_unify_auth(): OK break\n");										
						break;
					}
					for(i = 0, p = buf+3; *p != '\t'; ++i, ++p)
						unify_user->token[i] = *p;
					unify_user->token[i] = '\0';
					++p;
					unify_user->renew = atol(p);
					unify_user->last = time(0);
				}else{
					LOG(L_ERR, "ERROR unify process_unify_auth(): get NO from Authenticator for ip %s token %s\n", s_addr, unify_user->token);
					memset((void *)unify_user, 0, sizeof(unify_user_t));
					break;
				}
			}
		}
	}
	
	if(writen(scktfd, "exit\n", 5)){
		exit_val |= CLOSE_SOCK;
		goto exit;
	}

	LOG(L_NOTICE, "NOTICE unify process_unify_auth(): ip %s token %s renew %d\n", s_addr,
			unify_user->token, (int)unify_user->renew);

	close_tcp_conn(scktfd);
exit:	
	if(exit_val&CLOSE_SOCK)	// if exit_val = CLOSE_SOCK
		close_tcp_conn(scktfd);  // then close the socket
	
	/* Mutex unlock: release access to shared memory */
	mtxunlock(unify_shm->semid, (ip->u.addr[3] % MAX_SEM_NUM));
	

	if(exit_val&EXIT_OK)
		return 1;
	return 0;
}

/* Tell Authenticator to authenticate a certain user (whose IP=s_addr) */
static int send_auth(int scktfd, char* s_addr, str *user, str *realm, char *token){
	
	if(writen(scktfd, "authenticate_sip\t", 17))
		return 0;
	if(writen(scktfd, s_addr, strlen(s_addr)))
		return 0;
	if(writen(scktfd, "\t", 1))
		return 0;
	if(writen(scktfd, user->s, user->len))
		return 0;
	if(writen(scktfd, "\t", 1))
		return 0;
	if(writen(scktfd, user->s, user->len))
		return 0;
	if(writen(scktfd, "@", 1))
		return 0;
	if(writen(scktfd, realm->s, realm->len))
		return 0;
	if(writen(scktfd, "\t", 1))
		return 0;
	if(writen(scktfd, token, strlen(token)))
		return 0;
	if(writen(scktfd, "\t0\n", 3))
		return 0;
	LOG(L_NOTICE, "DEBUG unify authenticate_sip: IP %s, token %s\n", s_addr, token);
	return 1;
}

/* Tell Authenticator to deauthenticate a certain user (whose IP=s_addr) */
static int revoke_auth(int scktfd, char* s_addr, char *token){
	
	if(writen(scktfd, "revoke_sip\t", 11))
		return 0;
	if(writen(scktfd, s_addr, strlen(s_addr)))
		return 0;
	if(writen(scktfd, "\t", 1))
		return 0;
	if(writen(scktfd, token, strlen(token)))
		return 0;
	if(writen(scktfd, "\n", 1))
		return 0;
	LOG(L_NOTICE, "DEBUG unify revoke_sip: IP %s, token %s\n", s_addr, token);
	return 1;
}

/* This function extract the expires value from the Sip message. Notice that the value of Expire
 * can be specified either in an apposite Header field or either as a Via parameter, but it can also be omitted.. */
int expires_comp(struct sip_msg* msg)
{
	int x, err;
	contact_body_t *contact;
	str *body,
		foo;

	body = 0;
	err = 0;
	/* lookup into the message for EXPIRES header*/
	if ( !msg->expires ) {
		
		if  ( parse_headers( msg , HDR_EXPIRES, 0 ) == -1 ){
			LOG( L_ERR , "ERROR unify expires_comp():"
				" parsing EXPIRES header failed!\n");
			return -2;
		}
		/*
		 * If Expires header not found lookup into the
		 * message for CONTACT header
		 */
		if (!msg->expires) {
			
			LOG(L_NOTICE, "L_DBG unify expires_comp(): EXPIRES header not found!\n");
			if(!msg->contact){
				/*  */
				if( parse_headers( msg , HDR_CONTACT, 0 ) == -1 ){
					LOG( L_ERR , "ERROR unify expires_comp():"
						" parsing CONTACT header failed!\n");
					return -2;
				}
			}
			
			if(!msg->contact){
				LOG(L_NOTICE, "NOTICE unify expires_comp(): CONTACT header not found (fetch bindings)!\n");
				return -1;
			}
			
			/*
			 * Parse contact header and get the expires value
			 */
			if(!parse_contact(msg->contact)){
				contact = (contact_body_t *)msg->contact->parsed;				
				//LOG(L_NOTICE, "L_DBG unify expires_comp(): &contact->contacts->expires=%x \n", &contact->contacts->expires);
				if(contact->contacts->expires)
					body = &contact->contacts->expires->body;
			}
			/*else{
				LOG( L_ERR , "ERROR unify expires_comp():"
				" parsing CONTACT header failed!\n");
				return err;
			}*/
		}
		else
			body = &msg->expires->body;
	}
	else
		body = &msg->expires->body;
	
	if(!body){
		LOG(L_NOTICE, "NOTICE unify expires_comp(): "
				"user relies on default expires time!\n");
		return 1;
	}
	
	/* if header is present, trim to get only the string containing numbers */
	trim_len(foo.len , foo.s , *body);		
	/* converts from string to number */
	x = str2s( foo.s,foo.len,&err);
	if (err){
		LOG(L_ERR, "ERROR unify expires_comp(): "
			" unable to parse the EXPIRES number !\n");
		return -2;
	}
	LOG(L_DBG, "DEBUG unify expires_comp() value = %d \n",x);
	return x;
}
